unit oDBConnection;

interface

uses
  Dialogs, Classes, TRPCB, SysUtils, IMDK_DBConnection, OInterfaces;

type
  TDBConnection = class(TCustomInterfacedObject, I_RPCExecutor)
  private
    FRPCBroker: TRPCBroker;
    FRPCBrokerResults: TStringList;
    FRPCLog: TStringList;
    procedure DecodeResults;
  public
    property ExecResults: TStringList read FRPCBrokerResults;
    constructor Create;
    destructor Destroy; override;
    function IsConnected: Boolean;
    function Connect(AParams: array of string): Boolean;
    function ExecRemoteProc(ARPCName: string; AParams: array of string; ABigString: string): Boolean; overload;
    function ExecRemoteProc(ARPCName: string; AParams: array of string; AMultList: TStringList): Boolean; overload;
    function GetExecResults: TStringList;
  end;

function NewDBConnection(ARPCBroker: TRPCBroker): TDBConnection;

implementation

uses
  IdCoderUUE,
{$IFDEF RPCLOG}
  UZZ_RPCEvent, UZZ_DescribedItem, FZZ_EventLog;
{$ELSE}
  UMDK_RPCEvent;
{$ENDIF}

const
  UUE_HEADER    = '-- Begin UUEncoded Data --';
  UUE_FOOTER    = '--- End UUEncoded Data ---';
  MAX_RPC_LINES = 100;

function NewDBConnection(ARPCBroker: TRPCBroker): TDBConnection;
begin
  Result := TDBConnection.Create;
  if Assigned(ARPCBroker) then Result.FRPCBroker := ARPCBroker
  else Result.FRPCBroker := TRPCBroker.Create(nil);
end;
{ TDBConnection }

constructor TDBConnection.Create;
begin
  inherited;
  FRPCBroker := nil;
  FRPCBrokerResults := TStringList.Create;
  FRPCLog := TStringList.Create;
end;

destructor TDBConnection.Destroy;
begin
  FRPCBroker := nil;
  FreeAndNil(FRPCBrokerResults);
  while FRPCLog.Count > 0 do begin
    try
      if FRPCLog.Objects[0] <> nil then TRPCEventItem(FRPCLog.Objects[0]).Free;
      FRPCLog.Delete(0);
    except
    end;
  end;
  FreeAndNil(FRPCLog);
  inherited;
end;

function TDBConnection.Connect(AParams: array of string): Boolean;
begin
  try
    FRPCBroker.Server := AParams[0];
    FRPCBroker.ListenerPort := StrToInt(AParams[1]);
    if Length(AParams) > 2 then FRPCBroker.AccessVerifyCodes := AParams[2];
    FRPCBroker.Connected := True;
    Result := FRPCBroker.Connected;
  except
    Result := False;
  end;
end;

function TDBConnection.IsConnected: Boolean;
begin
  Result := FRPCBroker.Connected;
end;

(* -- 2FA ----------------------------------------------------------------------
procedure TDBConnection.DecodeResults;
{ Scan the fRPCBrokerResults for and encoded data and set it back to the way it was sent }
var
  I: Integer;
{$IFDEF D10}
  S, Ss: string;
{$ELSE}
  AStringStream: TStringStream;
{$ENDIF}
begin
  // if the data is encoded both header and footer should be present
  // zzzzzzandria 060721
  if (Pos(UUE_HEADER, FRPCBrokerResults.Text) = 0) or (Pos(UUE_FOOTER, FRPCBrokerResults.Text) = 0) then Exit;
  for I := FRPCBrokerResults.Count - 1 downto 0 do
    if FRPCBrokerResults[I] = UUE_HEADER then begin
{$IFDEF D10}
      S := '';
{$ELSE}
      AStringStream := TStringStream.Create('');
{$ENDIF}
      with TIdDecoderUUE.Create(nil) do
        try
{$IFDEF D10}
          Ss := '';
{$ELSE}
          AStringStream.Position := 0;
{$ENDIF}
          while FRPCBrokerResults[I + 1] <> UUE_FOOTER do begin
{$IFDEF D10}
            Ss := FRPCBrokerResults[I + 1];
            Ss := DecodeString(Ss);
            S := S + Ss;
{$ELSE}
            DecodeToStream(FRPCBrokerResults[I + 1], AStringStream);
{$ENDIF}
            FRPCBrokerResults.Delete(I + 1);
          end;
          FRPCBrokerResults.Delete(I + 1);
{$IFDEF D10}
          FRPCBrokerResults[I] := S;
{$ELSE}
          AStringStream.Seek(0, SoFromBeginning);
          FRPCBrokerResults[I] := AStringStream.DataString;
{$ENDIF}
          { Replace header with original text }
        finally
{$IFNDEF D10}
          FreeAndNil(AStringStream);
{$ENDIF}
          Free;
        end;
    end;
end;
------------------------------------------------------------------------- 2FA *)
procedure TDBConnection.DecodeResults;
{Scan the fRPCBrokerResults for and encoded data and set it back to the way it was sent}
var
  i: integer;
  s,ss: String;
begin
  // if the data is encoded both header and footer should be present
  //                                             zzzzzzandria 060721
  if (pos(UUE_HEADER, fRPCBrokerResults.Text) = 0)
    or (pos(UUE_FOOTER, fRPCBrokerResults.Text) = 0) then
    Exit;

  for i := fRPCBrokerResults.Count - 1 downto 0 do
    if fRPCBrokerResults[i] = UUE_HEADER then
      begin
        s := '';
        with TIdDecoderUUE.Create(nil) do
          try
            ss := '';
            while fRPCBrokerResults[i + 1] <> UUE_FOOTER do
              begin
                ss := fRPCBrokerResults[i + 1];
                ss := DecodeString(ss);
                s := s + ss;
                fRPCBrokerResults.Delete(i + 1);
              end;
            fRPCBrokerResults.Delete(i + 1);
            fRPCBrokerResults[i] := s;
            {Replace header with original text}
          finally
            Free;
          end;
      end;
end;

function TDBConnection.ExecRemoteProc(ARPCName: string; AParams: array of string; AMultList: TStringList): Boolean;
var
  I, J: Integer;
{$IFDEF RPCLOG}
  AStart, AStop: TDateTime;
  AnEvent: TRPCEventItem;
{$ENDIF}
begin
  FRPCBroker.RemoteProcedure := ARPCName;
  { Load any params }
  I := 0;
  if SizeOf(AParams) > 0 then
    while I <= high(AParams) do begin
      if (Copy(AParams[I], 1, 1) = '@') and (Length(AParams[I]) > 1) then
      // Is it passed as a reference???
      begin
        FRPCBroker.Param[I].PType := Reference;
        FRPCBroker.Param[I].Value := Copy(AParams[I], 2, Length(AParams[I]));
      end else begin
        FRPCBroker.Param[I].PType := Literal;
        FRPCBroker.Param[I].Value := AParams[I];
      end;
      Inc(I);
    end;
  { Load up the multiple if it is assigned }
  if Assigned(AMultList) then begin
    FRPCBroker.Param[I].PType := List;
    if FRPCBroker.Param[I].Mult.Count > 0 then
        ShowMessage('Broker Param list was not cleaned.' + #13#10 + 'Possible connection failure...' + #13#10 +
        'Close the application and try again later');
    for J := 0 to AMultList.Count - 1 do FRPCBroker.Param[I].Mult[IntToStr(J)] := AMultList[J];
  end;
  try
{$IFDEF RPCLOG}
    AStart := Now;
    AnEvent := GetTRPCBEventItem(FRPCBroker);
{$ENDIF}
    FRPCBrokerResults.Clear;
    FRPCBroker.Results.Clear; // 061129
    FRPCBroker.LstCall(FRPCBrokerResults);
{$IFDEF RPCLOG}
    AStop := Now;
{$ENDIF}
    DecodeResults; { Checks for any UUEncode data coming back }
    Result := (Copy(FRPCBrokerResults.Text, 1, 2) <> '-1');
{$IFDEF RPCLOG}
    AnEvent.AppendResults(FRPCBrokerResults, AStart, AStop);
    AddRPCEvent(AnEvent);
{$ENDIF}
    { Check for error flag }
  except
    on E: EBrokerError do begin
      FRPCBrokerResults.Insert(0, '-1^' + E.Message);
      Result := False;
{$IFDEF RPCLOG}
      AnEvent.AppendResults(FRPCBrokerResults, AStart, AStop);
      AddRPCEvent(AnEvent);
{$ENDIF}
    end;
    else
      raise;
  end;
end;

function TDBConnection.ExecRemoteProc(ARPCName: string; AParams: array of string; ABigString: string): Boolean;
var
  AStrStream: TStringStream;
  AStrList: TStringList;
  AStrListPartial: TStringList; { This is used to send data down in chunks }
begin
  { The purpose here is to force a complex string into UUEncode before calling above }
  AStrStream := TStringStream.Create(ABigString);
  AStrList := TStringList.Create;
  // aStrListPartial := TStringList.Create;  // 061129 zzzzzzandria
  with TIdEncoderUUE.Create(nil) do
    try
      AStrList.Add(UUE_HEADER);
      AStrStream.Position := 0;
      while AStrStream.Position < AStrStream.Size do AStrList.Add(Encode(AStrStream, 45));
      AStrList.Add(UUE_FOOTER);
    finally
      FreeAndNil(AStrStream);
      Free;
    end;
  { Now we have a UUEncoded TStringList instead of aBigString }
  if AStrList.Count < MAX_RPC_LINES then { We can safely send this tiny list to M }
  begin
    Result := ExecRemoteProc(ARPCName, AParams, AStrList);
    FreeAndNil(AStrList);
  end
  else
    try { Create partial list object and do it in pieces }
      AStrListPartial := TStringList.Create;
      Result := True;
      while AStrList.Count > 0 do begin
        { Move MAX_RPC_LINES lines to smaller array }
        AStrListPartial.Clear;
        while (AStrListPartial.Count < MAX_RPC_LINES) and (AStrList.Count > 0) do begin
          AStrListPartial.Add(AStrList[0]);
          AStrList.Delete(0);
        end;
        { Call the RPC with the smaller list }
        Result := ExecRemoteProc(ARPCName, AParams, AStrListPartial);
      end;
      FreeAndNil(AStrListPartial);
    except
      FreeAndNil(AStrListPartial);
      Result := False;
    end;
end;

function TDBConnection.GetExecResults: TStringList;
begin
  Result := FRPCBrokerResults;
end;

end.
