{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 2/12/09 2:35p $
  *       Site Name:    Hines OIFO
  *       Date Created: 4/29/07
  *       Developers:   den.petit@va.gov
  *       Description:  Contains the application project file
  *       Notes:        Updated by
  *                     andrey.andriyevskiy@va.gov
  *
  ================================================================================
  *
}

unit oDBController_Renal;

interface

uses
  Dialogs, Classes, SysUtils, TRPCB, ODBController, Controls;

type
  TRenalVitalSign = (VtTemp, VtPulse, VtRespiration, VtBloodPressure, VtPain, VtWeight);

  TDBController_Renal = class(TDBController)
  private
    function IENS(AIEN: Int64): string; overload;
    function IENS(AIEN: string): string; overload;
  public
    ResultInfo: string;
    function GetActiveRenalPatients(var APatientList: string): Boolean;
    function GetAllStudiesByPatient(APatientID: Int64; var AStudyList: string): Boolean;
    function GetStudy(AStudyID: Integer; var AStudyInfo, AStudySummary, AStudyFlowsheet, AMedLog, ANoteList,
      AnEventLog: string): Boolean;
    function SetStudy(AStudyID: Integer; AStudyInfo, AStudySummary, AStudyFlowsheet, AMedLog, ANoteList,
      AnEventLog: string): Boolean;
    function GetStudyItem(AFile, ANode, AnItemID: string; var AValue: string): Boolean;
    function SetStudyItem(AFile, ANode, AnItemID: string; AValue: string; APatient: string = ''): Boolean;
    function GetInstrumentResultList(AStudyID: Integer; var AInstrumentResultList: string;
      AGetAll: Boolean = False): Boolean;
    function GetInstrumentResult(AResultID: Integer; var AInstrumentResult: string): Boolean;
    function GetBillingData(AStudyID: Integer; var ABillingData: string): Boolean;
    function SetBillingData(AStudyID: Integer; AICD01, AICD02, AICD03, AICD04, ACPT01: string;
      ASvcConn, AMST, AAOExposure, AIRExposure, AEnvContaminants, AHeadNeckCancer, ACombatVet, ACampLejeune: Integer;
      AProviderList: array of string): Boolean;
    function GetLockStatus(AStudyID: Integer): string;
    function LockStudy(AStudyID: Integer): Boolean;
    function UnlockStudy(AStudyID: Integer): Boolean;
    function SubmitStudy(AStudyID: Integer; AUserESig: string; ADocumentText: TStringList): Boolean;
    function GetPatientAccessPoints(APatientID: Int64; var AAccessPoints, AAccessHistory,
      AInfectionHistory: string): Boolean;
    function SetPatientAccessPoints(APatientID: Int64;
      AAccessPoints, AAccessHistory, AInfectionHistory: string): Boolean;
    function LockSetting(ASettingName: string): Boolean;
    function UnlockSetting(ASettingName: string): Boolean;
    function GetUserOption(AOptionName: string; var AOptionValue: string): Boolean;
    function SetUserOption(AOptionName, AOptionValue: string): Boolean;
    function GetUserOptionByDUZ(AOptionName, ADUZ: string; var AOptionValue: string): Boolean;
    function SetUserOptionByDUZ(AOptionName, ADUZ, AOptionValue: string): Boolean;
    function DeleteUserOption(AOptionName: string): Boolean;
    function GetSystemOption(AOptionName: string; var AOptionValue: string): Boolean;
    function SetSystemOption(AOptionName, AOptionValue: string): Boolean;
    function DeleteSystemOption(AOptionName: string): Boolean;
    function ValidateElectronicSignature(ASignature: string): Boolean;
    function LookupProvider(APartialName: string; var AReturnList: string): Boolean;
    function LookupFacility(APartialName: string; var AReturnList: string): Boolean;
    function LookupAdministrator(APartialName: string; var AReturnList: string): Boolean;
    function GetCCOWContext: string;
    { Old style RPC calls }// ====================================================
    procedure GetPatientInfo(ADFN: string; Results: TStringList);
    procedure GetPatientLabInfo(ADFN: string; Results: TStringList);
    procedure GetPatientAllergies(ADFN: string; Results: TStringList);
    procedure GetPatientDirectives(ADFN: string; Results: TStringList);
    procedure GetPatientWarnings(ADFN: string; Results: TStringList);
    procedure GetPatientVAccination(ADFN: string; Results: TStringList);
    function SetVitalsMeasurement(ADateTimeTaken: TDateTime; APatientDFN: string; AVitalsType: TRenalVitalSign;
      AHospitalLocation: string; AUserDUZ: string; AValue: string): Boolean;
    function SubmitNoteNow(AStudyID: Integer; AUserID: string; AUserESig: string;
      // The users E-Sig in clear text, it will be encrypted before sending
      ADocumentText: TStringList; ATitle: string): Boolean;
    function GetNoteList(AStudyID: Integer): Boolean;
    function GetNote(AStudyID: Integer; ANote: string): Boolean;
    function GetClinicProcedures(ADFN, AClinic: string; var AReturnList: string): Boolean;
    function GetClinicDiagnosis(ADFN, AClinic: string; var AReturnList: string): Boolean;
    function GetServiceConnections(ADFN, AClinic: string; var AReturnList: string): Boolean;
    function GetLexiconData(ATarget, AnOption, StudyID: string; var AReturnList: string): Boolean;
    function SetFMDate(ImpDate: TDateTime): string; // 09/20/2012 zzzzzzcarnej
    function GetEncounterData(AStudy: string; var AReturnList: string): Boolean;
    function SetEncounterData(AStudy: string; ABillingData: TStringList; var AReturnList: string): Boolean;
    function GetLookupEntries(AFile, ATarget: string): TStringList;
    function GetLocation(AStudy: string): string;
    function SetLocation(AStudy, ALocation: string): string;
    function ValidatePCEData(AStudy, AData: string): string;
    function GetUserNameByIEN(AnIEN: string): string;
    function GetUserNameByIEN2(AnIEN: string): string;
    function GetQualifiersByVitalType(ATypeIEN: string): string;
    function GetQualifiersAll: string;
    function GetVitalTypesAll: string;
    function GetImplementationDate: string;
    function GetGatewayStatus: string;
    function GetGatewayStatusInfo: string;
    function GetGatewayRunningStatus: string;
    function GetServerCurrentDateTime: string;
    function GetRPCResults(AParamList, ADelim: string): string;
    function GetStudyStatus(AStudy: string): string;
    function RegisterAccessToSensitiveData(AnID, AnExeNameAndVersion: string): string;
    function IsDataSensitive(APatientID: string; var SSecurityWarning: string): Boolean;
    function GetSensitiveRecordAccessInfo(ADFN: string): string;
    function GetSensitiveRecordBulletinInfo(ADFN: string): string;
    function GetOptionListXML(AUserDUZ: string = '.5'): string;
    function SetStudyStatus(AStudyID, AStatus: string): string;
    function GetPCEInfo(APatientID, AStudyID: string): string;
    function GetUserOptionList(AUser: string): string;
    function GetCPTIUTitles(ATarget: string): string;
    function GetAppintmentInfo(APatientID, AStudyID: string): string;
    function GetVersionInfo(AGUIVersion: string): string;
    function GetSupportedVersion: string;
    function GetPhysicianOrders(APatient, AnOrderIEN: string): string;
    function SetStudyUseStatus(AStudyID, AStatusID, AStatusBy, AStatusAt, AWS: string): string;
    function GetStudyUseStatus(SStudyID: string; var SStatus, SStatusBy, SStatusAt, SWS: string): Boolean;
    function GetLockedStudies: string;
    function GetUserInfo: string;
    function GetXPar(AName: string): string;
    function SetXPar(AName, AValue: string): string;
  end;

function NewDBController_Renal(ARPCBroker: TRPCBroker): TDBController_Renal;

var
  VerifyWriteIsComplete: Boolean;

implementation

uses
  ODBConnection, 
// 2FA
  XWBHash,              // For E-sig encoding before sending over the wire
    MFunStr, DateUtils, UMDK_Utils, // Standard M Function for Piece call
    UMDK_LocalDB, UZZ_DescribedItem, FZZ_EventLog;

const
  RENAL_VITAL_SIGN_NAME: array [TRenalVitalSign] of string = ('T', 'P', 'R', 'BP', 'PN', 'WT');
  BOOL201: array [Boolean] of string                       = ('0', '1');
  RPC_BILLING_WIDGET                                       = 'MD TMDWIDGET';
  RPC_CLINIC_INFO                                          = 'MD TMDCIDC';
  RPC_CLINIC_WORKLOAD                                      = 'MD TMDCLIN';
  RPC_CurrentTime                                          = 'GMV GET CURRENT TIME';
  RPC_ENCOUNTER                                            = 'MD TMDENCOUNTER';
  RPC_FILE_VITALS                                          = 'GMV ADD VM';
  RPC_GATEWAY                                              = 'MD GATEWAY';
  RPC_GETSET_RENAL                                         = 'MDK GET/SET RENAL DATA';
  RPC_GET_VISTA_DATA                                       = 'MDK GET VISTA DATA';
  RPC_LEXICON                                              = 'MD TMDLEX';
  RPC_MANAGER                                              = 'GMV MANAGER';
  RPC_PHYSICIAN_ORDERS                                     = 'MDK GET ORDERS';
  RPC_PT_INQUIRY                                           = 'ORWPT PTINQ';
  RPC_SENSITIVE_RA                                         = 'DG SENSITIVE RECORD ACCESS';
  RPC_SENSITIVE_RB                                         = 'DG SENSITIVE RECORD BULLETIN';
  RPC_TIU_TITLES                                           = 'MD TMDRECORDID';
  RPC_UTILITY                                              = 'MDK UTILITY';
  RPC_VERSION_INFO                                         = RPC_GETSET_RENAL;
  SYSTEM_ID                                                = '.5';

function NewDBController_Renal(ARPCBroker: TRPCBroker): TDBController_Renal;
begin
  Result := TDBController_Renal.Create(NewDBConnection(ARPCBroker));
end;
/// /////////////////////////////////////////////////////////////////////////////
{ TDBController_Renal }

function TDBController_Renal.IENS(AIEN: Int64): string;
{ Private function to take an integer and return an IENS for fileman }
begin
  Result := Format('%d,', [AIEN]);
end;

// zzzzzzandria 060419 ////////////////////////////////////////////////////start
function TDBController_Renal.IENS(AIEN: string): string;
{ Private function to take an integer and return an IENS for fileman }
begin
  Result := Format('%s,', [AIEN]);
end;
// zzzzzzandria 060419 //////////////////////////////////////////////////////end

function TDBController_Renal.GetActiveRenalPatients(var APatientList: string): Boolean;
{ TODO 1 -oPetit -cMandatory : Get Active Renal Patients }
begin
  if not Assigned(DBConnection) then Result := GetLocal(LocalName(LmActiveStudies, []), APatientList)
  else
    try
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['ACTIVE'], nil);
      APatientList := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmActiveStudies, []), APatientList)
    except
      on E: Exception do begin
        Result := False;
        APatientList := Format(XML_STATUS_ERROR, [E.Message]);
      end;
    end;
end;

function TDBController_Renal.GetAllStudiesByPatient(APatientID: Int64; var AStudyList: string): Boolean;
begin
  if not Assigned(DBConnection) then
      Result := GetLocal(LocalName(LmPatientStudyList, [IntToStr(APatientID)]), AStudyList)
  else
    try
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['ALLBYPT', IntToStr(APatientID)], nil);
      AStudyList := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmPatientStudyList, [IntToStr(APatientID)]), AStudyList)
    except
      on E: Exception do begin
        Result := False;
        AStudyList := Format(XML_STATUS_ERROR, [E.Message]);
      end;
    end;
end;

function TDBController_Renal.GetCCOWContext: string;
begin
  try
    if DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['CCOW'], nil) then Result := DBConnection.ExecResults[0]
    else Result := '';
  except
    Result := '';
  end;
end;

function TDBController_Renal.GetInstrumentResult(AResultID: Integer; var AInstrumentResult: string): Boolean;
begin
  if not Assigned(DBConnection) then
      Result := GetLocal(LocalName(LmInstrumentResults, [IntToStr(AResultID)]), AInstrumentResult)
  else
    try
      if DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETRSLT', IntToStr(AResultID)], nil) then begin
        DBConnection.ExecResults.Delete(0); // peel off the counter flag
        AInstrumentResult := DBConnection.ExecResults.Text;
        Result := True;
        if SaveRPCResults then SetLocal(LocalName(LmInstrumentResults, [IntToStr(AResultID)]), AInstrumentResult)
      end
      else Result := False;
    except
      on E: Exception do begin
        Result := False;
        AInstrumentResult := Format(XML_STATUS_ERROR, [E.Message]);
      end;
    end;
end;

function TDBController_Renal.GetInstrumentResultList(AStudyID: Integer; var AInstrumentResultList: string;
  AGetAll: Boolean = False): Boolean;
begin
  if not Assigned(DBConnection) then
      Result := GetLocal(LocalName(LmStudyResultList, [IntToStr(AStudyID)]), AInstrumentResultList)
  else
    try
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['ALLRSLT', IntToStr(AStudyID), BOOL201[AGetAll]], nil);
      AInstrumentResultList := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmStudyResultList, [IntToStr(AStudyID)]), AInstrumentResultList)
    except
      on E: Exception do begin
        Result := False;
        AInstrumentResultList := Format(XML_STATUS_ERROR, [E.Message]);
      end;
    end;
end;

function TDBController_Renal.GetBillingData(AStudyID: Integer; var ABillingData: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := GetLocal(LocalName(LmStudyBilling, [IntToStr(AStudyID)]), ABillingData)
  else
    try
      Result := DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['GETBILL', IntToStr(AStudyID)], nil);
      ABillingData := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmStudyBilling, [IntToStr(AStudyID)]), ABillingData)
    except
      on E: Exception do begin
        Result := False;
        ABillingData := Format(XML_STATUS_ERROR, [E.Message]);
      end;
    end;
end;

function TDBController_Renal.SetBillingData(AStudyID: Integer; AICD01, AICD02, AICD03, AICD04, ACPT01: string;
  ASvcConn, AMST, AAOExposure, AIRExposure, AEnvContaminants, AHeadNeckCancer, ACombatVet,
  ACampLejeune: Integer { boolean }; AProviderList: array of string): Boolean;
var
  ABillingData: TStringList;
  AProviderID: Integer;
  I: Integer;
begin
  ABillingData := TStringList.Create;
  ABillingData.Add(AICD01);
  ABillingData.Add(AICD02);
  ABillingData.Add(AICD03);
  ABillingData.Add(AICD04);
  ABillingData.Add(ACPT01);
  ABillingData.Add(IntToStr(ASvcConn));
  ABillingData.Add(IntToStr(AMST));
  ABillingData.Add(IntToStr(AAOExposure));
  ABillingData.Add(IntToStr(AIRExposure));
  ABillingData.Add(IntToStr(AEnvContaminants));
  ABillingData.Add(IntToStr(AHeadNeckCancer));
  ABillingData.Add(IntToStr(ACombatVet));
  ABillingData.Add(IntToStr(ACampLejeune));
  if Length(AProviderList) > 0 then
    for I := low(AProviderList) to high(AProviderList) do begin
      AProviderID := StrToIntDef(Piece(AProviderList[I], '^', 1), -1);
      if AProviderID > 0 then ABillingData.Add(Format('`%d', [AProviderID]));
    end;
  if not Assigned(DBConnection) then
      Result := SetLocal(LocalName(LmStudyBilling, [IntToStr(AStudyID)]), ABillingData.Text)
  else begin
    try
      Result := DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['SETBILL', IntToStr(AStudyID)], ABillingData);
    except
      Result := False;
    end;
    if SaveRPCResults then SetLocal(LocalName(LmStudyBilling, [IntToStr(AStudyID)]), ABillingData.Text);
  end;
  // zzzzzzandria 050427
  if ABillingData <> nil then ABillingData.Free;
end;

function TDBController_Renal.GetUserOption(AOptionName: string; var AOptionValue: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := GetLocal(LocalName(LmOptions, ['DUZ', AOptionName]), AOptionValue)
  else begin
    Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETOPT', AOptionName, '@DUZ'], nil);
    AOptionValue := DBConnection.ExecResults.Text;
    if SaveRPCResults then SetLocal(LocalName(LmOptions, ['DUZ', AOptionName]), AOptionValue)
  end;
end;

function TDBController_Renal.SetUserOption(AOptionName, AOptionValue: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := SetLocal(LocalName(LmOptions, ['DUZ', AOptionName]), AOptionValue)
  else begin
    // zzzzzzandria 2007-07-27 ----------------------------------------------- begin
    Result := LockSetting(AOptionName);
    if not Result then Exit;
    // zzzzzzandria 2007-07-27 ------------------------------------------------- end
    Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETOPT', AOptionName, '@DUZ'], AOptionValue);
    // zzzzzzandria 2007-07-27 ----------------------------------------------- begin
    UnLockSetting(AOptionName);
    // zzzzzzandria 2007-07-27 ------------------------------------------------- end
  end;
end;

function TDBController_Renal.GetUserOptionByDUZ(AOptionName, ADUZ: string; var AOptionValue: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := GetLocal(LocalName(LmOptions, [ADUZ, AOptionName]), AOptionValue)
  else begin
    Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETOPT', AOptionName, ADUZ], nil);
    AOptionValue := DBConnection.ExecResults.Text;
    if SaveRPCResults then SetLocal(LocalName(LmOptions, [ADUZ, AOptionName]), AOptionValue)
  end;
end;

function TDBController_Renal.SetUserOptionByDUZ(AOptionName, ADUZ, AOptionValue: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := SetLocal(LocalName(LmOptions, [ADUZ, AOptionName]), AOptionValue)
  else begin
    // zzzzzzandria 2007-08-29 No locking for user option
    // zzzzzzandria 2007-07-27 ----------------------------------------------- begin
    // Result := LockSetting(aOptionName);
    // if not Result then Exit;
    // zzzzzzandria 2007-07-27 ------------------------------------------------- end
    Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETOPT', AOptionName, ADUZ], AOptionValue);
    // zzzzzzandria 2007-08-29 No locking for user option
    // zzzzzzandria 2007-07-27 ----------------------------------------------- begin
    // UnLockSetting(aOptionName);
    // zzzzzzandria 2007-07-27 ------------------------------------------------- end
  end;
end;

function TDBController_Renal.DeleteUserOption(AOptionName: string): Boolean;
begin
  // if not Assigned(DBConnection) then
  // Result :=  deleteLocal('RPC\MDK GETSET RENAL DATA\GETOPT\USER\'+aOptionName)
  // else
  Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['DELOPT', AOptionName, '@DUZ'], nil);
end;

function TDBController_Renal.GetSystemOption(AOptionName: string; var AOptionValue: string): Boolean;
begin
  // GETS ICD9/10 HERE zzzzzzcarnej
  // 'MDK GET/SET RENAL DATA'
  if not Assigned(DBConnection) then
    // Result := getLocal(LocalName(lmOptions, ['SYSTEM', aOptionName]), aOptionValue)
      Result := GetLocal(LocalName(LmOptions, ['.5', AOptionName]), AOptionValue)
  else begin
    Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETOPT', AOptionName, '.5'], nil);
    AOptionValue := DBConnection.ExecResults.Text;
    if SaveRPCResults then SetLocal(LocalName(LmOptions, ['.5', AOptionName]), AOptionValue)
  end;
end;

function TDBController_Renal.SetSystemOption(AOptionName, AOptionValue: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := SetLocal(LocalName(LmOptions, ['5', AOptionName]), AOptionValue)
  else begin
    // zzzzzzandria 2007-07-27 ----------------------------------------------- begin
    Result := LockSetting(AOptionName);
    if not Result then Exit;
    // zzzzzzandria 2007-07-27 ------------------------------------------------- end
    Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETOPT', AOptionName, '.5'], AOptionValue);
    // zzzzzzandria 2007-07-27 ----------------------------------------------- begin
    UnLockSetting(AOptionName);
    // zzzzzzandria 2007-07-27 ------------------------------------------------- end
    if SaveRPCResults then Result := SetLocal(LocalName(LmOptions, ['5', AOptionName]), AOptionValue)
  end;
end;

function TDBController_Renal.DeleteSystemOption(AOptionName: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := DeleteLocal(LocalName(LmOptions, ['5', AOptionName]))
  else Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['DELOPT', AOptionName, '.5'], nil);
end;

function TDBController_Renal.GetPatientAccessPoints(APatientID: Int64;
  var AAccessPoints, AAccessHistory, AInfectionHistory: string): Boolean;
var
  SID: string;
begin
  if not Assigned(DBConnection) then begin
    SID := IntToStr(APatientID);
    GetLocal(LocalName(LmPatientInfo, [SID, 'AccessPoints.XML']), AAccessPoints);
    GetLocal(LocalName(LmPatientInfo, [SID, 'AccessHistory.XML']), AAccessHistory);
    GetLocal(LocalName(LmPatientInfo, [SID, 'AccessInfectionHistory.XML']), AInfectionHistory);
    Result := True;
  end
  else
    try
      SID := IENS(APatientID);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.201', SID, '.1'], nil);
      AAccessPoints := DBConnection.ExecResults.Text;
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.201', SID, '.2'], nil);
      AAccessHistory := DBConnection.ExecResults.Text;
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.201', SID, '.3'], nil);
      AInfectionHistory := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then begin
        SID := IntToStr(APatientID);
        SetLocal(LocalName(LmPatientInfo, [SID, 'AccessPoints.XML']), AAccessPoints);
        SetLocal(LocalName(LmPatientInfo, [SID, 'AccessHistory.XML']), AAccessHistory);
        SetLocal(LocalName(LmPatientInfo, [SID, 'AccessInfectionHistory.XML']), AInfectionHistory);
      end;
    except
      on E: Exception do Result := False;
    end;
end;

function TDBController_Renal.SetPatientAccessPoints(APatientID: Int64;
  AAccessPoints, AAccessHistory, AInfectionHistory: string): Boolean;
var
  SID: string;
begin
  if not Assigned(DBConnection) then begin
    SID := IntToStr(APatientID);
    SetLocal(LocalName(LmPatientInfo, [SID, 'AccessPoints.XML']), AAccessPoints);
    SetLocal(LocalName(LmPatientInfo, [SID, 'AccessHistory.XML']), AAccessHistory);
    SetLocal(LocalName(LmPatientInfo, [SID, 'AccessInfectionHistory.XML']), AInfectionHistory);
    Result := True;
  end
  else
    try
      SID := IENS(APatientID);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.201', SID, '.1'], AAccessPoints);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.201', SID, '.2'], AAccessHistory);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.201', SID, '.3'], AInfectionHistory);
      Result := True;
    except
      Result := False;
    end;
end;

function TDBController_Renal.GetStudy(AStudyID: Integer; var AStudyInfo, AStudySummary, AStudyFlowsheet, AMedLog,
  ANoteList, AnEventLog: string): Boolean;
var
  SID: string;
begin
  if not Assigned(DBConnection) then begin
    SID := IntToStr(AStudyID);
    GetLocal(LocalName(LmStudyComponent, [SID, 'Info']), AStudyInfo);
    GetLocal(LocalName(LmStudyComponent, [SID, 'Summary']), AStudySummary);
    GetLocal(LocalName(LmStudyComponent, [SID, 'Flowsheet']), AStudyFlowsheet);
    GetLocal(LocalName(LmStudyComponent, [SID, 'MedLog']), AMedLog);
    GetLocal(LocalName(LmStudyComponent, [SID, 'Notes']), ANoteList);
    GetLocal(LocalName(LmStudyComponent, [SID, 'Events']), AnEventLog);
    Result := True;
  end
  else
    try
      SID := IENS(AStudyID);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', SID, '.1'], nil);
      AStudyInfo := DBConnection.ExecResults.Text;
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', SID, '.2'], nil);
      AStudySummary := DBConnection.ExecResults.Text;
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', SID, '.3'], nil);
      AStudyFlowsheet := DBConnection.ExecResults.Text;
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', SID, '.4'], nil);
      AMedLog := DBConnection.ExecResults.Text;
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', SID, '.5'], nil);
      ANoteList := DBConnection.ExecResults.Text;
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', SID, '.6'], nil);
      AnEventLog := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then begin
        SID := IntToStr(AStudyID);
        SetLocal(LocalName(LmStudyComponent, [SID, 'Info']), AStudyInfo);
        SetLocal(LocalName(LmStudyComponent, [SID, 'Summary']), AStudySummary);
        SetLocal(LocalName(LmStudyComponent, [SID, 'Flowsheet']), AStudyFlowsheet);
        SetLocal(LocalName(LmStudyComponent, [SID, 'MedLog']), AMedLog);
        SetLocal(LocalName(LmStudyComponent, [SID, 'Notes']), ANoteList);
        SetLocal(LocalName(LmStudyComponent, [SID, 'Events']), AnEventLog);
      end;
    except
      Result := False;
    end;
end;

function TDBController_Renal.GetStudyItem(AFile, ANode, AnItemID: string; var AValue: string): Boolean;
var
  ComponentName: string;
begin
  Result := False;
  AValue := '';
  if (AFile = '704.202') //
    and (ANode <> '.1')  // StudyInfo
    and (ANode <> '.2')  // StudySummary
    and (ANode <> '.3')  // StudyFlowsheet
    and (ANode <> '.4')  // StudyMedLog
    and (ANode <> '.5')  // StudyNoteList
    and (ANode <> '.6') { // StudyEventLog } then Exit;
  if (AFile = '704.201') //
    and (ANode <> '.1')  // AP
    and (ANode <> '.2')  // APH
    and (ANode <> '.3') { // APIH } then Exit;
  if not Assigned(DBConnection) then begin
    if (AFile = '704.202') then begin
      if (ANode = '.1') then ComponentName := 'Info';      // StudyInfo
      if (ANode = '.2') then ComponentName := 'Summary';   // StudySummary
      if (ANode = '.3') then ComponentName := 'Flowsheet'; // StudyFlowsheet
      if (ANode = '.4') then ComponentName := 'MedLog';    // StudyMedLog
      if (ANode = '.5') then ComponentName := 'Notes';     // StudyNoteList
      if (ANode = '.6') then ComponentName := 'Events';    // StudyEventLog
      Result := GetLocal(LocalName(LmStudyComponent, [AnItemID, ComponentName]), AValue);
    end else begin
      if (ANode = '.1') then ComponentName := 'AccessPoints'; // StudyInfo
      if (ANode = '.2') then ComponentName := 'AccessHistory'; // StudySummary
      if (ANode = '.3') then ComponentName := 'AccessInfectionHistory'; // StudyFlowsheet
      Result := GetLocal(LocalName(LmPatientInfo, [AnItemID, ComponentName]), AValue);
    end;
  end
  else
    try
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', AFile, IENS(AnItemID), ANode], nil);
      AValue := DBConnection.ExecResults.Text;
      Result := True;
    except
      Result := False;
    end;
end;

function TDBController_Renal.SetStudy(AStudyID: Integer; AStudyInfo, AStudySummary, AStudyFlowsheet, AMedLog, ANoteList,
  AnEventLog: string): Boolean;
var
  SID: string;
begin
  if not Assigned(DBConnection) then begin
    SID := IntToStr(AStudyID);
    SetLocal(LocalName(LmStudyComponent, [SID, 'Info']), AStudyInfo);
    SetLocal(LocalName(LmStudyComponent, [SID, 'Summary']), AStudySummary);
    SetLocal(LocalName(LmStudyComponent, [SID, 'Flowsheet']), AStudyFlowsheet);
    SetLocal(LocalName(LmStudyComponent, [SID, 'MedLog']), AMedLog);
    SetLocal(LocalName(LmStudyComponent, [SID, 'Notes']), ANoteList);
    SetLocal(LocalName(LmStudyComponent, [SID, 'Events']), AnEventLog);
    Result := True;
  end
  else
    try
      SID := IENS(AStudyID);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202', IENS(AStudyID), '.1'], AStudyInfo);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202', IENS(AStudyID), '.2'], AStudySummary);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202', IENS(AStudyID), '.3'], AStudyFlowsheet);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202', IENS(AStudyID), '.4'], AMedLog);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202', IENS(AStudyID), '.5'], ANoteList);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202', IENS(AStudyID), '.6'], AnEventLog);
      Result := True;
      if SaveRPCResults then begin
        SID := IntToStr(AStudyID);
        SetLocal(LocalName(LmStudyComponent, [SID, 'Info']), AStudyInfo);
        SetLocal(LocalName(LmStudyComponent, [SID, 'Summary']), AStudySummary);
        SetLocal(LocalName(LmStudyComponent, [SID, 'Flowsheet']), AStudyFlowsheet);
        SetLocal(LocalName(LmStudyComponent, [SID, 'MedLog']), AMedLog);
        SetLocal(LocalName(LmStudyComponent, [SID, 'Notes']), ANoteList);
        SetLocal(LocalName(LmStudyComponent, [SID, 'Events']), AnEventLog);
      end;
    except
      Result := False;
    end;
end;

function TDBController_Renal.SetStudyItem(AFile, ANode, AnItemID: string; AValue: string;
  APatient: string = ''): Boolean;
var
  ComponentName: string;
begin
  Result := False;
  if (AFile <> '704.202') and (AFile <> '704.201') then Exit;
  if (AFile = '704.202') // 704.202
    and (ANode <> '.1')  // StudyInfo
    and (ANode <> '.2')  // StudySummary
    and (ANode <> '.3')  // StudyFlowsheet
    and (ANode <> '.4')  // StudyMedLog
    and (ANode <> '.5')  // StudyNoteList
    and (ANode <> '.6') { // StudyEventLog } then Exit;
  if (AFile = '704.201') // 704.201
    and (ANode <> '.1')  // AP
    and (ANode <> '.2')  // APH
    and (ANode <> '.3') { // APIH } then Exit;
  if not Assigned(DBConnection) then begin
    if (AFile = '704.202') then begin
      if (ANode = '.1') then ComponentName := 'Info';      // StudyInfo
      if (ANode = '.2') then ComponentName := 'Summary';   // StudySummary
      if (ANode = '.3') then ComponentName := 'Flowsheet'; // StudyFlowsheet
      if (ANode = '.4') then ComponentName := 'MedLog';    // StudyMedLog
      if (ANode = '.5') then ComponentName := 'Notes';     // StudyNoteList
      if (ANode = '.6') then ComponentName := 'Events';    // StudyEventLog
      Result := SetLocal(LocalName(LmStudyComponent, [AnItemID, ComponentName]), AValue);
    end else begin
      if (ANode = '.1') then ComponentName := 'AccessPoints.XML'; // StudyInfo
      if (ANode = '.2') then ComponentName := 'AccessHistory.XML'; // StudySummary
      if (ANode = '.3') then ComponentName := 'AccessInfectionHistory.XML'; // StudyFlowsheet
      Result := SetLocal(LocalName(LmPatientInfo, [APatient, ComponentName]), AValue);
    end;
  end
  else
    try
      // Note SETXML requires ',' to be added to the study ID
      //
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', AFile, AnItemID, ANode], AValue);
      Result := True;
    except
      Result := False;
    end;
end;

function TDBController_Renal.LockStudy(AStudyID: Integer): Boolean;
{ TODO 1 -oPetit -cMandatory : Lock Study }
var
  S: string;
begin
  if not Assigned(DBConnection) then begin
    // Result := True
    Result := GetLocalLock(IntToStr(AStudyID), S);
    if S <> 'LOCKED' then Result := SetLocalLock(IntToStr(AStudyID), 'LOCKED');
  end else if DBConnection.ExecRemoteProc(RPC_UTILITY, ['LOCK', '704.202', IENS(AStudyID)], nil) then
      Result := Copy(DBConnection.ExecResults.Text, 1, 2) <> '-1'
  else Result := False;
  if Assigned(DBConnection) then ResultInfo := Piece(DBConnection.ExecResults.Text, '^', 2);
end;

function TDBController_Renal.UnlockStudy(AStudyID: Integer): Boolean;
var
  S: string;
begin
  if not Assigned(DBConnection) then begin
    // Result := True
    Result := GetLocalLock(IntToStr(AStudyID), S);
    if S = 'LOCKED' then Result := SetLocalLock(IntToStr(AStudyID), '');
  end else if DBConnection.ExecRemoteProc(RPC_UTILITY, ['UNLOCK', '704.202', IENS(AStudyID)], nil) then
      Result := Copy(DBConnection.ExecResults.Text, 1, 2) <> '-1'
  else Result := False;
  if Assigned(DBConnection) then ResultInfo := Piece(DBConnection.ExecResults.Text, '^', 2);
end;

function FormatInfo(AnInfo: string): string;
var
  Ss, S: string;
begin
  S := AnInfo;
  Ss := Piece(S, '^', 4, 4);
  Ss := Copy(Ss, 4, 2) + '/' + Copy(Ss, 6, 2) + '/' + Copy(Ss, 2, 2) + '  ' + Copy(Ss, 9, 2) + ':' + Copy(Ss, 11, 2) +
    ':' + Copy(Ss, 13, 2) + #13#10 + '             (' + Ss + ')';
  Result := 'Reason:   ' + Piece(S, '^', 2, 2) + #13#10 + 'User DUZ: ' + Piece(S, '^', 3, 3) + #13#10 + 'Since:    ' +
    Ss + #13#10 + 'Job #:    ' + Piece(S, '^', 5, 5);
end;

function TDBController_Renal.LockSetting(ASettingName: string): Boolean;
var
  S: string;
begin
  if not Assigned(DBConnection) then begin
    Result := True;
    ResultInfo := '';
  end else begin
    if DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['LOCKOPT', ASettingName, '1'], nil) then begin
      Result := (Copy(DBConnection.ExecResults.Text, 1, 2) <> '-1');
    end
    else Result := { False; }
        (Copy(DBConnection.ExecResults.Text, 1, 18) = '-1^No such setting');
    S := DBConnection.ExecResults.Text;
    ResultInfo := FormatInfo(S);
  end;
end;

function TDBController_Renal.UnlockSetting(ASettingName: string): Boolean;
var
  S: string;
begin
  if not Assigned(DBConnection) then begin
    Result := True;
    ResultInfo := '';
  end else begin
    if DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['LOCKOPT', ASettingName, '0'], nil) then begin
      Result := Copy(DBConnection.ExecResults.Text, 1, 2) <> '-1';
    end
    else Result := False;
    S := DBConnection.ExecResults.Text;
    ResultInfo := FormatInfo(S);
  end;
end;

function TDBController_Renal.ValidateElectronicSignature(ASignature: string): Boolean;
{ Encrypt is included in the Hash Unit from VistA Broker }
begin
  if not Assigned(DBConnection) then Result := ASignature = 'TRUE'
  else begin
    ASignature := XWBHash.Encrypt(ASignature); // 2FA
    Result := DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['ESIG', ASignature], nil);
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.LookupProvider(APartialName: string; var AReturnList: string): Boolean;
begin
  if not Assigned(DBConnection) then begin
    GetLocal(LocalName(LmProviders, []), AReturnList);
    Result := True;
  end
  else
    try
      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['GETPROV', APartialName], nil);
      AReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then SetLocal(LocalName(LmProviders, []), AReturnList);
    except
      Result := False;
    end;
end;

function TDBController_Renal.LookupFacility(APartialName: string; var AReturnList: string): Boolean;
var
  Sl: TStringList;
  I: Integer;
begin
  if not Assigned(DBConnection) then GetLocal(LocalName(LmFacilities, []), AReturnList)
  else
    try
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETINST', APartialName], nil);
      AReturnList := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmFacilities, []), AReturnList)
    except
    end;
  Sl := TStringList.Create;
  if Pos('<STATUS>OK</STATUS>', AReturnList) = 0 then Result := False
  else begin
    Result := True;
    Sl.Text := AReturnList;
    AReturnList := #13#10;
    for I := 0 to Sl.Count - 1 do begin
      if Pos('<STATION_NUMBER>', Sl[I]) <> 1 then Continue
      else AReturnList := AReturnList + Piece(Piece(Sl[I], '<STATION_NUMBER>', 2), '<', 1) + '^' +
          Piece(Piece(Sl[I + 1], '<STATION_NAME>', 2), '<', 1) + #13#10;
    end;
  end;
  Sl.Free;
end;

function TDBController_Renal.LookupAdministrator(APartialName: string; var AReturnList: string): Boolean;
var
  Sl: TStringList;
  I: Integer;
begin
  if not Assigned(DBConnection) then GetLocal(LocalName(LmAdministrators, []), AReturnList)
  else
    try
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETADM', APartialName], nil);
      AReturnList := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmAdministrators, []), AReturnList)
    except
    end;
  Sl := TStringList.Create;
  if Pos('<STATUS>OK</STATUS>', AReturnList) = 0 then Result := False
  else begin
    Sl.Text := AReturnList;
    AReturnList := #13#10;
    for I := 0 to Sl.Count - 1 do begin
      if Pos('<NUMBER>', Sl[I]) <> 1 then Continue
      else AReturnList := AReturnList + Piece(Piece(Sl[I], '<NUMBER>', 2), '<', 1) + '^' +
          Piece(Piece(Sl[I + 1], '<NAME>', 2), '<', 1) + #13#10;
    end;
    Result := True;
  end;
  Sl.Free;
end;

function TDBController_Renal.SubmitNoteNow(AStudyID: Integer; AUserID: string; AUserESig: string;
  // E-Sig in clear text, it will be encrypted before sending
  ADocumentText: TStringList; ATitle: string): Boolean;
begin
  if not Assigned(DBConnection) then begin
    SetLocal(LocalName(LmTIU, [IntToStr(AStudyID), FormatDateTime('yyyymmddhhnnss', Now)]), ADocumentText.Text);
    Result := True;
  end
  else
    try
      Result := DBConnection.ExecRemoteProc('MD TMDNOTE', ['NEWDOC', IntToStr(AStudyID), '', '', AUserID,
        Encrypt(AUserESig), ATitle], ADocumentText);
      ResultInfo := DBConnection.ExecResults.Text;
    except
      Result := False;
    end;
end;

function TDBController_Renal.SubmitStudy(AStudyID: Integer; AUserESig: string;
  // E-Sig in clear text, it will be encrypted before sending
  ADocumentText: TStringList): Boolean;
var
  S: string;
begin
  if not Assigned(DBConnection) then begin
    S := ADocumentText.Text;
    SetLocal(LocalName(LmStudyReport, [IntToStr(AStudyID)]), S);
    Result := True;
  end
  else
    try
      Result := DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['SUBMIT', IntToStr(AStudyID), Encrypt(AUserESig)],
        ADocumentText);
      ResultInfo := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmStudyReport, [IntToStr(AStudyID)]), S);
    except
      Result := False;
    end;
end;

procedure TDBController_Renal.GetPatientInfo(ADFN: string; Results: TStringList);
var
  I: Integer;
  S: string;
begin
  if not Assigned(DBConnection) then begin
    GetLocal(LocalName(LmPatientInfo, [ADFN, 'Inquiry.txt']), S);
    Results.Text := S;
  end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_PT_INQUIRY, [ADFN], nil);
      for I := 0 to DBConnection.ExecResults.Count - 1 do Results.Add(DBConnection.ExecResults[I]);
      if SaveRPCResults then SetLocal(LocalName(LmPatientInfo, [ADFN, 'Inquiry.txt']), Results.Text);
    except
      on E: Exception do Results.Text := E.Message;
    end;
end;

procedure TDBController_Renal.GetPatientLabInfo(ADFN: string; Results: TStringList);
var
  I: Integer;
  S: string;
  SDFN: string;
begin
  SDFN := Piece(ADFN, '^', 1);
  if not Assigned(DBConnection) then begin
    GetLocal(LocalName(LmPatientInfo, [SDFN, 'Lab.txt']), S);
    Results.Text := S;
  end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['LAB', ADFN], nil);
      if Pos('-1', DBConnection.ExecResults[0]) = 1 then
          Results.Add( { 'Error getting patient lab information'#13#10#13#10+ }
          Piece(DBConnection.ExecResults[0], '^', 2))
      else
        for I := 1 to DBConnection.ExecResults.Count - 1 do begin
          S := Piece(DBConnection.ExecResults[I], '^', 2);
          try
            S := Copy(S, 5, 2) + '/' + Copy(S, 7, 2) + '/' + Copy(S, 1, 4);
          except
            S := '';
          end;
          Results.Add(Piece(DBConnection.ExecResults[I], '^', 1) + '  -- ' + S + ' --  ' +
            Piece(DBConnection.ExecResults[I], '^', 3) + ' ' + Piece(DBConnection.ExecResults[I], '^', 4));
        end;
      if SaveRPCResults then SetLocal(LocalName(LmPatientInfo, [SDFN, 'Lab.txt']), S);
    except
      on E: Exception do Results.Text := E.Message;
    end;
end;

procedure TDBController_Renal.GetPatientAllergies(ADFN: string; Results: TStringList);
var
  I: Integer;
  S: string;
begin
  if not Assigned(DBConnection) then begin
    GetLocal(LocalName(LmPatientInfo, [ADFN, 'Allergy.txt']), S);
    Results.Text := S;
    Exit;
  end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['ALLERGY', ADFN], nil);
      S := Piece(DBConnection.ExecResults[0], '^', 1);
      if S = '-1' then Results.Add('No Allergies')
      else begin
        if (DBConnection.ExecResults.Count = 2) and (Pos('No', DBConnection.ExecResults[1]) <> 1) then
            Results.Add('Allergies:');
        for I := 1 to DBConnection.ExecResults.Count - 1 do Results.Add(DBConnection.ExecResults[I]);
      end;
      if SaveRPCResults then SetLocal(LocalName(LmPatientInfo, [ADFN, 'Allergy.txt']), S);
    except
      on E: Exception do Results.Text := E.Message;
    end;
end;

procedure TDBController_Renal.GetPatientDirectives(ADFN: string; Results: TStringList);
var
  I: Integer;
  S: string;
begin
  if not Assigned(DBConnection) then begin
    GetLocal(LocalName(LmPatientInfo, [ADFN, 'Directives.txt']), S);
    Results.Text := S;
  end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['AD', ADFN], nil);
      S := Piece(DBConnection.ExecResults[0], '^', 1);
      if S = '-1' then Results.Add('No Advanced Directive')
        // changed to singular by zzzzzzandria 050506
      else
        for I := 1 to DBConnection.ExecResults.Count - 1 do
            Results.Add( { 'Advanced directives: ' + } DBConnection.ExecResults[I]);
      if SaveRPCResults then SetLocal(LocalName(LmPatientInfo, [ADFN, 'Directives.txt']), S);
    except
      on E: Exception do Results.Text := E.Message;
    end;
end;

procedure TDBController_Renal.GetPatientWarnings(ADFN: string; Results: TStringList);
var
  I: Integer;
  S: string;
begin
  if not Assigned(DBConnection) then begin
    GetLocal(LocalName(LmPatientInfo, [ADFN, 'Warnings.txt']), S);
    Results.Text := S;
  end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['CW', ADFN], nil); // ??
      S := Piece(DBConnection.ExecResults[0], '^', 1);
      if S = '-1' then Results.Add('No Personal Warnings')
        // changed to singular by zzzzzzandria 050506
      else
        for I := 1 to DBConnection.ExecResults.Count - 1 do begin
          S := DBConnection.ExecResults[I];
          if Pos('None', S) > 0 then Results.Add(S)
          else Results.Add(FMDateTimeStr(Piece(S, '^', 5)));
        end;
      if SaveRPCResults then SetLocal(LocalName(LmPatientInfo, [ADFN, 'Warnings.txt']), S);
    except
      on E: Exception do Results.Text := E.Message;
    end;
end;

procedure TDBController_Renal.GetPatientVAccination(ADFN: string; Results: TStringList);
var
  I: Integer;
  S: string;
begin
  if not Assigned(DBConnection) then begin
    GetLocal(LocalName(LmPatientInfo, [ADFN, 'Vaccination.txt']), S);
    Results.Text := S;
  end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['SHOTS', ADFN], nil);
      S := Piece(DBConnection.ExecResults[0], '^', 1);
      if S = '-1' then Results.Add('No Data Found')
      else begin
        if (DBConnection.ExecResults.Count = 2) and (Pos('No', DBConnection.ExecResults[1]) <> 1) then
            Results.Add('Vaccination:');
        for I := 1 to DBConnection.ExecResults.Count - 1 do Results.Add(DBConnection.ExecResults[I]);
      end;
      if SaveRPCResults then SetLocal(LocalName(LmPatientInfo, [ADFN, 'Vaccination.txt']), S);
    except
      on E: Exception do Results.Text := E.Message;
    end;
end;

function TDBController_Renal.SetVitalsMeasurement(ADateTimeTaken: TDateTime; APatientDFN: string;
  AVitalsType: TRenalVitalSign; AHospitalLocation: string; AUserDUZ: string; AValue: string): Boolean;
  function QuickFMDateTime(ADT: TDateTime): string;
  var
    Y, M, D, Hh, Mm, Ss, Ms: Word;
  begin
    DecodeDateTime(ADT, Y, M, D, Hh, Mm, Ss, Ms);
    Y := Y - 1700; { Fileman specific }
    Result := FloatToStr((Y * 10000) + (M * 100) + D + (Hh * 0.01) + (Mm * 0.0001) + (Ss * 0.000001));
  end;

begin
  if not Assigned(DBConnection) then begin
    SetLocal(LocalName(LmPatientInfo, [APatientDFN, 'LatestVitals']), AValue);
    Result := True;
  end
  else
    try
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['VITALS', QuickFMDateTime(ADateTimeTaken), { P1 }
        APatientDFN, { P2 }
        RENAL_VITAL_SIGN_NAME[AVitalsType], { P3 }
        AValue,                             { P4 }
        AHospitalLocation,                  { P5 }
        AUserDUZ { +'*'+ QUALIFIER +';' }   { P6 }
        ], nil);
      if SaveRPCResults then SetLocal(LocalName(LmPatientInfo, [APatientDFN, 'LatestVitals']), AValue);
    except
      Result := False;
    end;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.GetNoteList(AStudyID: Integer): Boolean;
var
  S: string;
begin
  if not Assigned(DBConnection) then begin
    Result := GetLocalList(LocalName(LmTIUList, [AStudyID]), S);
    ResultInfo := S;
  end
  else
    try
      Result := DBConnection.ExecRemoteProc('MD TMDNOTE', ['NOTELIST', IntToStr(AStudyID)], '');
      ResultInfo := DBConnection.ExecResults.Text;
    except
      Result := False;
    end;
end;

function TDBController_Renal.GetNote(AStudyID: Integer; ANote: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := GetLocal(LocalName(LmTIU, [IntToStr(AStudyID), ANote]), ResultInfo)
  else
    try
      Result := DBConnection.ExecRemoteProc('MD TMDNOTE', ['VIEWTIU', IntToStr(AStudyID), ANote], '');
      ResultInfo := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmTIU, [IntToStr(AStudyID), ANote]), ResultInfo);
    except
      Result := False;
    end;
  if SaveRPCResults then SetLocal(LocalName(LmTIU, [IntToStr(AStudyID), ANote]), ResultInfo);
end;
/// /////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.GetLexiconData(ATarget, AnOption, StudyID: string; var AReturnList: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := GetLocal(LocalName(LmLexicon, [AnOption]), AReturnList)
  else
    try
      DBConnection.ExecRemoteProc(RPC_LEXICON, [ATarget, AnOption, StudyID], nil);
      // added StudyID 09/21/2012 zzzzzzcarnej
      AReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then SetLocal(LocalName(LmLexicon, [AnOption]), AReturnList);
    except
      Result := False;
    end;
end;

/// /////////////////////////////////////////////////////////////////////////////
function TDBController_Renal.SetFMDate(ImpDate: TDateTime): string; // 09/20/2012 zzzzzzcarnej
var
  Yr, Md: string;
  CDate: TDateTime;
begin
  CDate := Impdate;
  if CDate = 0 then CDate := Now;
  Yr := InttoStr(StrtoInt(FormatDateTime('yyyy', CDate)) - 1700);
  Md := FormatDateTime('mmdd', CDate);
  Result := Yr + Md;
end;
/// //////////////////////////////////////////////////////////////////////////

function TDBController_Renal.GetClinicProcedures(ADFN, AClinic: string; var AReturnList: string): Boolean;
begin
  if not Assigned(DBConnection) then
      Result := GetLocal(LocalName(LmPatientProceduresInfo, [ADFN, AClinic]), AReturnList)
  else
    try
      DBConnection.ExecRemoteProc(RPC_CLINIC_INFO, ['PROC', ADFN, AClinic], nil);
      AReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then SetLocal(LocalName(LmPatientProceduresInfo, [ADFN, AClinic]), AReturnList)
    except
      Result := False;
    end;
end;

function TDBController_Renal.GetClinicDiagnosis(ADFN, AClinic: string; var AReturnList: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := GetLocal(LocalName(LmPatientDiagnosisInfo, [ADFN, AClinic]), AReturnList)
  else
    try
      DBConnection.ExecRemoteProc(RPC_CLINIC_INFO, ['DIAG', ADFN, AClinic], nil);
      AReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then SetLocal(LocalName(LmPatientDiagnosisInfo, [ADFN, AClinic]), AReturnList)
    except
      Result := False;
    end;
end;

function TDBController_Renal.GetServiceConnections(ADFN, AClinic: string; var AReturnList: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := GetLocal(LocalName(LmPatientServiceInfo, [ADFN, AClinic]), AReturnList)
  else
    try
      DBConnection.ExecRemoteProc(RPC_CLINIC_INFO, ['SCDISP', ADFN, AClinic], nil);
      AReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then SetLocal(LocalName(LmPatientServiceInfo, [ADFN, AClinic]), AReturnList)
    except
      Result := False;
    end;
end;

function TDBController_Renal.GetEncounterData(AStudy: string; var AReturnList: string): Boolean;
begin
  if not Assigned(DBConnection) then Result := GetLocal(LocalName(LmStudyEncounter, [AStudy]), AReturnList)
  else
    try
      DBConnection.ExecRemoteProc(RPC_ENCOUNTER, [AStudy], nil);
      AReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then SetLocal(LocalName(LmStudyEncounter, [AStudy]), AReturnList)
    except
      Result := False;
    end;
end;

function TDBController_Renal.SetEncounterData(AStudy: string; ABillingData: TStringList;
  var AReturnList: string): Boolean;
begin
  AReturnList := '';
  if not Assigned(DBConnection) then Result := SetLocal(LocalName(LmStudyEncounter, [AStudy]), AReturnList)
  else
    try
      // MD TMDWIDGET
      RESULT := DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['PCE', AStudy, '', '', '', '', ''], ABillingData);
      AReturnList := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmStudyEncounter, [AStudy]), AReturnList)
    except
      Result := False;
    end;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.GetLookupEntries(AFile, ATarget: string): TStringList;
var
  SL: TStringList;
  S: string;
begin
  SL := TStringList.Create;
  if not Assigned(DBConnection) then begin
    if ATarget = '' then ATarget := '_';
    GetLocal(LOCAL_DIR + AFile + '\' + ATarget, S);
    SL.Text := S;
  end else begin
    DBConnection.ExecRemoteProc(RPC_MANAGER, ['LOOKUP', AFile + '^' + ATarget], nil);
    SL.Text := DBConnection.ExecResults.Text;
    if SaveRPCResults then begin
      if ATarget = '' then ATarget := '_';
      SetLocal(LOCAL_DIR + AFile + '\' + ATarget, SL.Text);
    end;
  end;
  Result := SL;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.GetLocation(AStudy: string): string;
var
  S: string;
begin
  if not Assigned(DBConnection) then begin
    GetLocal(LocalName(LmStudyLocation, [AStudy]), S);
    Result := S;
  end
  else
    try
      DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['GETLOC', AStudy], nil);
      Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmStudyLocation, [AStudy]), S);
    except
      Result := '-1^RPC ' + RPC_BILLING_WIDGET + ' Failed' + #13#10 + 'Study:' + AStudy;
    end;
end;

function TDBController_Renal.SetLocation(AStudy, ALocation: string): string;
begin
  if not Assigned(DBConnection) then begin
    SetLocal(LocalName(LmStudyLocation, [AStudy]), ALocation);
    Result := '';
  end
  else
    try
      DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['SETLOC', AStudy, ALocation], nil);
      Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmStudyLocation, [AStudy]), ALocation);
    except
      Result := '-1^RPC ' + RPC_BILLING_WIDGET + ' Failed' + #13#10 + 'Study:' + AStudy + #13#10 + 'Location:' +
        ALocation;
    end;
end;

function TDBController_Renal.ValidatePCEData(AStudy, AData: string): string;
begin
  if not Assigned(DBConnection) then begin
    Result := '';
    Exit;
  end
  else
    try
      DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['CHECK', AStudy, AData], nil);
      Result := DBConnection.ExecResults.Text;
    except
      Result := '-1^RPC ' + RPC_BILLING_WIDGET + ' Failed' + #13#10 + 'Study:' + AStudy + #13#10 + 'Data:' + AData;
    end;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.GetUserNameByIEN(AnIEN: string): string;
begin
  if not Assigned(DBConnection) then Result := GetItemLocal(LocalName(LmUserNamesList, []), AnIEN)
  else
    try
      DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['NAME', '', AnIEN], nil);
      Result := Piece(DBConnection.ExecResults.Text, '^', 2);
      if Result = '' then Result := AnIEN;
      if SaveRPCResults then SetItemLocal(LocalName(LmUserNamesList, []), Result, AnIEN);
    except
      Result := '-1^RPC ' + RPC_BILLING_WIDGET + ' Failed' + #13#10 + 'Option: NAME' + #13#10 + 'IEN:' + AnIEN;
    end;
end;

function TDBController_Renal.GetUserNameByIEN2(AnIEN: string): string;
begin
  Result := '';
  if not Assigned(DBConnection) then Result := GetItemLocal(LocalName(LmUserNamesList, []), AnIEN)
  else
    try
      if DBConnection.ExecRemoteProc(RPC_MANAGER, ['GETDATA', '200^' + AnIEN + '^.01'], nil) then
          Result := DBConnection.ExecResults.Text
      else Result := '<' + AnIEN + '>'; // 061128 zzzzzzandria
      if SaveRPCResults then SetItemLocal(LocalName(LmUserNamesList, []), Result, AnIEN);
    except
    end;
end;

function TDBController_Renal.GetStudyStatus(AStudy: string): string;
begin
  if not Assigned(DBConnection) then GetLocal(LocalName(LmStudyStatus, [AStudy]), Result)
  else
    try
      DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['STAT', AStudy], nil);
      Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmStudyStatus, [AStudy]), Result)
    except
      Result := '-1^RPC ' + RPC_BILLING_WIDGET + ' Failed' + #13#10 + 'Option: STAT' + #13#10 + 'Study ID:' + AStudy;
    end;
end;

function TDBController_Renal.GetSensitiveRecordAccessInfo(ADFN: string): string;
begin
  Result := '';
  if not Assigned(DBConnection) then GetLocal(LocalName(LmSensitivAccess, [ADFN]), Result)
  else
    try
      if DBConnection.ExecRemoteProc(RPC_SENSITIVE_RA, [ADFN], nil) then Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmSensitivAccess, [ADFN]), Result)
    except
    end;
end;

function TDBController_Renal.GetSensitiveRecordBulletinInfo(ADFN: string): string;
begin
  Result := '';
  if not Assigned(DBConnection) then GetLocal(LocalName(LmSensitivBulletin, [ADFN]), Result)
  else
    try
      if DBConnection.ExecRemoteProc(RPC_SENSITIVE_RB, [ADFN], nil) then Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmSensitivBulletin, [ADFN]), Result)
    except
    end;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.GetQualifiersByVitalType(ATypeIEN: string): string;
begin
  Result := '';
  if not Assigned(DBConnection) then GetLocal(LocalName(LmVitalsQualifiers, [ATypeIEN]), Result)
  else
    try
      if DBConnection.ExecRemoteProc(RPC_MANAGER, ['GETCATS', ATypeIEN], nil) then
          Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmVitalsQualifiers, [ATypeIEN]), Result)
    except
    end;
end;

function TDBController_Renal.GetQualifiersAll: string;
begin
  Result := '';
  if not Assigned(DBConnection) then GetLocal(LocalName(LmVitalsQualifiersAll, []), Result)
  else
    try
      if DBConnection.ExecRemoteProc(RPC_MANAGER, ['GETLIST', '120.52'], nil) then
          Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmVitalsQualifiersAll, []), Result)
    except
    end;
end;

function TDBController_Renal.GetVitalTypesAll: string;
begin
  Result := '';
  if not Assigned(DBConnection) then GetLocal(LocalName(LmVitalsTypes, []), Result)
  else
    try
      if DBConnection.ExecRemoteProc(RPC_MANAGER, ['GETLIST', '120.51'], nil) then
          Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then SetLocal(LocalName(LmVitalsTypes, []), Result)
    except
    end;
end;

function TDBController_Renal.GetImplementationDate: string; // added 09/24/2012 zzzzzzcarnej
begin
  Result := '';
  if not Assigned(DBConnection) then Result := ''
  else
    try
      if DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['MDICDDT'], nil) then Result := DBConnection.ExecResults.Text
      else begin
        ShowMessage('ICD10 Remediation' + #13#10 + Result + 'Check if the MD1*0P29 was installed');
{$IFDEF RPCLOG}
        Result := '3140311';
        AddEvent('ICD10 Remediation', GetDescribedItem('ICD10 Remediation',
          'MDICDDT not found' + #13#10 + 'Implementation date set to <' + Result + '>'));
{$ENDIF}
      end;
    except
      on E: Exception do ShowMessage('GetImplementationDate ERROR:' + #13#10 + E.Message);
    end;
end;

function TDBController_Renal.GetGatewayStatus: string;
var
  S: string;
begin
  Result := 'Down';
  if not Assigned(DBConnection) then Result := 'Gateway Status:' + 'Demo'
  else
    try
      if not DBConnection.ExecRemoteProc(RPC_GATEWAY, ['STATUS'], nil) then Exit;
      S := DBConnection.ExecResults.Text;
      if Pos('Shutdown Flag^Yes', S) = 0 then Result := 'Up ';
    except
    end;
end;

function TDBController_Renal.GetGatewayRunningStatus: string;
var
  S: string;
begin
  Result := 'Down';
  if not Assigned(DBConnection) then Result := 'Gateway Status:' + 'Demo Running'
  else
    try
      if not DBConnection.ExecRemoteProc(RPC_GATEWAY, ['RUNNING'], nil) then Exit;
      S := DBConnection.ExecResults.Text;
      if Pos('1', S) = 1 then Result := 'Up ';
    except
    end;
end;

function TDBController_Renal.GetGatewayStatusInfo: string;
var
  Ss, C, S: string;
  I: Integer;
begin
  Result := 'OFF';
  if not Assigned(DBConnection) then Result := 'Gateway Status Info:' + #13#10#13#10 + 'Demo'
  else
    try
      if not DBConnection.ExecRemoteProc(RPC_GATEWAY, ['STATUS'], nil) then Exit;
      S := DBConnection.ExecResults.Text;
      Ss := '';
      for I := 1 to Length(S) do begin
        C := Copy(S, I, 1);
        if C = '^' then C := ': ';
        Ss := Ss + C;
      end;
      Result := Ss;
    except
      Result := 'Error processing Gateway status';
    end;
end;

function TDBController_Renal.GetServerCurrentDateTime: string;
begin
  Result := '';
  if not Assigned(DBConnection) then Exit;
  if DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['TIME'], nil) then
    // dummy parameter
      Result := DBConnection.ExecResults.Text
  else Result := '';
  if Pos(#13#10, Result) > 0 then Result := Copy(Result, 1, Length(Result) - 2);
end;

function TDBController_Renal.GetRPCResults(AParamList, ADelim: string): string;
var
  Params: array of string;
  I: Integer;
  SName, S: string;
  Sl: TStringList;
begin
  Result := AParamList;
  Sl := TStringList.Create;
  I := 1;
  repeat
    S := Piece(AParamList, ADelim, I);
    if S = '' then Break;
    if Pos(#13#10, S) > 0 then S := Copy(S, 1, Pos(#13#10, S) - 1);
    Sl.Add(S);
    Inc(I);
  until False;
  SName := '';
  if Sl.Count > 0 then SName := Sl[0];
  if Sl.Count > 1 then begin
    SetLength(Params, Sl.Count - 1);
    for I := 1 to Sl.Count - 1 do begin
      Params[I - 1] := Sl[I];
    end;
  end;
  Sl.Free;
  if not Assigned(DBConnection) then Result := AParamList
  else if SName <> '' then begin
    if DBConnection.ExecRemoteProc(SName, Params, nil) then Result := DBConnection.ExecResults.Text
    else Result := AParamList;
  end;
end;

function TDBController_Renal.RegisterAccessToSensitiveData(AnID, AnExeNameAndVersion: string): string;
const
  RPC    = 'GMV PTSELECT';
  OPTION = 'LOGSEC';
begin
  if not Assigned(DBConnection) then begin
    Result := '';
  end
  else
    try
      DBConnection.ExecRemoteProc(RPC, [OPTION, AnID, 'RPCCALL^Clinical Procedure ' + AnExeNameAndVersion], nil);
      Result := DBConnection.ExecResults.Text;
    except
      Result := '-1^RPC ' + RPC + ' Failed' + #13#10 + 'Patient' + AnID;
    end;
end;

function TDBController_Renal.IsDataSensitive(APatientID: string; var SSecurityWarning: string): Boolean;
var
  S: string;
const
  RPC              = 'GMV PTSELECT';
  OPTION           = 'SELECT';
  SECURITY_HEADER3 = '$$MSGHDR^3';
  SECURITY_HEADER0 = '$$MSGHDR^0';
begin
  Result := False;
  if not Assigned(DBConnection) then begin
    S := GetItemLocal(LocalName(LmSensitivItem, []), APatientID);
    SSecurityWarning := 'Security Warning';
    Result := S = 'TRUE';
    Exit;
  end
  else
    try
      if DBConnection.ExecRemoteProc(RPC, [OPTION, APatientID], nil) then begin
        S := DBConnection.ExecResults.Text;
        Result := (Pos(SECURITY_HEADER3, S) <> 0) or (Pos(SECURITY_HEADER0, S) <> 0);
        if Result then begin
          SSecurityWarning := Copy(S, Pos('This record is', S), Length(S));
          SSecurityWarning := Piece(SSecurityWarning, '$$', 1);
        end;
      end else begin
        Result := True;
        SSecurityWarning := 'Failed to retreive the security information';
      end;
    except
    end;
end;

function TDBController_Renal.GetOptionListXML(AUserDUZ: string = '.5'): string;
begin
  Result := '';
  if not Assigned(DBConnection) then Result := ''
  else
    try
      if DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['ALLOPT', '', AUserDUZ], nil) then
          Result := DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.SetStudyStatus(AStudyID, AStatus: string): string;
(*
  The RPC is MD TMDWIDGET.

  D RPC^MDRPCOW(.RESULTS,OPTION,P1,P2)

  Results  Return global = 1^Done
  OPTION  STATUS
  P1  CP Study number
  P2- Status number

  0:New
  1:Submitted
  2:Error
  3:Complete
  4:Ready to Complete
  5:Pending Instrument Data
  6:Cancelled
*)
begin
  Result := '';
  if not Assigned(DBConnection) then Result := ''
  else
    try
      if DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['STATUS', AStudyID, AStatus], nil) then
          Result := DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.GetPCEInfo(APatientID, AStudyID: string): string;
begin
  if not Assigned(DBConnection) then Result := 'DB Connection is not assigned'
  else
    try
      (*
        MD TMDCIDC

        OPTION=PCEDISP
        DFN=PATIENT #
        MDSTUD=STUDY #

        >D RPC^MDRPCW(.RESULTS,"PCEDISP",400,3243)

        >D ^%G

        Global ^TMP("MDRPCW",$J -- NOTE: translation in effect
        ^TMP("MDRPCW",540310399,1)=Visit #:
        2)=Encounter Date/Time: 11/21/06
        3)=Visit # For Encounter Date:
        4)=Previous Visit #: 6481 11/20/06
        5)=Diagnosis: END STAGE RENAL DISEASE Primary
        6)=CPT: PERITON DIALYSIS PLAN DOCD-1
        7)=CPT: ESRD RELATED SERVICES, MONTH-7
      *)
      if DBConnection.ExecRemoteProc(RPC_CLINIC_INFO, ['PCEDISP', APatientID, AStudyID], nil) then
          Result := DBConnection.ExecResults.Text
      else Result := 'RPC Failed.' + #13#10 + DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.GetUserOptionList(AUser: string): string;
var
  I: Integer;
  SL: TStringList;
const
  Kw = '<SETTING_NAME>';
begin
  Result := '';
  if not Assigned(DBConnection) then Exit;
  try
    if DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['ALLOPT', '', '.5'], nil) then begin
      SL := TStringList.Create;
      SL.Text := DBConnection.ExecResults.Text;
      I := 0;
      while I < SL.Count do begin
        if Pos(Kw, Sl[I]) <> 1 then Sl.Delete(I)
        else begin
          Sl[I] := Piece(Piece(SL[I], Kw, 2), '<', 1);
          if Trim(Sl[I]) <> '' then Inc(I)
          else Sl.Delete(I);
        end;
      end;
      Result := SL.Text;
      SL.Free;
    end;
  except
  end;
end;
{
  Here is the RPC call for CP Titles: MD TMDRECORDID.
  Input: RESULTS  Return global array.

  OPTION: PTRLKUP
  DD: 702.01
  IENS: null
  FLD:.04
  DATA: What user typed in for query.

  Sample run below:

  D RPC^MDRPCOR(.RESULTS,"PTRLKUP",702.01,"",.04,"CP")

  DEV>ZW RESULTS
  RESULTS="^TMP(539038553)"
  DEV>D ^%G
  Global ^TMP($J -- NOTE: translation in effect
  ^TMP(539038553,0)=10
  1)=8925.1;1256^CP COLONOSCOPY
  2)=8925.1;1264^CP EKG
  3)=8925.1;746^CP ENDOSCOPY
  4)=8925.1;753^CP HEMO FALLS ASSESSMENT
  5)=8925.1;754^CP HEMO PROVIDER NOTE
  6)=8925.1;752^CP HEMODIALYSIS
  7)=8925.1;642^CP MUSE EKG
  8)=8925.1;609^CP PFT
  9)=8925.1;610^CP SBK PFT
  10)=8925.1;1271^CP_DEMO
}

function TDBController_Renal.GetCPTIUTitles(ATarget: string): string;
var
  S: string;
  I: Integer;
  SL: TStringList;
begin
  Result := '';
  if not Assigned(DBConnection) then begin
    GetLocal(LocalName(LmTIUTitles, []), Result);
    Exit;
  end;
  try
    if DBConnection.ExecRemoteProc(RPC_TIU_TITLES, ['PTRLKUP', '702.01', '', '.04', ATarget { 'CP' } ], nil) then begin
      SL := TStringList.Create;
      SL.Text := DBConnection.ExecResults.Text;
      I := 0;
      while I < SL.Count do begin
        // s := piece(SL[i],'^',2);
        S := SL[I];
        if Trim(S) = '' then SL.Delete(I)
        else begin
          SL[I] := S;
          Inc(I);
        end;
      end;
      Result := SL.Text;
      if SaveRPCResults then SetLocal(LocalName(LmTIUTitles, []), Result);
      SL.Free;
    end;
  except
  end;
end;
(*
  Here is the new RPC in MD TMDCIDC.  Use the RPC to grey out the billing Update buttons when the Summary tab is opened.

  RESULTS = Return result global
  OPTION =TIMEMET
  DFN = Patient IEN
  MDSTUD = Study number

  D RPC^MDRPCW(.RESULTS,"TIMEMET",305,3211)

  DEV>D ^%G

  Global ^TMP("MDRPCW",$J -- NOTE: translation in effect

  ^TMP("MDRPCW",539116008,0)=1^Appointment/Visit Date/Time have met.

  Other Possible results are:

  -1^No Visit Date/Time.
  -1^No Visit.
  0^Appointment/Visit Date/Time not met.

*)

function TDBController_Renal.GetAppintmentInfo(APatientID, AStudyID: string): string;
begin
  Result := '';
  if not Assigned(DBConnection) then begin
    Result := '1^Appointment/Visit Date/Time have met';
    Exit;
  end;
  try
    if DBConnection.ExecRemoteProc(RPC_CLINIC_INFO, ['TIMEMET', APatientID, AStudyID], nil) then begin
      Result := DBConnection.ExecResults.Text;
      // if SaveRPCResults then
      // setLocal(LocalName(lmTIUTitles, []), Result);
    end;
  except
  end;
end;

function TDBController_Renal.GetVersionInfo(AGUIVersion: string): string;
begin
  Result := '';
  if not Assigned(DBConnection) then begin
    Result := 'YES';
    Exit;
  end;
  try
    if DBConnection.ExecRemoteProc(RPC_VERSION_INFO, ['GUI', AGUIVersion], nil) then
        Result := DBConnection.ExecResults[0];
  except
  end;
end;

function TDBController_Renal.GetSupportedVersion: string;
begin
  Result := '<Unknown>';
  if not Assigned(DBConnection) then begin
    // ?
    Exit;
  end;
  try
    if DBConnection.ExecRemoteProc(RPC_VERSION_INFO, ['CURRENT', '*'], nil) then Result := DBConnection.ExecResults[0];
  except
  end;
end;

function TDBController_Renal.GetPhysicianOrders(APatient, AnOrderIEN: string): string;
begin
  Result := '';
  try
    if DBConnection.ExecRemoteProc(RPC_PHYSICIAN_ORDERS, ['ORDERS', APatient, AnOrderIEN], nil) then
        Result := Result + DBConnection.ExecResults.Text;
  except
  end;
end;

function TDBController_Renal.SetStudyUseStatus(AStudyID, AStatusID, AStatusBy, AStatusAt, AWS: string): string;
begin
  if not Assigned(DBConnection) then begin
    SetLocalStatus(AStudyID, AStatusID, AStatusBy, AStatusAt, AWS);
    Result := 'OK';
    Exit;
  end;
  try
    DBConnection.ExecRemoteProc('MDK GET/SET RENAL DATA', ['UPDUSER', AStudyID, AStatusBy, AStatusAt, AWS,
      AStatusID], nil);
    Result := DBConnection.ExecResults[0];
  except
  end;
end;

function TDBController_Renal.GetStudyUseStatus(SStudyID: string;
  var SStatus, SStatusBy, SStatusAt, SWS: string): Boolean;
begin
  if not Assigned(DBConnection) then begin
    Result := GetLocalStatus(SStudyID, SStatus, SStatusBy, SStatusAt, SWS);
    Exit;
  end
  else raise Exception.Create('Undefined getStudyUseStatus');
end;

function TDBController_Renal.GetLockStatus(AStudyID: Integer): string;
var
  S: string;
begin
  Result := 'Unknown';
  if not Assigned(DBConnection) then begin
    if GetLocalLock(IntToStr(AStudyID), S) then Result := S;
  end else begin
    if LockStudy(AStudyID) then begin
      Result := 'ENABLED';
      UnLockStudy(AStudyID);
    end
    else Result := 'IN USE';
  end;
end;

function TDBController_Renal.GetLockedStudies: string;
begin
  Result := '';
  if not Assigned(DBConnection) then Exit
  else
    try
      if DBConnection.ExecRemoteProc('MDK UTILITY', ['INUSE'], nil) then Result := DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.GetUserInfo: string;
begin
  Result := '';
  if not Assigned(DBConnection) then Exit
  else
    try
      if DBConnection.ExecRemoteProc('MDK GET/SET RENAL DATA', ['USERINF'], nil) then
          Result := DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.GetXPar(AName: string): string;
begin
  Result := '-1';
  if not Assigned(DBConnection) then Exit
  else
    try
      if DBConnection.ExecRemoteProc('MDK GET VISTA DATA', ['GETLD'], nil) then Result := DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.SetXPar(AName, AValue: string): string;
begin
  Result := '-1';
  if not Assigned(DBConnection) then Exit
  else
    try
      if DBConnection.ExecRemoteProc('MDK GET VISTA DATA', ['SETLD', AValue], nil) then
          Result := DBConnection.ExecResults.Text;
    except
    end;
end;

initialization

VerifyWriteIsComplete := False;

end.
