unit fMDK_FlowSheetEdit;
{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 12/03/07 1:06p $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  Contains the Application Flowsheet Editor form
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Forms/fMDK_FlowSheetEdit.pas $
  *
  *       $ History: fMDK_FlowSheetEdit.pas $
  *
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, StdCtrls, Grids, ValEdit, DBClient, DB, Buttons, ExtCtrls, System.Actions;

type
  TfrmFlowSheetEdit = class(TForm)
    ActionList1: TActionList;
    AcSave: TAction;
    Panel1: TPanel;
    Panel2: TPanel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    GroupBox1: TGroupBox;
    VleFlow: TValueListEditor;
    procedure VleFlowEditButtonClick(Sender: TObject);
    procedure VleFlowDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure FormActivate(Sender: TObject);
    procedure VleFlowGetEditMask(Sender: TObject; ACol, ARow: Integer; var Value: string);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure BitBtn2Click(Sender: TObject);
  private
    FIgnore: Boolean;
    procedure PrepareList(SL: TStringList; AnInsert: Boolean = False);
    procedure PrepareListByCDS(ACDS: TClientDataSet; AnInsert: Boolean = False);
    function ValidRecord: Boolean;
  public
    { Public declarations }
  end;

var
  FrmFlowSheetEdit: TfrmFlowSheetEdit;

procedure EditFSRecord(ACDS: TClientDataSet; APatient, AStudy, ASource, AStatus: string);
procedure AddFSRecord(ACDS: TClientDataSet; APatient, AStudy, ASource, AStatus: string);

implementation

uses
  UMDK_Common, UROR_CommonUtils, UMDK_CDSCommon, System.UITypes, UMDK_Engine, UMDK_Options, UMDK_Flowsheet,
  UMDK_Dictionary, UMDK_Vitals;

{$R *.dfm}

function Now: TdateTime;
begin
  try
    Result := Engine.GetCurrentServerDateTime;
  except
    Result := Now;
  end;
end;

procedure SetUpCDSBySL(ACDS: TClientDataSet; SL: TStrings; APatient, AStudy, ASource, AStatus: string;
  AnInsert: Boolean = False);
var
  SLabel, SValue: string;
  I, JPos: Integer;
begin
  if AnInsert then ACDS.Insert
  else ACDS.Edit;
  ACDS.FieldByName(FnFS_PatientID).AsString := APatient;
  ACDS.FieldByName(FnFS_SessionID).AsString := AStudy;
  ACDS.FieldByName(FnFS_Source).AsString := ASource;
  ACDS.FieldByName(FnFS_Status).AsString := AStatus;
{$IFDEF IGNORE_DUPLICATES}
  ACDS.FieldByName(FnFS_Duplicate).AsString := 'False';
{$ENDIF}
  for I := 0 to ACDS.Fields.Count - 1 do begin
    if not ACDS.Fields[I].Visible then Continue;
    try
      SLabel := ACDS.Fields[I].DisplayLabel;
      JPos := SL.IndexOfName(SLabel);
      if JPos >= 0 then begin
        SValue := SL.Names[JPos];
        ACDS.FieldByName(ACDS.FieldDefs[I].Name).AsString := SL.Values[SValue];
      end;
    except
    end;
  end;
  ACDS.Post;
end;

procedure EditFSRecord(ACDS: TClientDataSet; APatient, AStudy, ASource, AStatus: string);
begin
  if not Assigned(FrmFlowSheetEdit) then FrmFlowSheetEdit := TfrmFlowSheetEdit.Create(Application);
  with FrmFlowSheetEdit do begin
    FIgnore := False;
    PrepareListByCDS(ACDS);
    if ShowModal <> MrCancel then SetUpCDSBySL(ACDS, VleFlow.Strings, APatient, AStudy, ASource, AStatus, False);
  end;
end;

procedure AddFSRecord(ACDS: TClientDataSet; APatient, AStudy, ASource, AStatus: string);
begin
  if not Assigned(FrmFlowSheetEdit) then FrmFlowSheetEdit := TfrmFlowSheetEdit.Create(Application);
  with FrmFlowSheetEdit do begin
    FIgnore := False;
    PrepareListByCDS(ACDS, True);
    if ShowModal <> MrCancel then SetUpCDSBySL(ACDS, VleFlow.Strings, APatient, AStudy, ASource, AStatus, True);
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmFlowSheetEdit.PrepareList(SL: TStringList; AnInsert: Boolean = False);
var
  I: Integer;
  Dt: TdateTime;
begin
  VleFlow.Strings.Assign(SL);
  Dt := Engine.GetCurrentServerDateTime;
  if not AnInsert then Exit;
  for I := 0 to VleFlow.Strings.Count - 1 do begin
    if VleFlow.Strings.Names[I] = 'Date' then VleFlow.Strings[I] := 'Date=' + FormatDateTime(GMDK_DateFmt, Dt);
    if VleFlow.Strings.Names[I] = 'Time' then VleFlow.Strings[I] := 'Time=' + FormatDateTime(GMDK_TimeFmt, Dt);
  end;
end;

procedure TfrmFlowSheetEdit.PrepareListByCDS(ACDS: TClientDataSet; AnInsert: Boolean = False);
var
  IStart, I: Integer;
  SL: TStringList;
begin
  SL := TStringList.Create;
  IStart := 0;
  for I := IStart to ACDS.Fields.Count - 1 do // patch, sorry
  begin
{$IFDEF FLOWEVENTS}
    if ACDS.Fields[I].FieldName = FnFS_Event then Continue;
    if ACDS.Fields[I].FieldName = FnFS_Comments then Continue;
{$ENDIF}
{$IFDEF IGNORE_DUPLICATES}
    if ACDS.Fields[I].FieldName = FnFS_Duplicate then Continue;
{$ENDIF}
    // if not aCDS.Fields[i].Visible then continue;
    // Date and Time should be added always regardless of visibility
    if ACDS.Fields[I].DisplayLabel = 'Time' then begin
      if AnInsert then SL.Add('Time=' + FormatDateTime(GMDK_TimeFmt, Now))
      else SL.Add('Time=' + FormatDateTime(GMDK_TimeFmt, ACDS.Fields[I].AsDateTime));
    end else if ACDS.Fields[I].DisplayLabel = 'Date' then begin
      if AnInsert then SL.Add('Date=' + FormatDateTime(GMDK_DateFmt, Now))
      else SL.Add('Date=' + FormatDateTime(GMDK_DateFmt, ACDS.Fields[I].AsDateTime));
    end else if ACDS.Fields[I].FieldName = FnFS_SessionID then Continue
    else if ACDS.Fields[I].FieldName = FnFS_PatientID then Continue
    else if ACDS.Fields[I].FieldName = FnFS_Source then Continue
    else if not ACDS.Fields[I].Visible then Continue
    else begin
      if AnInsert then SL.Add(ACDS.Fields[I].DisplayLabel + '=')
      else SL.Add(ACDS.Fields[I].DisplayLabel + '=' + ACDS.Fields[I].AsString);
    end;
  end;
  PrepareList(SL);
  SL.Free;
end;

procedure TfrmFlowSheetEdit.VleFlowEditButtonClick(Sender: TObject);
begin
  ShowMessage('Custom edit window for the parameter:' + #13#13 + '                    ' + VleFlow.Strings
    [VleFlow.Row - 1]);
  // Header adjustment
end;

procedure TfrmFlowSheetEdit.VleFlowDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  with VleFlow.Canvas do begin
    if (ARow = 0) then begin
      Brush.Color := ClBtnFace;
      Font.Style := [FsBold];
    end else if ACol <> 0 then begin
      Brush.Color := ClWindow;
      Font.Color := ClWindowText;
      Font.Style := [];
    end else begin
      Brush.Color := ClBtnFace;
      // Brush.Color := clWindow;
      // Font.Style := [fsBold];
    end;
    FillRect(Rect);
    TextOut(Rect.Left + 2, Rect.Top + 2, VleFlow.Cells[ACol, ARow]);
  end;
end;

procedure TfrmFlowSheetEdit.FormActivate(Sender: TObject);
begin
  ActiveControl := VleFlow;
end;

procedure TfrmFlowSheetEdit.VleFlowGetEditMask(Sender: TObject; ACol, ARow: Integer; var Value: string);
begin
  if VleFlow.Cells[0, ARow] = 'Time' then Value := DevOptions.SValue(DonFlowsheetTimeInputMask)
  else if VleFlow.Cells[0, ARow] = 'Date' then Value := DevOptions.SValue(DonFlowsheetDateInputMask)
  else Value := GetDictionaryEditMaskByDisplayLabel('cdsFlowSheet', VleFlow.Cells[0, ARow]);
end;

procedure TfrmFlowSheetEdit.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if not FIgnore then CanClose := ValidRecord;
end;

function TfrmFlowSheetEdit.ValidRecord: Boolean;
var
  T: TdateTime;
  SMessage, SName, SValue: string;
  I: Integer;

  function ValidDate(AValue: string): string;
  begin
    Result := '';
    try
      T := StrToDate(AValue);
    except
      on E: EConvertError do Result := 'Wrong Date format: <' + AValue + '> (should be ' + GMDK_DateFmt + ')' + #13#10;
    end;
  end;

  function ValidTime(AValue: string): string;
  begin
    Result := '';
    try
      T := StrToTime(AValue);
    except
      on E: EConvertError do Result := 'Wrong time format: <' + AValue + '> ( should be ' + GMDK_TimeFmt + ')' + #13#10;
    end;
  end;

  function ValidBPressure(AValue: string): string;
  var
    _s: string;
    FLow, FHigh: Double;
  begin
    Result := '';
    if AValue = '' then begin
      _s := GetDictionaryFieldByDisplayLabel('cdsFlowSheet', 'B/P', FnDict_Required);
      if ( { (_s = '') or } (Uppercase(_s) = 'TRUE')) and (Trim(AValue) = '') then
          Result := 'Entry value for required field B/P is not provided' + #13
    end else if not ValidBP(AValue) then begin
      FLow := GMVVitalLoRange[VtBP];
      FHigh := GMVVitalHiRange[VtBP];
      Result := 'Invalid format for B/P (' + AValue + ')' + #13 + 'The entry should be in the following format:' + #13 +
        'nnn/nnn  or nnn/nnn/nnn' + #13 + 'with values from  ' + FloatToStr(FLow) + ' to ' + FloatToStr(FHigh) + #13;
    end;
  end;

  function ValidField(AName, AValue: string): string;
  var
    // _Value: Double;
    _Value: Extended; // b. 312 #15
    SMin, SMax: string;
  begin
    Result := '';
    if (Trim(AValue) = '') then begin
      SMin := GetDictionaryFieldByDisplayLabel('cdsFlowSheet', AName, FnDict_Required);
      if ( { (sMin = '') or } (Uppercase(SMin) = 'TRUE')) then
          Result := 'Entry value for required field ' + AName + ' is not provided' + #13
    end else begin
      SMin := GetDictionaryFieldByDisplayLabel('cdsFlowSheet', AName, FnDict_MinValue);
      SMax := GetDictionaryFieldByDisplayLabel('cdsFlowSheet', AName, FnDict_MaxValue);
      try
        _Value := StrToFloat(AValue);
        if (SMin <> '') and (_Value < StrToFloat(SMin)) then
            Result := 'Entry Value for ' + AName + ' field (' + AValue + ') is below the valid minimum of ' +
            SMin + #13;
        // if (sMax <> '') and (_Value > StrToFloat(sMax)) then
        if (SMax <> '') and (StrToFloat(SMax) < _Value) then
            Result := 'Value for ' + AName + ' field (' + AValue + ') exceeds the maximum of ' + SMax + #13;
      except
        on E: EConvertError do Result := 'Invalid integer of ' + AName + ' field (' + AValue + ')' + #13;
      end;
    end;
  end;

begin
  SMessage := '';
  for I := 0 to VleFlow.RowCount - 2 do begin
    SName := VleFlow.Strings.Names[I];
    SValue := VleFlow.Strings.Values[SName];
    if SName = 'Date' then SMessage := SMessage + ValidDate(SValue)
    else if SName = 'Time' then SMessage := SMessage + ValidTime(SValue)
    else if (Pos('B/P', SName) > 0) then SMessage := SMessage + ValidBPressure(SValue)
    else SMessage := SMessage + ValidField(SName, SValue);
  end;
  Result := False;
  if SMessage <> '' then
      MessageDlg('Sorry, one or more of the input fields contains errors:' + #13#10#13#10 + SMessage + #13#10#13#10 +
      'Please correct the value.', MtError, [MbOK], 0)
  else Result := True;
end;

procedure TfrmFlowSheetEdit.BitBtn2Click(Sender: TObject);
begin
  FIgnore := True;
  Close;
end;

end.
