unit fMDK_GeneralInfoDLG;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, DBCtrls, Mask, ActnList, System.Actions;

type
  TfrmGeneralInfo = class(TForm)
    GbObservations: TGroupBox;
    Panel69: TPanel;
    Panel1: TPanel;
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    BitBtn1: TBitBtn;
    PnlTreatmentInfo: TPanel;
    PnlInfo: TPanel;
    Label18: TLabel;
    Label33: TLabel;
    Label34: TLabel;
    Label35: TLabel;
    Label36: TLabel;
    Label38: TLabel;
    Label40: TLabel;
    DbeNephrologist: TDBEdit;
    DbcCodeStatus: TDBComboBox;
    DbcModality: TDBComboBox;
    DbeInitialTherapy: TDBEdit;
    DbeDiagnosisDate: TDBEdit;
    DbcESRD: TDBComboBox;
    DbeCTDate: TDBEdit;
    BtnSelectProvider: TBitBtn;
    BbDiagnosisDate: TBitBtn;
    BbInitialTherapy: TBitBtn;
    Panel2: TPanel;
    BbTreatmentDate: TBitBtn;
    PnlSchedule: TPanel;
    Bevel11: TBevel;
    Label17: TLabel;
    DbcbMon: TDBCheckBox;
    DbcbTue: TDBCheckBox;
    DbcbWed: TDBCheckBox;
    DbcbTh: TDBCheckBox;
    DbcbFri: TDBCheckBox;
    DbcbSat: TDBCheckBox;
    DbcbSun: TDBCheckBox;
    PnlTransplantStatus: TPanel;
    Bevel1: TBevel;
    Label5: TLabel;
    DbcWIP: TDBCheckBox;
    DbcReferred: TDBCheckBox;
    RbNonCandidate: TRadioButton;
    RbCandidate: TRadioButton;
    ActionList1: TActionList;
    AcTherapyDate: TAction;
    AcDiagnosisDate: TAction;
    AcTreatmentDate: TAction;
    Panel3: TPanel;
    AcAttendingNephrologist: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AcTherapyDateExecute(Sender: TObject);
    procedure AcDiagnosisDateExecute(Sender: TObject);
    procedure AcTreatmentDateExecute(Sender: TObject);
    procedure DbeDiagnosisDateChange(Sender: TObject);
    procedure RbCandidateClick(Sender: TObject);
    procedure AcAttendingNephrologistExecute(Sender: TObject);
  private
    Originals: TStringList;
    FIgnore: Boolean;
    procedure SetUp;
    procedure SaveOriginals;
    procedure RestoreOriginals;
    procedure ReloadData;
  public
    { Public declarations }
  end;

var
  FrmGeneralInfo: TfrmGeneralInfo;

function UpdateGeneralInfoData: Integer;

implementation

uses
  UMDK_Summary, UMDK_Info, DMDK_Data, FMDK_DateTimeSelector, DB,
  UMDK_Engine, UMDK_Utils, UMDK_Common, FMDK_SelectProvider;

{$R *.dfm}

function UpdateGeneralInfoData: Integer;
begin
  if not Assigned(FrmGeneralInfo) then Application.CreateForm(TfrmGeneralInfo, FrmGeneralInfo);
  FrmGeneralInfo.SetUp;
  FrmGeneralInfo.SaveOriginals;
  Result := FrmGeneralInfo.ShowModal;
  if Result = MrCancel then FrmGeneralInfo.RestoreOriginals
  else
    try
      if DmData.CdsSummary.State = DsEdit then DmData.CdsSummary.Post;
      if DmData.CdsInfo.State = DsEdit then DmData.CdsInfo.Post;
    except
    end;
  FreeAndNil(FrmGeneralInfo);
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmGeneralInfo.FormCreate(Sender: TObject);
begin
  inherited;
  DbeCTDate.DataField := FnI_StudyDate;
  DbcESRD.DataField := FnI_Diagnosis;
  DbeDiagnosisDate.DataField := FnI_DiagnosisDate;
  DbeInitialTherapy.DataField := FnI_TherapyDate;
  DbcModality.DataField := FnI_Modality;
  DbcCodeStatus.DataField := FnI_CodeStatus;
  DbeNephrologist.DataField := FnI_Nephrologist;
  DbcbMon.DataField := FnI_Mon;
  DbcbTue.DataField := FnI_Tue;
  DbcbWed.DataField := FnI_Wed;
  DbcbTh.DataField := FnI_Thu;
  DbcbFri.DataField := FnI_Fri;
  DbcbSat.DataField := FnI_Sat;
  DbcbSun.DataField := FnI_sun;
  // Transplant Status /////////////////////////////////////////////////////////
  DbcWIP.DataField := FnS_TrnWIP;
  DbcReferred.DataField := FnS_TrnReferred;
  ReloadData;
  Originals := TStringList.Create;
end;

procedure TfrmGeneralInfo.FormDestroy(Sender: TObject);
begin
  FreeAndNil(Originals);
  inherited;
end;

procedure TfrmGeneralInfo.SetUp;

  function GetValues(AnOptionName: string): string;
  var
    Sl: TStringList;
    I: Integer;
  begin
    Sl := nil;
    try
      Result := '';
      Sl := TStringList.Create;
      Sl.Text := Engine.GetSystemOptionByName(AnOptionName);
      for I := 0 to Sl.Count - 1 do Result := Result + Piece(Sl[I], '=', 2) + #13#10;
    finally
      Sl.Free;
    end;
  end;

  procedure UpdateField(AField: TDBComboBox; SText: string);
  var
    SOldValue: string;
    I: Integer;
  begin
    try
      SOldValue := DmData.CdsInfo.FieldByName(AField.DataField).AsString;
      // aField.Text;
      AField.Items.Text := SText;
      if SOldValue <> '' then
        try
          I := AField.Items.IndexOf(SOldValue);
          if I > -1 then AField.ItemIndex := I;
        except
          on E: Exception do ShowMessage(E.Message);
        end;
    except
    end;
  end;

begin
  if not DmData.CdsInfo.Active then DmData.CdsInfo.Active := True;
  FIgnore := True;
  UpdateField(DbcESRD, GetValues(OnESRD_Diagnosys));
  UpdateField(DbcModality, GetValues(OnModalities));
  UpdateField(DbcCodeStatus, GetValues(OnCodeStatuses));
  FIgnore := False;
end;

procedure TfrmGeneralInfo.AcTherapyDateExecute(Sender: TObject);
var
  SDate: string;
begin
  SDate := DbeInitialTherapy.Text;
  if GetDateTime('Select Initial Therapy Date', 'mm/dd/yyyy', SDate) = MrOK then
    try
      // fIgnore := True;
      if DmData.CdsInfo.State <> DsEdit then DmData.CdsInfo.Edit;
      DbeInitialTherapy.Text := SDate;
      DmData.CdsInfo.Post;
      // fIgnore := False;
    except
      on E: Exception do ShowMessage(E.Message);
    end;
end;

procedure TfrmGeneralInfo.AcDiagnosisDateExecute(Sender: TObject);
var
  SDate: string;
begin
  SDate := DbeDiagnosisDate.Text;
  if GetDateTime('Select Diagnosis Date', 'mm/dd/yyyy', SDate) = MrOK then
    try
      // fIgnore := True;
      if DmData.CdsInfo.State <> DsEdit then DmData.CdsInfo.Edit;
      DbeDiagnosisDate.Text := SDate;
      DmData.CdsInfo.Post;
      // fIgnore := False;
    except
      on E: Exception do ShowMessage(E.Message);
    end;
end;

procedure TfrmGeneralInfo.AcTreatmentDateExecute(Sender: TObject);
var
  SDate: string;
begin
  SDate := DbeCTDate.Text;
  if GetDateTime('Select Treatment Date', 'mm/dd/yyyy', SDate) = MrOK then
    try
      if DmData.CdsInfo.State <> DsEdit then DmData.CdsInfo.Edit;
      DbeCTDate.Text := SDate;
      DmData.CdsInfo.Post;
    except
      on E: Exception do ShowMessage(E.Message);
    end;
end;

procedure TfrmGeneralInfo.SaveOriginals;

  procedure SaveItem(AName: string);
  begin
    try
      Originals.Add(AName + '=' + DmData.CdsInfo.FieldByName(AName).AsString);
    except
    end;
  end;

begin
  Originals.Clear;
  SaveItem(FnI_StudyDate);
  SaveItem(FnI_Diagnosis);
  SaveItem(FnI_DiagnosisDate);
  SaveItem(FnI_TherapyDate);
  SaveItem(FnI_Modality);
  SaveItem(FnI_CodeStatus);
  SaveItem(FnI_Nephrologist);
  SaveItem(FnI_Mon);
  SaveItem(FnI_Tue);
  SaveItem(FnI_Wed);
  SaveItem(FnI_Thu);
  SaveItem(FnI_Fri);
  SaveItem(FnI_Sat);
  SaveItem(FnI_Sun);
  SaveItem(FnS_TrnWIP);
  SaveItem(FnS_TrnReferred);
end;

procedure TfrmGeneralInfo.RestoreOriginals;
var
  SValue: string;

  procedure RestoreItem(AName: string);
  begin
    try
      SValue := Originals.Values[AName];
      DmData.CdsInfo.FieldByName(AName).AsString := SValue;
    except
    end;
  end;

begin
  with DmData.CdsSummary do begin
    if State <> DsEdit then Edit;
    RestoreItem(FnI_StudyDate);
    RestoreItem(FnI_Diagnosis);
    RestoreItem(FnI_DiagnosisDate);
    RestoreItem(FnI_TherapyDate);
    RestoreItem(FnI_Modality);
    RestoreItem(FnI_CodeStatus);
    RestoreItem(FnI_Nephrologist);
    RestoreItem(FnI_Mon);
    RestoreItem(FnI_Tue);
    RestoreItem(FnI_Wed);
    RestoreItem(FnI_Thu);
    RestoreItem(FnI_Fri);
    RestoreItem(FnI_Sat);
    RestoreItem(FnI_Sun);
    RestoreItem(FnS_TrnWIP);
    RestoreItem(FnS_TrnReferred);
    Post;
  end;
end;

procedure TfrmGeneralInfo.DbeDiagnosisDateChange(Sender: TObject);
begin
  if TDBEdit(Sender).Field.AsString = '12/30/1899' then TDBEdit(Sender).Text := '';
end;

procedure TfrmGeneralInfo.RbCandidateClick(Sender: TObject);
begin
  if FIgnore then Exit;
  try
    DmData.CdsSummary.Edit;
    DmData.CdsSummary.FieldByName(FnS_TrnCandidate).AsBoolean := RbCandidate.Checked;
    if not RbCandidate.Checked then begin
      DmData.CdsSummary.FieldByName(FnS_TrnWIP).AsBoolean := False;
      DmData.CdsSummary.FieldByName(FnS_TrnReferred).AsBoolean := False;
    end;
    DmData.CdsSummary.Post;
  except
  end;
  DbcWIP.Enabled := RbCandidate.Checked;
  DbcReferred.Enabled := RbCandidate.Checked;
end;

procedure TfrmGeneralInfo.ReloadData;
begin
  try
    RbCandidate.Checked := DmData.CdsSummary.FieldByName(FnS_TrnCandidate).AsBoolean;
    RbNonCandidate.Checked := not DmData.CdsSummary.FieldByName(FnS_TrnCandidate).AsBoolean;
  except
  end;
end;

procedure TfrmGeneralInfo.AcAttendingNephrologistExecute(Sender: TObject);
var
  S: string;
begin
  S := SelectProvider('', Engine.FRenalController, 5);
  S := Piece(S, '^', 1);
  if S <> '' then begin
    try
      if DmData.CdsInfo.State <> DsEdit then DmData.CdsInfo.Edit;
      DbeNephrologist.Text := S;
      DmData.CdsInfo.Post;
    except
      on E: Exception do ShowMessage(E.Message);
    end;
  end;
end;

end.
