unit fMDK_TIUNote;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 4/17/08 4:00p $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     TIU Note form
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Forms/fMDK_TIUNote.pas $
  *
  *       $ History: fMDK_TIUNote.pas $
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls;

type
  TfrmNote = class(TForm)
    Panel2: TPanel;
    PnlSignature: TPanel;
    BbtnSave: TBitBtn;
    BitBtn3: TBitBtn;
    Panel3: TPanel;
    PnlTiuText: TPanel;
    PnlTIUTemplate: TPanel;
    PnlTiuTitle: TPanel;
    GroupBox1: TGroupBox;
    CbTIUNoteTitles: TComboBox;
    GroupBox2: TGroupBox;
    LbTemplates: TListBox;
    GroupBox3: TGroupBox;
    Panel4: TPanel;
    Panel7: TPanel;
    Label6: TLabel;
    Label1: TLabel;
    EdProvider: TEdit;
    EdSignature: TEdit;
    BtnFallsAssessment: TBitBtn;
    Panel1: TPanel;
    MmNote: TMemo;
    Panel5: TPanel;
    Panel6: TPanel;
    Panel8: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure SbSaveClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure CbTemplatesChange(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure EdSignatureKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure MmNoteChange(Sender: TObject);
    procedure EdSignatureChange(Sender: TObject);
  private
    IAttempt: Integer;
  public
    StudyID, ProviderID: string;
    ProviderName: string;
    NoteID: string;
    NoteText: string;
  end;

var
  FrmNote: TfrmNote;

function AddTIUNote(AStudyID, AUserName, AUserDUZ: string): TModalResult;

implementation

uses
  UMDK_Engine, UROR_CommonUtils, UMDK_Options, UMDK_ReportEngine, System.UITypes,
  FMDK_FallAssessement, UMDK_CDSCommon, UMDK_Common;

{$R *.dfm}

function AddTIUNote(AStudyID, AUserName, AUserDUZ: string): TModalResult;
var
  Dlg: TfrmNote;
begin
  Application.CreateForm(TfrmNote, Dlg);
  Dlg.StudyID := AStudyID;
  Dlg.ProviderID := AUserDUZ;
  Dlg.ProviderName := AUserName;
  Dlg.EdProvider.Text := AUserName;
  Dlg.IAttempt := 0;
  try
    Dlg.ActiveControl := Dlg.MmNote;
  except
  end;
  Result := Dlg.ShowModal;
  FreeAndNil(Dlg);
end;

procedure TfrmNote.FormCreate(Sender: TObject);
var
  S: string;
  Sl: TStringList;
  I: Integer;
begin
  BtnFallsAssessment.Visible := Preferences.BValue(OnFallsAssessmentAsTIU);
  PnlTIUTemplate.Visible := Preferences.BValue(OnShowTIUNoteTemplates);
  MmNote.Text := '';
  EdSignature.Text := '';
  Sl := TStringList.Create;
  try
    Sl.Text := Engine.GetSystemOptionByName(OnNoteTemplates);
    for I := 0 to Sl.Count - 1 do Sl[I] := Sl.Names[I];
    LbTemplates.Items.Text := Sl.Text;
    Sl.Clear;
    Sl.Text := Engine.GetSystemOptionByName(OnTIUNoteTitles);
    I := 0;
    while I < Sl.Count do begin
      S := Sl.Values[Sl.Names[I]];
      if Trim(S) <> '' then begin
        Sl[I] := S;
        Inc(I);
      end
      else Sl.Delete(I);
    end;
    if Sl.Count > 0 then CbTIUNoteTitles.Items.Text := Sl.Text;
  finally
    Sl.Free;
  end;
end;

procedure TfrmNote.SbSaveClick(Sender: TObject);
var
  ASL: TStringList;
  S: string;
  I: Integer;

  function ValidESig: Boolean;
  begin
    Result := Engine.IsESigValid(EdSignature.Text);
    if not Result then
      if (Engine.InvalidESigCount <= 2) then
          MessageDlg('Signature is not valid.' + #13#10 + 'Try again.', MtWarning, [MbOK], 0)
      else begin
        MessageDlg('e-Signature verification failed 3 times' + #13#10 + 'The application will be closed', MtError,
          [MbOK], 0);
        ModalResult := MrCancel;
      end;
  end;

begin
  if Trim(CbTIUNoteTitles.Text) = '' then begin
    I := MessageDlg('TIU Note title is not assigned' + #13 + 'Do you want to save the note without title?', MtWarning,
      [MbYes, MbNo, MbCancel], 0);
    case I of
      MrNo: Exit;
      MrCancel: begin
          ModalResult := MrCancel;
          Exit;
        end;
    end;
  end;
  if not ValidESig then Exit;
  ASL := TStringList.Create;
  ASL.Text := MmNote.Lines.Text;
  S := Engine.SubmitNote(ProviderID, EdSignature.Text, ASL, CbTIUNoteTitles.Text);
  ASL.Free;
  if S <> '' then begin
    if Pos('-', S) <> 1 then begin
      NoteText := MmNote.Lines.Text;
      NoteID := S;
      ModalResult := MrOK;
    end else begin
      MessageDlg(Piece(S, '^', 2), MtWarning, [MbOK], 0);
      if Pos('incorrect El', S) <> 0 then begin
        if IAttempt > 1 then ModalResult := MrCancel;
        EdSignature.SelStart := 0;
        EdSignature.SelLength := Length(EdSignature.Text);
        ActiveControl := EdSignature;
        Inc(IAttempt);
      end;
    end;
  end else begin
    S := DateTimeToStr(Now);
    NoteText := MmNote.Lines.Text;
    NoteID := S;
  end;
end;

procedure TfrmNote.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_Escape then ModalResult := MrCancel;
end;

procedure TfrmNote.CbTemplatesChange(Sender: TObject);
var
  STemplateName: string;
begin
  if Sender = LbTemplates then
    try
      if LbTemplates.ItemIndex < 0 then MmNote.Text := ''
      else begin
        STemplateName := Engine.GetSystemOptionByName(LbTemplates.Items[LbTemplates.ItemIndex]);
        MmNote.Text := GetReportValue(STemplateName);
      end;
    except
      MmNote.Text := '';
    end;
end;

procedure TfrmNote.SpeedButton1Click(Sender: TObject);
var
  ISelStart: Integer;
  SPre, SPost: string;
begin
  if not Assigned(FallAssessment) then Application.CreateForm(TFallAssessment, FallAssessment);
  FallAssessment.CleanUpForm;
  SPre := '';
  SPost := '';
  with MmNote do
    try
      if SelLength = 0 then begin
        SPre := Copy(Text, 1, SelStart);
        SPost := Copy(Text, SelStart + 1, Length(Text));
      end else begin
        SPre := Copy(Text, 1, SelStart);
        SPost := Copy(Text, SelStart + SelLength + 1, Length(Text));
      end;
    except
    end;
  try
    FallAssessment.EdAge.Text := Engine.CurrentPatient.Age;
    ISelStart := MmNote.SelStart;
    if FallAssessment.ShowModal = MrOK then begin
      MmNote.Text := SPre + FallAssessment.Report.Text + SPost;
      MmNote.SelStart := ISelStart;
      MmNote.SelLength := Length(FallAssessment.Report.Text);
    end;
  except
  end;
end;

procedure TfrmNote.EdSignatureKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_Return then SbSaveClick(nil);
end;

procedure TfrmNote.MmNoteChange(Sender: TObject);
begin
  UpdateMemo(Sender, Engine.Signature);
end;

procedure TfrmNote.EdSignatureChange(Sender: TObject);
begin
  BbtnSave.Enabled := Length(EdSignature.Text) >= DevOptions.IValue(DonMinESigLength);
end;

end.
