unit frMDK_StudyNoteSection;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 3/26/08 7:23a $
  *       Site Name:    Hines OIFO
  *       Date Created: 3/28/07
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     frMDK_StudyNoteSection definition
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_Common.pas $
  *
  *       $ History: uMDK_Common.pas $
  *
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Buttons, StdCtrls, ExtCtrls, ActnList, Grids, System.Actions;

type
  TFindNext    = procedure(Up: Boolean; aSender: TObject) of object;
  TFindCurrent = procedure(aKeyValue: string; aSender: TObject) of object;

  TFrameStudyNoteSection = class(TFrame)
    ActionList1: TActionList;
    acCollapseExpand: TAction;
    pnlCanvas: TPanel;
    pnlDate: TPanel;
    pnlLeft: TPanel;
    Panel3: TPanel;
    pnlStatusDetails: TPanel;
    lblDate: TLabel;
    pnlTypeIcon: TPanel;
    imgMedication: TImage;
    imgNotes: TImage;
    pnlCollapse: TPanel;
    imCollapsed: TImage;
    imExpanded: TImage;
    imgExpandedCurrent: TImage;
    imgCollapsedCurrent: TImage;
    pnlInclude: TPanel;
    pnlTime: TPanel;
    Panel8: TPanel;
    lblTime: TLabel;
    pnlDetails: TPanel;
    lblID: TLabel;
    lblStatus: TLabel;
    lblStatusBy: TLabel;
    lblStatusDate: TLabel;
    pnlSignature: TPanel;
    shpDivider: TShape;
    lblBy: TLabel;
    pnlBody: TPanel;
    shpTop: TShape;
    imgTIU: TImage;
    imgCPReport: TImage;
    imgLock: TImage;
    ckbInclude: TCheckBox;
    lblTag: TLabel;
    acChangeValidStatus: TAction;
    scUpdate: TAction;
    lblShortText: TLabel;
    imgMore: TImage;
    Panel1: TPanel;
    pnlMore: TPanel;
    procedure acCollapseExpandExecute(Sender: TObject);
    procedure edTitleKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure edEnter(Sender: TObject);
    procedure edExit(Sender: TObject);
    procedure edKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure pnlCanvasResize(Sender: TObject);
    procedure FrameEnter(Sender: TObject);
    procedure FrameExit(Sender: TObject);
    procedure pnlCanvasDblClick(Sender: TObject);
    procedure pnlLeftClick(Sender: TObject);
    procedure acChangeValidStatusExecute(Sender: TObject);
    procedure scUpdateExecute(Sender: TObject);
  private
    fCollapsed: Boolean;
    fSelected: Boolean;
    fMaxHeight: Integer;
    fMinHeight: Integer;
    fLinesCount: Integer;
    fID: string;
    fLock: string;
    fText: string;
    fAuthor: string;
    fStatus: string;
    fStatusBy: string;
    fStatusAt: string;
    procedure SetCollapse(aValue: Boolean);
    procedure SetSelect(aValue: Boolean);
    procedure SetLock(aValue: string);
    function getEditWidth: Integer;
  public
    onFindNext: TFindNext;
    onFindCurrent: TFindCurrent;
    property LockStatus: string read fLock write SetLock;
    property Collapsed: Boolean read fCollapsed write SetCollapse;
    property SectionSelected: Boolean read fSelected write SetSelect;
    property Text: string read fText;
    property Author: string read fAuthor;
    property ID: string read fID;
    procedure SetNoteFont;
    procedure FocusFirst;
    procedure FocusLast;
    procedure FocusEdit(anEdit: TEdit);
    procedure MoveUp;
    procedure CleanUp;
    procedure UpdateFrame;
  end;

function newStudyNoteSection(anOwner: TComponent; anID, aLock, aNum, aName, aDateTime, aText, aType: string;
  aStatus, aStatusBy, aStatusDate: string; maxHeight, tabOrder: Integer; aCollapsed: Boolean; FindNext: TFindNext = nil;
  UseLines: Boolean = False): TFrameStudyNoteSection;

implementation

uses
  uMDK_Flowsheet,
  uMDK_Utils, uMDK_Notes, dMDK_Data,
  ComCtrls;
{$R *.dfm}

function newStudyNoteSection(anOwner: TComponent; anID, aLock, aNum, aName, aDateTime, aText, aType: string;
  aStatus, aStatusBy, aStatusDate: string; maxHeight, tabOrder: Integer; aCollapsed: Boolean; FindNext: TFindNext = nil;
  UseLines: Boolean = False): TFrameStudyNoteSection;
var
  theSection: TFrameStudyNoteSection;
  iY: Integer;
  procedure addRE(aText, aLineID: string);
  var
    edt: TRichEdit;
  begin
    with theSection do begin
      fID := anID;
      fStatus := aStatus;
      fStatusBy := aStatusBy;
      fStatusAt := aStatusDate;
      fAuthor := aName;
      fLock := aLock;
      edt := TRichEdit.Create(pnlCanvas);
      edt.Font.Name := 'Courier New';
      edt.Parent := pnlCanvas;
      edt.Align := alTop;
      edt.tabStop := True;
      edt.Text := aText;
      edt.Height := (edt.Lines.Count + 1) * (edt.Font.Size + 6);
      lblShortText.Caption := edt.Lines[0];
      fLinesCount := edt.Lines.Count;
      edt.ScrollBars := ssVertical;
      // edt.WordWrap := False;
      edt.WordWrap := True;
      iY := iY + edt.Height - 1;
      edt.OnKeyDown := edKeyDown;
      edt.BorderStyle := bsNone;
      edt.ParentColor := True;
      edt.ParentFont := False;
      edt.ReadOnly := True;
      edt.Font.Color := clWIndowText; // clNavy;
      edt.tabStop := True;
      edt.Show;
      SetNoteFont;
    end;
  end;
  procedure setTypeImage(sType: string);
  begin
    if sType = 'NOTE' then theSection.imgNotes.Visible := True
    else if sType = 'MEDICATION' then theSection.imgMedication.Visible := True
    else if sType = 'TIU' then theSection.imgTIU.Visible := True
    else if sType = 'CP REPORT' then theSection.imgCPReport.Visible := True;
    if pos('@', aDateTime) < 1 then begin
      theSection.lblDate.Caption := Piece(aDateTime, ' ', 1);
      theSection.lblTime.Caption := Piece(aDateTime, ' ', 2, 3);
    end else begin
      theSection.lblDate.Caption := Piece(aDateTime, '@', 1);
      theSection.lblTime.Caption := Piece(aDateTime, '@', 2);
    end;
    if (sType = 'TIU') or (sType = 'CP REPORT') then begin
      theSection.pnlTypeIcon.Visible := True;
      theSection.pnlDate.Visible := True;
      theSection.shpDivider.Visible := False;
    end;
  end;

begin
  theSection := TFrameStudyNoteSection.Create(anOwner);
  theSection.Name := theSection.Name + aNum;
  with theSection do begin
    setTypeImage(aType);
    Align := alBottom;
    fMinHeight := pnlSignature.Height { + 18 };
    if pnlDetails.Visible then fMinHeight := fMinHeight + pnlDetails.Height;
    Height := fMinHeight;
    theSection.tabOrder := tabOrder;
    fCollapsed := aCollapsed;
    onFindNext := FindNext;
    theSection.fText := aText;
    lblBy.Caption := aName;
    shpDivider.Width := lblBy.Width + 8;
    shpDivider.Left := lblBy.Left - 4;
    lblID.Caption := anID;
    lblStatus.Caption := aStatus;
    lblStatusBy.Caption := aStatusBy;
    lblStatusDate.Caption := aStatusDate;
    iY := 1;
    if aText = '' then aText := 'Comment left blank';
    addRE(aText, '1');
    fMaxHeight := iY + 3 + pnlSignature.Height;
    if pnlDetails.Visible then fMaxHeight := fMaxHeight + pnlDetails.Height;
    SetCollapse(aCollapsed);
  end;
  Result := theSection;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TFrameStudyNoteSection.SetLock(aValue: string);
begin
  fLock := aValue;
  if fLock = fsLocked then imgLock.Visible := True
  else imgLock.Visible := False;
end;

function TFrameStudyNoteSection.getEditWidth: Integer;
begin
  Result := pnlCanvas.Width - 4;
end;

procedure TFrameStudyNoteSection.SetSelect(aValue: Boolean);
begin
  fSelected := aValue;
  imCollapsed.Visible := fCollapsed and not fSelected;
  imgCollapsedCurrent.Visible := fCollapsed and fSelected;
  imExpanded.Visible := not fCollapsed and not fSelected;
  imgExpandedCurrent.Visible := not fCollapsed and fSelected;
end;

procedure TFrameStudyNoteSection.SetCollapse(aValue: Boolean);
var
  i: Integer;
begin
  fCollapsed := aValue;
  if fCollapsed then Height := fMinHeight
  else Height := fMaxHeight;
  if fCollapsed then lblShortText.Visible := True
  else lblShortText.Visible := False;
  imgMore.Visible := fCollapsed and (fLinesCount > 1);
  SetSelect(fSelected);
  if Assigned(onFindCurrent) then onFindCurrent(fID, self);
  if pnlCanvas.ComponentCount < 1 then Exit;
  for i := 1 to pnlCanvas.ComponentCount - 1 do
    if pnlCanvas.Components[i] is TEdit then TEdit(pnlCanvas.Components[i]).tabStop := not Collapsed;
end;

procedure TFrameStudyNoteSection.FrameEnter(Sender: TObject);
begin
  SectionSelected := True;
  if Assigned(onFindCurrent) then onFindCurrent(fID, self);
end;

procedure TFrameStudyNoteSection.FrameExit(Sender: TObject);
begin
  SectionSelected := False;
end;

procedure TFrameStudyNoteSection.acCollapseExpandExecute(Sender: TObject);
begin
  if pnlCollapse.Visible then Collapsed := not Collapsed;
end;

procedure TFrameStudyNoteSection.edTitleKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_Return then acCollapseExpand.Execute
  else if not Assigned(onFindNext) then Exit
  else if Key = VK_Up then onFindNext(True, self)
  else if Key = VK_Down then begin
    if Collapsed then onFindNext(False, self)
    else begin
      if (pnlCanvas.ComponentCount > 0) then begin
        if (pnlCanvas.Components[0] is TEdit) then FocusEdit(TEdit(pnlCanvas.Components[0]))
        else TWinControl(pnlCanvas.Components[0]).SetFocus;
      end;
    end;
  end else if (Key = VK_Left) and (ssCtrl in Shift) and (ssShift in Shift) then onFindNext(True, self)
  else if (Key = VK_Right) and (ssCtrl in Shift) and (ssShift in Shift) then onFindNext(False, self);
end;

procedure TFrameStudyNoteSection.edEnter(Sender: TObject);
begin
  TEdit(Sender).Color := clWindow;
end;

procedure TFrameStudyNoteSection.edExit(Sender: TObject);
begin
  TEdit(Sender).ParentColor := True;
end;

procedure TFrameStudyNoteSection.edKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  edt: TEdit;
  function FindNext(Up: Boolean): TWinControl;
  var
    ii: Integer;
  begin
    Result := nil;
    if pnlCanvas.ComponentCount < 1 then Exit;
    for ii := 0 to pnlCanvas.ComponentCount - 1 do begin
      if (pnlCanvas.Components[ii] is TEdit) and (pnlCanvas.Components[ii] = Sender) then begin
        if Up then begin
          if ii = 0 then begin
            if Assigned(onFindNext) then onFindNext(True, self);
          end else begin
            edt := TEdit(pnlCanvas.Components[ii - 1]);
            FocusEdit(edt);
          end;
        end else begin
          if Collapsed or (ii = pnlCanvas.ComponentCount - 1) then begin
            edt := TEdit(pnlCanvas.Components[0]);
            if Assigned(onFindNext) then onFindNext(False, self);
          end else begin
            edt := TEdit(pnlCanvas.Components[ii + 1]);
            FocusEdit(edt);
          end;
        end;
        break;
      end;
    end;
  end;

begin
  if (Key = VK_Return) and (Sender = pnlCanvas.Components[0]) then acCollapseExpand.Execute
  else if Key = VK_Down then FindNext(False)
  else if (Key = VK_Up) then FindNext(True)
  else if (Key = VK_Left) and (ssCtrl in Shift) and (ssShift in Shift) then FocusFirst
  else if (Key = VK_Right) and (ssCtrl in Shift) and (ssShift in Shift) then
    if Assigned(onFindNext) then onFindNext(False, self);
end;

procedure TFrameStudyNoteSection.FocusFirst;
var
  edt: TEdit;
begin
  if pnlCanvas.ComponentCount < 1 then Exit;
  edt := TEdit(pnlCanvas.Components[0]);
  FocusEdit(edt);
end;

procedure TFrameStudyNoteSection.FocusLast;
var
  edActive: TEdit;
begin
  if pnlCanvas.ComponentCount < 1 then Exit;
  edActive := TEdit(pnlCanvas.Components[pnlCanvas.ComponentCount - 1]);
  FocusEdit(edActive);
end;

procedure TFrameStudyNoteSection.FocusEdit(anEdit: TEdit);
var
  aForm: TCustomForm;
begin
  aForm := GetParentForm(anEdit);
  if Assigned(aForm) then aForm.FocusControl(anEdit);
  anEdit.SelLength := 0;
end;

procedure TFrameStudyNoteSection.MoveUp;
begin
  Align := alTop;
end;

procedure TFrameStudyNoteSection.pnlCanvasResize(Sender: TObject);
var
  ii: Integer;
  edt: TEdit;
begin
  for ii := 0 to pnlCanvas.ComponentCount - 1 do begin
    if pnlCanvas.Components[ii] is TEdit then begin
      edt := TEdit(pnlCanvas.Components[ii]);
      edt.Width := getEditWidth;
    end;
  end;
end;

procedure TFrameStudyNoteSection.pnlCanvasDblClick(Sender: TObject);
begin
  acChangeValidStatus.Execute;
end;

procedure TFrameStudyNoteSection.CleanUp;
var
  re: TRichEdit;
  edt: TEdit;
  i: Integer;
begin
  i := 0;
  while i < pnlCanvas.ComponentCount do begin
    if pnlCanvas.Components[i] is TEdit then begin
      edt := TEdit(pnlCanvas.Components[i]);
      pnlCanvas.RemoveComponent(edt);
      if Assigned(edt) then FreeAndNil(edt);
    end else if pnlCanvas.Components[i] is TRichEdit then begin
      re := TRichEdit(pnlCanvas.Components[i]);
      pnlCanvas.RemoveComponent(re);
      if Assigned(re) then FreeAndNil(re);
    end else begin
      inc(i);
    end;
  end;
end;

procedure TFrameStudyNoteSection.pnlLeftClick(Sender: TObject);
begin
  if Assigned(onFindCurrent) then onFindCurrent(fID, self);
end;

procedure TFrameStudyNoteSection.SetNoteFont;
var
  i: Integer;
  re: TRichEdit;
  edt: TEdit;
  s: string;
  function CurrText: TTextAttributes;
  begin
    if re.SelLength > 0 then Result := re.SelAttributes
    else Result := re.DefAttributes;
  end;

begin
  if fStatus = fsInvalid then lblShortText.Font.Style := [fsStrikeOut]
  else lblShortText.Font.Style := [];
  if fStatus = fsInvalid then lblShortText.Font.Color := clRed
  else lblShortText.Font.Color := clWIndowText;
  for i := 0 to pnlCanvas.ComponentCount - 1 do begin
    if pnlCanvas.Components[i] is TEdit then begin
      edt := TEdit(pnlCanvas.Components[i]);
      if fStatus = fsInvalid then edt.Font.Style := [fsStrikeOut]
      else edt.Font.Style := [];
    end;
    if pnlCanvas.Components[i] is TRichEdit then begin
      re := TRichEdit(pnlCanvas.Components[i]);
      s := re.Text;
      re.SelStart := 0;
      re.SelLength := length(s);
      if fStatus = fsInvalid then CurrText.Style := [fsStrikeOut]
      else CurrText.Style := [];
      if fStatus = fsInvalid then CurrText.Color := clRed
      else CurrText.Color := clWIndowText;
      imgLock.Visible := fLock <> fsUnLocked;
    end;
  end;
end;

procedure TFrameStudyNoteSection.UpdateFrame;
begin
  lblStatus.Caption := fStatus;
  try
    lblStatusBy.Caption := dmData.cdsNotes.FieldByName(fnNL_StatusBy).AsString;
    lblStatusDate.Caption := dmData.cdsNotes.FieldByName(fnNL_StatusDate).AsString;
    lblTag.Caption := dmData.cdsNotes.FieldByName(fnNL_Tag).AsString;
    fLock := dmData.cdsNotes.FieldByName(fnNL_Lock).AsString;
  except
  end;
  SetNoteFont;
end;

procedure TFrameStudyNoteSection.acChangeValidStatusExecute(Sender: TObject);
begin
  dmData.ChangeCurrentNoteValidStatus;
  try
    fStatus := dmData.cdsNotes.FieldByName(fnNL_Status).AsString;
  except
  end;
  UpdateFrame;
end;

procedure TFrameStudyNoteSection.scUpdateExecute(Sender: TObject);
begin
  UpdateFrame;
end;

end.
