unit uMDK_Fresenius;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:38a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     Fresenius data field names
  *       Notes:
  *
  ================================================================================
  *       $Archive: /Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/APP-HEMODIALYSIS/Instruments/uMDK_Fresenius.pas $
  *
  * $History: uMDK_Fresenius.pas $
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 1/13/09    Time: 1:15p
  * Created in $/Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/APP-HEMODIALYSIS/Instruments
  * Patch 19 source code
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/24/08    Time: 4:23p
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/HEMODIALYSIS_1_0_6_319_#18/APP-HEMODIALYSIS/Instruments
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/02/08    Time: 9:25a
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI Current/HEMODIALYSIS_1_0_6_319/APP-HEMODIALYSIS/Instruments
  *
  * *****************  Version 3  *****************
  * User: Zzzzzzandria Date: 8/24/07    Time: 10:48a
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Instruments
  *
  * *****************  Version 2  *****************
  * User: Zzzzzzandria Date: 7/02/07    Time: 9:09a
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Instruments
  * #24: Minor changes of GUI
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 5/22/07    Time: 9:16a
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Instruments
}
interface

uses
  uMDK_InstrumentRecord, XMLIntf, SysUtils, Dialogs;

const
  kwFRES_BPSystolic             = 'BP Systolic';
  kwFRES_BPDiastolic            = 'BP Diastolic';
  kwFRES_Pulse                  = 'Pulse';
  kwFRES_BloodFlowRate          = 'Blood Flow Rate';
  kwFRES_ArterialPressure       = 'Arterial Pressure';
  kwFRES_VenousPressure         = 'Venous Pressure';
  kwFRES_UltraFiltrationRate    = 'Ultra Filtration Rate';
  kwFRES_Temperature            = 'Temperature';
  kwFRES_TransMembranePressure  = 'Trans Membrane Pressure';
  kwFRES_Conductivity           = 'Conductivity';
  kwFRES_DialysateFlowRate      = 'Dialysate Flow Rate';
  kwFRES_UltraFiltrationRemoved = 'Ultra Filtration Removed';
  kwFres_HeparinInfused         = 'Total Heparin Infused';
  kwFres_GoalWeight             = 'Goal Weight'; // Tx Goal Weight  zzzzzzandria 051208
  // kwFres_StatuionNumber = ''; // zzzzzzandria 051208
  // kwFres_MachineNumber = ''; // zzzzzzandria 051208
  // 050920 - zzzzzzandria
  kwFRES_PreTemperature          = 'PRE Vital Temperature';
  kwFRES_PreWeight               = 'PRE Vital Weight';
  kwFRES_PrePulse                = 'PRE Vital Pulse';
  kwFRES_PreBPSystolicSitting    = 'PRE BP Systolic Sitting';
  kwFRES_PreBPDiastolicSitting   = 'PRE BP Diastolic Sitting';
  kwFRES_PreBPSystolicStanding   = 'PRE BP Systolic Standing';
  kwFRES_PreBPDiastolicStanding  = 'PRE BP Diastolic Standing';
  kwFRES_PostTemperature         = 'POST Vital Temperature';
  kwFRES_PostWeight              = 'POST Vital Weight';
  kwFRES_PostPulse               = 'POST Vital Pulse';
  kwFRES_PostBPSystolicSitting   = 'POST BP Systolic Sitting';
  kwFRES_PostBPDiastolicSitting  = 'POST BP Diastolic Sitting';
  kwFRES_PostBPSystolicStanding  = 'POST BP Systolic Standing';
  kwFRES_PostBPDiastolicStanding = 'POST BP Diastolic Standing';
  kwFRES_RemainingTime           = 'Remaining Time On Dialysis';
  kwFRES_MAP                     = 'Mean Arterial Pressure';
  kwFRES_EffectiveKTV            = 'Effective KtV';
  kwFRES_TotalUF                 = 'Total UF';
  kwFRES_PreviousGoalWeight      = 'Previous Goal Weight';
  kwFRES_TotalUltraFiltration    = 'Total Ultra Filtration';
  kwFRES_TotalBV                 = 'Total Blood Volume';
  kwFRES_MeanBF                  = 'Mean Blood Flow';
  kwFRES_MeanDF                  = 'Mean Dialysate Flow';
  kwFRES_MeanUFR                 = 'Mean Ultra Filtration Rate';
  kwFRES_MeanTMP                 = 'Mean Trans Memb Pressure';
  kwFRES_MeanTemp                = 'Mean Temperature';
  kwFRES_TimeOn                  = 'Time On';
  kwFRES_TimeOff                 = 'Time Off';
  kwFRES_TargetWL                = 'Target Weight Loss';
  kwFRES_TreatmentStarted        = 'Time On';
  kwFRES_TreatmentStoped         = 'Time Off';
  // Prescription
  kwFRES_DialyzerType              = 'Dialyzer Type';
  kwFRES_DialyzerReuse             = 'Dialyzer Reuse Nbr';
  kwFRES_TreatmentLength           = 'Treatment Length';
  kwFRES_PrBloodFlowRate           = 'Blood Flow Rate';
  kwFRES_DialysateFormula          = 'Dialysate Formula';
  kwFRES_ModelingNa                = 'Modeling Na';
  kwFRES_SodiumProgram             = 'Sodium Program';
  kwFRES_SodiumProgramTime         = 'Sodium Program Time';
  kwFRES_SodiumBaseConductivity    = 'Sodium Base Conductivity';
  kwFRES_DialysateFlow             = 'Dialysate Flow';
  kwFRES_AnticoagulantLoad         = 'Anticoagulant Load';
  kwFRES_AnticoagulantUnitsPerHour = 'Anticoagulant Units Per Hour';
  kwFRES_AnticoagulantType         = 'Anticoagulant Type';
  kwFRES_DryWeight                 = 'Dry Weight';
  // Medications
  kwFRES_MedicationType                  = 'Medication Type';
  kwFRES_ICD9Description                 = 'ICD 9 Description';
  kwFRES_ICD9Code                        = 'ICD 9 Code';
  kwFRES_MeasurementType                 = 'Measurement Type';
  kwFRES_RouteType                       = 'Route Type';
  kwFRES_PatientCareInitials             = 'Patient Care Initials';
  kwFRES_Dose                            = 'Dose';
  kwFRES_CurrentPostTreatmentParameters  = 'Current POST Treatment Parameters';
  kwFRES_CurrentPreTreatmentParameters   = 'Current PRE Treatment Parameters';
  kwFRES_PreviousPostTreatmentParameters = 'Previous POST Treatment Parameters';
  kwFRES_PreviousPreTreatmentParameters  = 'Previous PRE Treatment Parameters';
  kwFRES_Note                            = 'Note';
  // zzzzzzandria 2007-06-27 - Notes are not coming through
  kwFRES_NoteCreatedDateTime = 'Note Created Date Time';
  kwFRES_NoteCreatedBy       = 'Note Created By';
  // 060627
  kwFres_StationID         = 'Station ID';
  kwFres_InstrumentID      = 'Machine Number';
  kwFres_TreatmentDuration = 'Treatment Duration';

procedure UpdateFreseniusMedicationList(aPatientID, aStudyID: string; anIR: TInstrumentRecord);
procedure setFreseniusRecord(aNode: IXMLNode; aProcType: string; anIR: TInstrumentRecord);
function FreseniusDateTimeStringToWindows(aDateTime: string): TDateTime;

implementation

uses
  dMDK_Data, uMDK_MedList, uROR_CommonUtils;

function FreseniusDateTimeStringToWindows(aDateTime: string): TDateTime;
var
  Year, Month, Day: Word;
  Hour, Min, Sec: Word;
begin
  try
    Year := StrToInt(Copy(aDateTime, 1, 4));
    Month := StrToInt(Copy(aDateTime, 5, 2));
    Day := StrToInt(Copy(aDateTime, 7, 2));
    Hour := StrToIntDef(Copy(aDateTime, 9, 2), 0);
    Min := StrToIntDef(Copy(aDateTime, 11, 2), 0);
    Sec := StrToIntDef(Copy(aDateTime, 13, 2), 0);
    Result := EncodeDate(Year, Month, Day) + EncodeTime(Hour, Min, Sec, 0);
  except
    on E: EConvertError do begin
{$IFDEF AANTEST}
      ShowMessage('Error Converting Date ' + aDateTime);
{$ENDIF}
      Result := Now;
    end;
    else
      raise;
  end;
end;

procedure UpdateFreseniusMedicationList(aPatientID, aStudyID: string; anIR: TInstrumentRecord);
begin
  if (anIR.Vendor = 'FRESENIUS') and ((anIR.MedType <> '') and (anIR.MedDose <> '') and (anIR.MedMeasurement <> '') and
    (anIR.MedRouteType <> '')) then begin
    dmData.cdsMedList.Insert;
    dmData.cdsMedList.FieldByName(fnML_PatientID).AsString := aPatientID;
    dmData.cdsMedList.FieldByName(fnML_StudyID).AsString := aStudyID;
    dmData.cdsMedList.FieldByName(fnML_Date).AsDateTime := anIR.FS_DateTime;
    dmData.cdsMedList.FieldByName(fnML_Time).AsDateTime := Frac(anIR.FS_DateTime);
    dmData.cdsMedList.FieldByName(fnML_Medication).AsString := anIR.MedType;
    dmData.cdsMedList.FieldByName(fnML_Dose).AsString := anIR.MedDose;
    dmData.cdsMedList.FieldByName(fnML_Units).AsString := anIR.MedMeasurement;
    dmData.cdsMedList.FieldByName(fnML_Rout).AsString := anIR.MedRouteType;
    dmData.cdsMedList.FieldByName(fnML_GivenBy).AsString := anIR.MedPatientCareInitials;
    dmData.cdsMedList.FieldByName(fnML_Reason).AsString := anIR.MedICD9Code + ' ' + anIR.MedICD9Description;
    dmData.cdsMedList.FieldByName(fnML_Source).AsString := 'FRESENIUS';
  end;
end;

procedure setFreseniusRecord(aNode: IXMLNode; aProcType: string; anIR: TInstrumentRecord);
var
  sValue, sValueName: string;
  iValueCode: Integer;
  dd: Double;

  procedure FreseniusMedication; // medications, events, intake
  var
    mlName, mlUnits, mlValue: string;
  begin
    case iValueCode of
      - 1:;
      7101 .. 7117: begin
          mlName := piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^', 2);
          mlUnits := aNode.ChildNodes['VALUE'].Text;
          mlValue := aNode.ChildNodes['UNITS'].Text;
          anIR.MedList.Add(mlName + '^' + mlUnits + '^' + mlValue);
        end;
      7301 .. 7302: anIR.ProblemList.Add( // sScheme+' '+
          anIR.Observation_Date_time + '  ' + piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^', 2) + '  ' +
          aNode.ChildNodes['VALUE'].Text + '  ' + aNode.ChildNodes['UNITS'].Text);
      7501 .. 7532: anIR.ProblemList.Add( // sScheme+' '+
          anIR.Observation_Date_time + '  ' + piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^', 2));
    end;
  end;

begin
  with anIR do begin
    Vendor := 'FRESENIUS';
    sValueName := aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text;
    sValue := aNode.ChildNodes['VALUE'].Text;
    // not sure the next use of Integer is correct
    iValueCode := StrToIntDef(piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^', 1), -1);
    ContainsFlowsheetData := False;
    if (aProcType = kwFRES_PreviousPostTreatmentParameters) or
    // zzzzzzandria 060306
      (aProcType = kwFRES_PreviousPreTreatmentParameters) then
      // zzzzzzandria 060306
        Exit;
    ContainsFlowsheetData := True;
    if sValueName = kwFRES_TotalUF then FS_Fluid := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_MAP) and (sValue <> '') then FS_MAP := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_BPSystolic) and (sValue <> '') then begin
      FS_Sys := piece(sValue, '^', 1);
      bFreseniusSummary := piece(sValue, '^', 2) <> '';
    end else if (sValueName = kwFRES_BPDiastolic) and (sValue <> '') then FS_Dias := piece(sValue, '^', 1)
    else if (sValueName = kwFRES_Conductivity) and (sValue <> '') then FS_COND := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_Pulse) and (sValue <> '') then FS_HR := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_BloodFlowRate) and (sValue <> '') then FS_BFR := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_VenousPressure) and (sValue <> '') then FS_VP := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_ArterialPressure) and (sValue <> '') then FS_AP := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_UltraFiltrationRate) and (sValue <> '') then FS_UFR := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_Temperature) and (sValue <> '') then FS_DIAL := LocalStrToFloat(sValue)
    else if (sValueName = kwFres_HeparinInfused) and (sValue <> '') then FS_HEP := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_DialysateFlowRate) and (sValue <> '') then FS_DFR := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_TransMembranePressure) and (sValue <> '') then FS_TMP := LocalStrToFloat(sValue)
    else if (sValueName = kwFRES_EffectiveKTV) and (sValue <> '') then TotalKTV := sValue
    else if (sValueName = kwFRES_TreatmentStarted) and (sValue <> '') then TreatmentStarted := sValue
    else if (sValueName = kwFRES_TreatmentStoped) and (sValue <> '') then TreatmentFinished := sValue
    else ContainsFlowsheetData := False;
    /// /////////////////////////////////////////////////////////////////////////
    if (sValueName = kwFRES_PreTemperature) and (sValue <> '') then PreTemperature := sValue
    else if (sValueName = kwFRES_PostTemperature) and (sValue <> '') then PostTemperature := sValue
    else if (sValueName = kwFRES_PreWeight) and (sValue <> '') then WeightBefore := sValue
    else if (sValueName = kwFRES_PreBPSystolicStanding) and (sValue <> '') then
        PreStandingSys := sValue { SToI(sValue) }
    else if (sValueName = kwFRES_PreBPDiastolicStanding) and (sValue <> '') then
        PreStandingDia := sValue { SToI(sValue) }
    else if (sValueName = kwFRES_PrePulse) and (sValue <> '') then PreSeatedPulse := SToI(sValue)
    else if (sValueName = kwFRES_PostWeight) and (sValue <> '') then WeightAfter := sValue
    else if (sValueName = kwFRES_PostBPSystolicStanding) and (sValue <> '') then
        PostStandingSys := sValue { SToI(sValue) }
    else if (sValueName = kwFRES_PostBPDiastolicStanding) and (sValue <> '') then
        PostStandingDia := sValue { SToI(sValue) }
    else if (sValueName = kwFRES_PostPulse) and (sValue <> '') then PostSeatedPulse := SToI(sValue)
    else if (sValueName = kwFRES_PreBPSystolicSitting) and (sValue <> '') then PreSeatedSys := sValue { SToI(sValue) }
    else if (sValueName = kwFRES_PreBPDiastolicSitting) and (sValue <> '') then PreSeatedDia := sValue { SToI(sValue) }
    else if (sValueName = kwFRES_PostBPSystolicSitting) and (sValue <> '') then PostSeatedSys := sValue { SToI(sValue) }
    else if (sValueName = kwFRES_PostBPDiastolicSitting) and (sValue <> '') then
        PostSeatedDia := sValue { SToI(sValue) }
    else if (sValueName = kwFRES_TotalUltraFiltration) and (sValue <> '') then TotalUF := sValue
    else if (sValueName = kwFRES_TotalBV) and (sValue <> '') then TotalLP := sValue
    else if (sValueName = kwFRES_MeanBF) and (sValue <> '') then AverageBFR := sValue
    else if (sValueName = kwFRES_MeanDF) and (sValue <> '') then AverageDFR := sValue
    else if (sValueName = kwFRES_MeanUFR) and (sValue <> '') then MeanUFR := sValue
    else if (sValueName = kwFRES_MeanTMP) and (sValue <> '') then MeanTMP := sValue
    else if (sValueName = kwFRES_MeanTemp) and (sValue <> '') then MeanDTemp := sValue
      // Prescription Info
    else if (sValueName = kwFRES_DialyzerType) and (sValue <> '') then Dialyzer := sValue
    else if (sValueName = kwFRES_DialyzerReuse) and (sValue <> '') then MaxReuseNbr := sValue
    else if (sValueName = kwFRES_TreatmentLength) and (sValue <> '') then begin
      dd := StrToInt(sValue) / (24 * 60);
      TxLength := FormatDateTime('hh:mm', dd);
    end
    // 060627 zzzzzzandria
    else if (sValueName = kwFres_TreatmentDuration) and (sValue <> '') then begin
      // dd := StrToInt(sValue)/(24*60);
      // Duration := FormatDateTime('hh:mm',dd);
      DiffusionTimeSec := StrToInt(sValue) * 60;
    end else if (sValueName = kwFRES_DryWeight) and (sValue <> '') then EDW := sValue
    else if (sValueName = kwFRES_PrBloodFlowRate) and (sValue <> '') then BFR := sValue
    else if (sValueName = kwFRES_DialysateFlow) and (sValue <> '') then DialysateFlow := sValue
    else if (sValueName = kwFRES_AnticoagulantType) and (sValue <> '') then DType := sValue
    else if (sValueName = kwFRES_AnticoagulantLoad) and (sValue <> '') then Bolus := sValue
    else if (sValueName = kwFRES_AnticoagulantUnitsPerHour) and (sValue <> '') then Maintenance := sValue
    else if (sValueName = kwFRES_DialyzerReuse) and (sValue <> '') then MaxReuseNbr := sValue
    else if (sValueName = kwFRES_ModelingNa) and (sValue <> '') then NA := sValue
    else if (sValueName = kwFRES_Note) and (sValue <> '') then NoteText := sValue
    else if (sValueName = kwFRES_NoteCreatedDateTime) and (sValue <> '') then NoteDateTime := sValue
    else if (sValueName = kwFRES_NoteCreatedBy) and (sValue <> '') then Noteby := sValue
    else if (sValueName = kwFRES_MedicationType) and (sValue <> '') then MedType := sValue
    else if (sValueName = kwFRES_ICD9Description) and (sValue <> '') then MedICD9Description := sValue
    else if (sValueName = kwFRES_ICD9Code) and (sValue <> '') then MedICD9Code := sValue
    else if (sValueName = kwFRES_Dose) and (sValue <> '') then MedDose := sValue
    else if (sValueName = kwFRES_MeasurementType) and (sValue <> '') then MedMeasurement := sValue
    else if (sValueName = kwFRES_RouteType) and (sValue <> '') then MedRouteType := sValue
    else if (sValueName = kwFRES_PatientCareInitials) and (sValue <> '') then MedPatientCareInitials := sValue
      // 060627 zzzzzzandria
    else if (sValueName = kwFres_InstrumentID) and (sValue <> '') then InstrumentID := sValue
    else if (sValueName = kwFres_StationID) and (sValue <> '') then StationID := sValue
    else if (sValueName = kwFres_GoalWeight) and (aProcType = kwFRES_CurrentPostTreatmentParameters) and (sValue <> '')
    then TxGoalWeight := sValue;
  end;
end;

end.
