unit uMDK_MDateTime;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:38a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     M date/time object
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_MDateTime.pas $
  *
  *       $ History: uMDK_MDateTime.pas $
}
interface

uses
  Forms;

type
  TMDateTime = class(TObject)
  private
    FDateTime: TDateTime;
    function getWDate: TDateTime;
    function getWTime: TDateTime;
    procedure setWDate(aDateTime: TDateTime);
    procedure setWTime(aDateTime: TDateTime);
    procedure setWDateTime(aDateTime: TDateTime);
    function getWSLong: string;
  public
    DateTimeFormat: string;
    DateFormat: string;
    TimeFormat: string;
    constructor Create;
    function getMDate: Double;
    function getMTime: Double;
    function getMDateTime: Double;
    procedure setMDate(aDateTime: Double);
    procedure setMTime(aDateTime: Double);
    procedure setMDateTime(aDateTime: Double);
    function getSWDate: string;
    function getSWTime: string;
    function getSWDateTime: string;
    function getSMDate: string;
    function getSMTime: string;
    function getSMDateTime: string;
    procedure setSMDateTime(aDateTime: string);
    procedure setSWDateTime(aDateTime: string);
    // published
    property WDate: TDateTime read getWDate write setWDate;
    property WTime: TDateTime read getWTime write setWTime;
    property WDateTime: TDateTime read FDateTime write setWDateTime;
    property WSLong: string read getWSLong;
  end;

  TMDateTimeRange = class(TObject)
  public
    Start: TMDateTime;
    Stop: TMDateTime;
    constructor Create;
    destructor Destroy; override;
    function setMRange(aRange: string): Boolean;
    function selectMRange(aRange: string): Boolean;
    function getSWRange: string;
    function GetDateRange(var FromDate, ToDate: TDateTime; AllowFutureDate: Boolean = True;
      AllowPastDate: Boolean = True): Boolean;
  end;

const
  sWRangeDelim = ' -- ';
  sDateRange   = 'DATE RANGE';

implementation

uses
  SysUtils, Controls, fMDK_DateRangeSelector, uROR_CommonUtils;

constructor TMDateTime.Create;
begin
  inherited;
  DateTimeFormat := 'mm/dd/yy hh:nn:ss';
  DateFormat := 'mm/dd/yy';
  TimeFormat := 'hh:nn:ss';
end;

function TMDateTime.getWDate: TDateTime;
begin
  Result := trunc(FDateTime);
end;

function TMDateTime.getWTime: TDateTime;
begin
  Result := frac(FDateTime);
end;

function TMDateTime.getWSLong: string;
begin
  Result := FormatDateTime('dddd mmmm mm, yyyy', FDateTime);
end;

procedure TMDateTime.setWDate(aDateTime: TDateTime);
begin
  FDateTime := trunc(aDateTime) + frac(FDateTime);
end;

procedure TMDateTime.setWTime(aDateTime: TDateTime);
begin
  FDateTime := trunc(FDateTime) + frac(aDateTime);
end;

procedure TMDateTime.setWDateTime(aDateTime: TDateTime);
begin
  FDateTime := aDateTime;
end;
/// ////////////////////////////////////////////////////////

function TMDateTime.getMDateTime: Double;
begin
  Result := WindowsDateTimeToFMDateTime(FDateTime);
end;

function TMDateTime.getMDate: Double;
begin
  Result := trunc(WindowsDateTimeToFMDateTime(FDateTime));
end;

function TMDateTime.getMTime: Double;
begin
  Result := frac(WindowsDateTimeToFMDateTime(FDateTime));
end;

procedure TMDateTime.setMDate(aDateTime: Double);
begin
  FDateTime := trunc(FMDateTimeToWindowsDateTime(aDateTime)) + frac(FDateTime);
end;

procedure TMDateTime.setMTime(aDateTime: Double);
begin
  FDateTime := trunc(FDateTime) + frac(FMDateTimeToWindowsDateTime(aDateTime));
end;

procedure TMDateTime.setMDateTime(aDateTime: Double);
begin
  FDateTime := FMDateTimeToWindowsDateTime(aDateTime);
end;
/// ///////////////////////////////////////////////////////////////

function TMDateTime.getSWDate: string;
begin
  if FDateTime <> 0 then Result := FormatDateTime(DateFormat, FDateTime)
  else Result := '';
end;

function TMDateTime.getSWTime: string;
begin
  Result := FormatDateTime(TimeFormat, FDateTime);
end;

function TMDateTime.getSWDateTime: string;
begin
  Result := FormatDateTime(DateTimeFormat, FDateTime);
end;
/// /////////////////////////////////////////////////////////////////

function TMDateTime.getSMDate: string;
var
  d: Double;
begin
  try
    d := WindowsDateTimeToFMDateTime(FDateTime);
    Result := FloatToStr(trunc(d));
  except
    Result := '';
  end;
end;

function TMDateTime.getSMTime: string;
var
  d: Double;
begin
  try
    d := WindowsDateTimeToFMDateTime(FDateTime);
    Result := FloatToStr(frac(d));
  except
    Result := '';
  end;
end;

function TMDateTime.getSMDateTime: string;
begin
  try
    Result := FloatToStr(WindowsDateTimeToFMDateTime(FDateTime));
  except
    Result := '';
  end;
end;

procedure TMDateTime.setSMDateTime(aDateTime: string);
begin
  try
    FDateTime := FMDateTimeToWindowsDateTime(StrToFloat(aDateTime));
  except
  end;
end;

procedure TMDateTime.setSWDateTime(aDateTime: string);
begin
  try
    FDateTime := StrToDateTime(aDateTime);
  except
  end;
end;
/// /////////////////////////////////////////////////////////////////////

constructor TMDateTimeRange.Create;
begin
  inherited;
  Start := TMDateTime.Create;
  Start.WDateTime := Now;
  Stop := TMDateTime.Create;
  Stop.WDateTime := Now;
end;

destructor TMDateTimeRange.Destroy;
begin
  Start.Free;
  Stop.Free;
  inherited;
end;

function TMDateTimeRange.getSWRange: string;
begin
  Result := Start.getSWDate + sWRangeDelim + Stop.getSWDate;
end;
(* *)

function TMDateTimeRange.GetDateRange(var FromDate, ToDate: TDateTime; AllowFutureDate: Boolean = True;
  AllowPastDate: Boolean = True): Boolean;
begin
  with TfrmGMV_DateRange.Create(Application) do
    try
      dtpFrom.DateTime := FromDate;
      dtpTo.DateTime := ToDate;
      dtpFrom.MaxDate := trunc(Now);
      dtpTo.MaxDate := dtpFrom.MaxDate;
      ShowModal;
      if ModalResult = mrOK then begin
        FromDate := dtpFrom.Date;
        ToDate := dtpTo.Date;
        Result := True;
      end
      else Result := False;
    finally
      Free;
    end;
end;

function TMDateTimeRange.setMRange(aRange: string): Boolean;
var
  i: Integer;
  _Now, aStart, aStop: TDateTime;
const
  DayKey = 'TODAY T-1 T-2 T-3 T-4 T-5 T-6 T-7 T-15 T-30 SIX MONTHS ONE YEAR TWO YEARS THREE MONTHS';
begin
  if UpperCase(aRange) = sDateRange then begin
    aStart := Start.WDateTime;
    aStop := Stop.WDateTime;
    if GetDateRange(aStart, aStop, False) then begin
      Start.WDateTime := aStart;
      Stop.WDateTime := aStop;
      Result := True;
    end
    else Result := False;
  end else if pos(UpperCase(aRange), DayKey) <> 0 then begin
    _Now := Now;
    aStop := trunc(_Now) + 1 - 1 / (3600 * 24);
    _Now := trunc(_Now);
    i := pos(UpperCase(aRange), DayKey);
    case i of
      1: aStart := _Now;
      7: aStart := _Now - 1;
      11: aStart := _Now - 2;
      15: aStart := _Now - 3;
      19: aStart := _Now - 4;
      23: aStart := _Now - 5;
      27: aStart := _Now - 6;
      31: aStart := _Now - 7;
      35: aStart := _Now - 15;
      40: aStart := _Now - 30;
      45: aStart := _Now - 183;
      56: aStart := _Now - 365;
      65: aStart := _Now - 730;
      75: aStart := _Now - 91;
    end;
    Start.WDateTime := aStart;
    Stop.WDateTime := aStop;
    Result := True;
  end else begin
    Start.setSWDateTime(Piece(aRange, sWRangeDelim, 1));
    Stop.setSWDateTime(Piece(aRange, sWRangeDelim, 2));
    Result := True;
  end;
end;

function TMDateTimeRange.selectMRange(aRange: string): Boolean;
var
  aStart, aStop: TDateTime;
begin
  aStart := Start.WDateTime;
  aStop := Stop.WDateTime;
  if GetDateRange(aStart, aStop, False) then begin
    Start.WDateTime := aStart;
    Stop.WDateTime := aStop;
    Result := True;
  end
  else Result := False;
end;

end.
