unit uMDK_ReportEngine;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 2/08/08 9:24a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     Report routenes
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_ReportEngine.pas $
  *
  *       $ History: uMDK_ReportEngine.pas $
}
interface

uses
  Variants, SysUtils, Dialogs, Classes, ComCtrls, Graphics, Printers;

function IsPatternCorrect(aPattern: string): Boolean;
function getPatternValue(aPattern: string): string;
function getReportValue(aTemplate: string): string;
function setReportWidth(aReport: string; Width: Integer = 80): string;
procedure PaintSyntax(aTemplate: TRichEdit; ShowBrackets: Boolean = True; anOpenBracket: string = '<';
  aCloseBracket: string = '>'; ColorBracket: TColor = clPurple; ColorResult: TColor = clNavy;
  ColorKeyword: TColor = clNavy; ColorField: TColor = clGreen; ColorUnknown: TColor = clRed);
procedure PrintReportString(aPrinter: TPrinter; aValue: string; imgBackground: TBitmap);

const
  kw_MedicineAdministrated = 'MEDICINE ADMINISTRATED';
  kw_MedicineReport        = 'MEDICINE TABLE';
  kw_Execute               = 'EXECUTE';
  kw_FlowsheetReport       = 'FLOWSHEET';
  kw_ScheduleReport        = 'SCHEDULE';
  kw_AccessUsed            = 'ACCESS USED';
  kw_FlowNotesReport       = 'Flowsheet Notes';
  kw_PrePainReport         = 'Pre Pain Report';
  kw_PostPainReport        = 'Post Pain Report';
  kw_Now                   = 'Now';
  kw_Version               = 'Version';
  kw_UserName              = 'UserName';
  kw_RxLabComments         = 'RxLab Comments';
  kw_PreComments           = 'Pre-Treatment Comments';
  kw_PostComments          = 'Post-Treatment Comments';
  kw_VAComments            = 'VA Comments';
  kw_FlowsheetComments     = 'Flowsheet Comments';
  kw_SummaryComments       = 'Summary Comments';
  kw_FallsAssessment       = 'Falls Assessment';

implementation

uses
  dMDK_Data, uMDK_Patients, uMDK_Common, fMDK_Access, uMDK_Engine, uMDK_CDSCommon, uMDK_Options, uMDK_Dictionary,
  uMDK_Notes, Types, uMDK_Summary, fMDK_Post;

const
  sBBegin = '<';
  sBEnd   = '>';
  CRLF    = #13#10;

var
  kwc_RxLab,
  kwc_Pre, kwc_Post, kwc_Summary: string;

function PatternIsKeyword(aPattern: string): Boolean;
begin
  Result := ((aPattern = kw_PtName) or (aPattern = kw_PtAge) or (aPattern = kw_PtSSN) or (aPattern = kw_PtSex) or
    (aPattern = kw_PtDOB) or (aPattern = kw_PtID) or (aPattern = kw_StationID) or (aPattern = kw_StudyID) or
    (aPattern = kw_Now) or (aPattern = kw_UserName) or (aPattern = kw_Version) or (aPattern = kw_MedicineAdministrated)
    or (aPattern = kw_MedicineReport) or (aPattern = kw_AccessUsed) or (aPattern = kw_ScheduleReport) or
    (aPattern = kw_FlowsheetReport) or (aPattern = kw_FlowNotesReport) or (aPattern = kw_PostPainReport) or
    (aPattern = kw_PrePainReport) or (aPattern = kw_RxLabComments) or (aPattern = kw_PreComments) or
    (aPattern = kw_VAComments) or (aPattern = kw_PostComments) or (aPattern = kw_PreComments) or
    (aPattern = kw_FlowsheetComments) or (aPattern = kw_SummaryComments) or (aPattern = kw_FallsAssessment) or
    (aPattern = kw_Execute))
end;

function PatternIsField(aPattern: string): Boolean;
begin
  Result := dmData.cdsDict.Locate(fnDict_Tag, VarArrayOf([aPattern]), [])
end;

function PatternIsOption(aPattern: string): Boolean;
begin
  Result := Preferences.ValueList.IndexOfName(aPattern) > -1;
end;

function IsPatternCorrect(aPattern: string): Boolean;
begin
  Result := PatternIsField(aPattern) or PatternIsKeyword(aPattern) or PatternIsOption(aPattern);
end;

function getPatternValue(aPattern: string): string;
var
  sBlank, sValue: string;
begin
  if aPattern = kw_ScheduleReport then Result := dmData.getScheduleReport
  else if aPattern = kw_FlowsheetReport then Result := dmData.getFlowsheetReport
  else if aPattern = kw_FallsAssessment then begin
    if Assigned(frmPost) then Result := frmPost.getFallsAssessment
    else Result := '';
  end else if aPattern = kw_MedicineAdministrated then Result := dmData.getMedicineAdministrated
  else if aPattern = kw_MedicineReport then Result := dmData.getMedicineReport
  else if aPattern = kw_AccessUsed then Result := frmAccess.AssessReport // Patch
  else if aPattern = kw_PrePainReport then Result := dmData.getPainReport(True) // Patch
  else if aPattern = kw_PostPainReport then Result := dmData.getPainReport(False) // Patch
  else if aPattern = kw_FlowNotesReport then Result := dmData.getFlowNotesReport // Patch
  else if aPattern = kw_Now then Result := FormatDateTime(gMDK_DateTimeFMT, Now)
  else if aPattern = kw_Version then Result := Engine.ExeNameAndVersion
  else if aPattern = kw_UserName then Result := Engine.CurrentOperatorID
  else if aPattern = kw_RxLabComments then Result := dmData.getCommentsReport(getNotesContextFilter(1))
  else if aPattern = kw_PreComments then Result := dmData.getCommentsReport(getNotesContextFilter(2))
  else if aPattern = kw_VAComments then Result := dmData.getCommentsReport(getNotesContextFilter(3))
  else if aPattern = kw_FlowsheetComments then Result := dmData.getCommentsReport(getNotesContextFilter(4))
  else if aPattern = kw_PostComments then Result := dmData.getCommentsReport(getNotesContextFilter(5))
  else if aPattern = kw_SummaryComments then Result := dmData.getCommentsReport(getNotesContextFilter(6))
{$IFDEF AANTEST}
  else if pos(kw_Execute, aPattern) = 1 then Result := Engine.getRPCResults(aPattern)
{$ENDIF}
  else if PatternIsOption(aPattern) then Result := Preferences.sValue(aPattern) // zzzzzzandria 060613
  else begin
    Result := '';
    sValue := dmData.getFieldData(aPattern);
    if aPattern = kwc_RxLab then Result := dmData.getCommentsReport(getNotesContextFilter(1));
    if aPattern = kwc_Pre then Result := dmData.getCommentsReport(getNotesContextFilter(2));
    if aPattern = kwc_Post then Result := dmData.getCommentsReport(getNotesContextFilter(5));
    if aPattern = kwc_Summary then Result := dmData.getCommentsReport(getNotesContextFilter(6));
    sBlank := Preferences.sValue(onBlanksPlaceholder);
    if (sValue = sBlank) then begin
      if Result = '' then Result := sValue;
    end else begin
      if Result <> '' then Result := sValue + CRLF + Result
      else Result := sValue;
    end;
  end;
end;

function getExecutionValue(aTemplate: string): string;
var
  iBegin, iEnd: Integer;
  sBBegin, sBEnd, sHead, sTail, sTemplate, sPattern, sResult: string;
begin
  sTemplate := aTemplate;
  sResult := '';
  sBBegin := kw_Execute + '(';
  sBEnd := ')';
  try
    repeat
      iBegin := pos(sBBegin, sTemplate);
      if iBegin > 0 then begin
        sHead := copy(sTemplate, 1, iBegin - 1);
        sTail := copy(sTemplate, iBegin + Length(sBBegin), Length(sTemplate));
        iEnd := pos(sBEnd, sTail);
        if iEnd > 0 then begin
          sPattern := copy(sTail, 1, iEnd - 1);
          try
            sPattern := Engine.getRPCResults(sPattern);
          except
          end;
          sTail := copy(sTail, iEnd + Length(sBEnd), Length(sTail));
        end else begin
          sResult := sResult + sTemplate;
          break;
        end;
        sResult := sResult + sHead + sPattern;
        sTemplate := sTail;
      end else begin
        sResult := sResult + sTemplate;
        break;
      end;
    until sTemplate = '';
  except
  end;
  Result := sResult;
end;

function getReportValue(aTemplate: string): string;
var
  iBegin, iEnd: Integer;
  sHead, sTail, sTemplate, sPattern, sResult: string;
begin
  sTemplate := aTemplate;
  sResult := '';
  kwc_RxLab := getDictionaryTag('cdsSummary', fnS_LabNotes);
  kwc_Pre := getDictionaryTag('cdsSummary', fnS_PreNotes);
  kwc_Post := getDictionaryTag('cdsSummary', fnS_PostNotes);
  kwc_Summary := getDictionaryTag('cdsSummary', fnS_SummaryNotes);
  try
    repeat
      iBegin := pos(sBBegin, sTemplate);
      if iBegin > 0 then begin
        sHead := copy(sTemplate, 1, iBegin - 1);
        sTail := copy(sTemplate, iBegin + Length(sBBegin), Length(sTemplate));
        iEnd := pos(sBEnd, sTail);
        if iEnd > 0 then begin
          sPattern := copy(sTail, 1, iEnd - 1);
          try
            sPattern := Engine.getPatternValue(sPattern);
          except
            on E: Exception do ShowMessage('Error converting <' + sPattern + '>' + CRLF + E.Message);
          end;
          sTail := copy(sTail, iEnd + Length(sBEnd), Length(sTail));
        end else begin
          sResult := sResult + sTemplate;
          break;
        end;
        sResult := sResult + sHead + sPattern;
        sTemplate := sTail;
      end else begin
        sResult := sResult + sTemplate;
        break;
      end;
    until sTemplate = '';
  except
    on E: Exception do ShowMessage('Error converting <' + sPattern + '>' + CRLF + E.Message);
  end;
{$IFDEF AANTEST}
  sResult := getExecutionValue(sResult);
{$ENDIF}
  Result := setReportWidth(sResult, 80);
end;

function setReportWidth(aReport: string; Width: Integer = 80): string;
var
  sl: TStringList;
  s, sHead, sTail: string;
  ii, i: Integer;
begin
  sl := TStringList.Create;
  sl.Text := aReport;
  sTail := '';
  for i := 0 to sl.Count - 1 do begin
    s := sl[i];
    repeat
      sHead := copy(s, 1, Width);
      if Length(sHead) < Width then ii := Length(sHead)
      else
        for ii := Width downto 1 do
          if copy(sHead, ii, 1) = ' ' then break;
      if ii = 0 then ii := Length(sHead);
      sHead := copy(s, 1, ii);
      s := copy(s, ii + 1, Length(s));
      sTail := sTail + sHead + CRLF;
    until Length(s) = 0;
  end;
  Result := sTail;
  sl.Free;
end;

procedure PaintResults(aRE: TRichEdit; aPattern: string; aColor: TColor = clNavy; aBold: Boolean = False);
begin
  aRE.SelAttributes.Color := aColor;
  if aBold then aRE.SelAttributes.Style := [fsBold];
  aRE.SelText := aPattern;
end;

procedure PaintPattern(aRE: TRichEdit; aPattern: string; ColorBracket: TColor = clPurple; ColorKeyword: TColor = clNavy;
  ColorField: TColor = clGreen; ColorUnknown: TColor = clRed);
begin
  if PatternIsKeyword(aPattern) then begin
    aRE.SelAttributes.Color := ColorKeyword;
    aRE.SelAttributes.Style := [fsBold];
  end else if PatternIsField(aPattern) then begin
    aRE.SelAttributes.Color := ColorField;
    aRE.SelAttributes.Style := [fsBold];
  end else begin
    aRE.SelAttributes.Style := [fsBold];
    aRE.SelAttributes.Color := ColorUnknown;
  end;
end;

procedure PaintSyntax(aTemplate: TRichEdit; ShowBrackets: Boolean = True; anOpenBracket: string = '<';
  aCloseBracket: string = '>'; ColorBracket: TColor = clPurple; ColorResult: TColor = clNavy;
  ColorKeyword: TColor = clNavy; ColorField: TColor = clGreen; ColorUnknown: TColor = clRed);
var
  i, j, iPos: Integer;
  iBegin, iEnd: Integer;
  sTemplate, sResult, sPattern, sHead, sTail: string;
  reTemplate: TRichEdit;
  sBBegin, sBEnd: string;
  lCRLF: integer;
begin
  lCRLF := Length(CRLF);
  reTemplate := aTemplate;
  iPos := reTemplate.SelStart;
  reTemplate.PlainText := True;
  sHead := reTemplate.Text;
  sTemplate := sHead;
  sResult := '';
  i := 0;
  sBBegin := anOpenBracket;
  sBEnd := aCloseBracket;
  repeat
    iBegin := pos(sBBegin, sTemplate);
    j := pos(CRLF, sTemplate);
    if j < iBegin then begin
      sResult := sResult + copy(sTemplate, 1, j + 1);
      sTemplate := copy(sTemplate, j + lCRLF, Length(sTemplate));
      i := i + j;
//      i := i + j + 1;
    end else if iBegin > 0 then begin
      sHead := copy(sTemplate, 1, iBegin - 1);
      sTail := copy(sTemplate, iBegin + Length(sBBegin), Length(sTemplate));
      iEnd := pos(sBEnd, sTail);
      if iEnd > 0 then begin
        sPattern := copy(sTail, 1, iEnd - 1);
        // verify Pattern and Paint
        if not ShowBrackets then begin
          reTemplate.SelStart := i + iBegin - Length(sBBegin);
          reTemplate.SelLength := Length(sPattern) + Length(sBBegin) + Length(sBEnd);
//          reTemplate.SelLength := iEnd + length(sBEnd);
          if sBBegin = '[' then PaintResults(reTemplate, sPattern, clNavy, True) // Bold
          else PaintResults(reTemplate, sPattern);
          i := i - Length(sBBegin) - Length(sBEnd);
        end else begin
          reTemplate.SelStart := i + iBegin;
          reTemplate.SelLength := Length(sPattern);
          PaintPattern(reTemplate, sPattern);
        end;
        sTail := copy(sTail, iEnd + Length(sBEnd), Length(sTail));
      end else begin
        sResult := sResult + sTemplate;
        break;
      end;
      sResult := sResult + sHead + sPattern;
      i := i + Length(sHead) + Length(sPattern) + 2;
      if sBEnd = CRLF then i := i + 1;
      sTemplate := sTail;
    end else begin
      sResult := sResult + sTemplate;
      break;
    end;
  until sTemplate = '';
  reTemplate.PlainText := False;
  reTemplate.SelStart := iPos;
end;

procedure PrintReportString(aPrinter: TPrinter; aValue: string; imgBackground: TBitmap);
var
  sl: TStringList;
  rr: TRect;
  iPage, i, j, iTotal: Integer;
const
  LEFT_MARGIN  = 100;
  RIGHT_MARGIN = 180;
  LEFT_TOP_X   = 580;
  LEFT_TOP_Y   = 180;
  procedure DrawFrame(aR: TRect);
  begin
    Printer.Canvas.Brush.Color := clBlack;
    Printer.Canvas.FrameRect(aR);
    Printer.Canvas.Brush.Style := bsClear;
  end;
  procedure DrawBackground(aR: TRect);
  begin
    if imgBackground <> nil then Printer.Canvas.StretchDraw(aR, imgBackground);
  end;
  function SimpleBottomLine: string;
  begin
    Result := Format('*** Page %d of %d ***', [iPage, iTotal]);
  end;
  function SimpleHeaderLine: string;
  begin
    Result := Format('*** WORK COPY ONLY *** %s *** %s ***', [FormatDateTime('mm/dd/yyyy hh:mm:ss', Now),
      Engine.ExeNameAndVersion]);
  end;
  procedure ClosePage;
  var
    s: string;
    iPos: Integer;
  begin
    s := SimpleBottomLine;
    iPos := Printer.PageWidth - Printer.Canvas.TextWidth(s) - RIGHT_MARGIN;
    Printer.Canvas.TextOut(iPos, Printer.PageHeight - 100, s);
    Printer.Canvas.TextOut(LEFT_TOP_X, 60, SimpleHeaderLine);
    inc(iPage);
  end;
  function TotalPages: Integer;
  var
    jBlank, jCount, jHeight, jPage, i: Integer;
  begin
    Result := 1;
    if sl.Count <= 1 then Exit;
    jBlank := 0;
    jCount := 0;
    jPage := 1;
    for i := 0 to sl.Count - 1 do begin
      if (jBlank = 0) and (trim(sl[i]) <> '') then jBlank := Printer.Canvas.TextHeight(sl[i]);
      jHeight := Printer.Canvas.TextHeight(sl[i]);
      if jHeight = 0 then jHeight := jBlank;
      if jHeight = 0 then jHeight := 88; // the default value;
      if jCount + jHeight > Printer.PageHeight - LEFT_TOP_Y * 2 then begin
        inc(jPage);
        jCount := jHeight
      end
      else jCount := jCount + jHeight;
    end;
    Result := jPage;
  end;

begin
  with Printer do begin
    sl := TStringList.Create;
    sl.Text := aValue;

    // Printer.Canvas.Font.Assign(mmReport.Font);
    iTotal := TotalPages;
    rr := Rect(LEFT_TOP_X, LEFT_TOP_Y, (PageWidth - LEFT_TOP_Y), (PageHeight - LEFT_TOP_Y));
    BeginDoc;
    DrawBackground(rr);
    Canvas.Brush.Style := bsClear;
    j := 0;
    iPage := 1;
    for i := 0 to sl.Count - 1 do begin
      if ((j + 1) * Canvas.TextHeight(sl[i])) > (PageHeight - LEFT_TOP_Y * 2) then { 400 }
      begin
        DrawFrame(rr);
        ClosePage;
        Newpage;
        DrawBackground(rr);
        j := 0;
      end;
      Canvas.TextOut(LEFT_TOP_X + 40, LEFT_TOP_Y + 40 + (j * Canvas.TextHeight(sl[i])), sl[i]);
      inc(j);
    end;
    DrawFrame(rr);
    ClosePage;
    EndDoc;
    sl.Free;
  end;
end;

end.
