unit uMDK_XMLDocument;

interface

uses
  SysUtils, Classes, xmldom, XMLIntf, msxmldom, XMLDoc
  , Dialogs, oxmldom,    Controls
  , uMDK_Patients, xCheckBox, xYesNo, xListView, xEdit, xPanel, xComboBox
  ;

type
  TdmHemodialysis = class(TDataModule)
    XMLDoc: TXMLDocument;
    procedure DataModuleCreate(Sender: TObject);
  private
    { Private declarations }
    // Test nodes ==============================================================
    fXMLAlert: IXMLNode;
    fXMLxcb: IXMLNode;
    fXMLYesno: IXMLNode;

    fXMLList: IXMLNode;
    fXMLEdit: IXMLNode;
    fXMLPanel: IXMLNode;

    fTreatmentRecord: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  Cover Sheet  ///////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // Treatment Information ===================================================
    fTreatmentInfo: IXMLNode;

    fToday: IXMLNode;
    fESDRDiagnosis: IXMLNode;

    fDiagnosisDate: IXMLNode;
    fTherapyDate: IXMLNode;
    fModality: IXMLNode;
    fCodeStatus: IXMLNode;
    fNephrologist: IXMLNode;
    fAdvancedDirectives: IXMLNode;

    fSchedule: IXMLNode;

    // Vaccination =============================================================

    fHepBAntibodyDate: IXMLNode;
    fHepBAntibodyResult: IXMLNode;
    fHepBAntigenDate: IXMLNode;
    fHepBAntigenResult: IXMLNode;

    fHepCAntibodyDate: IXMLNode;
    fHepCAntibodyResult: IXMLNode;

    fPPDDate: IXMLNode;
    fPPDResult: IXMLNode;

    fInfluenzaDate: IXMLNode;
    fPneumovaxDate: IXMLNode;

    // TreatmentHistory ========================================================
    fHistoryDate: IXMLNode;

    // Weight
    fHistoryWeightPre: IXMLNode;
    fHistoryWeightPost: IXMLNode;
    fHistoryWeightGoal: IXMLNode;

    // BP
    fHistoryBPPreSeated: IXMLNode;
    fHistoryBPPreStanding: IXMLNode;
    fHistoryBPPostSeated: IXMLNode;
    fHistoryBPPostStanding: IXMLNode;

    //Temp
    fHistoryTempPre: IXMLNode;
    fHistoryTempPost: IXMLNode;

    //Pulse
    fHistoryPulsePre: IXMLNode;
    fHistoryPulsePost: IXMLNode;

    // Vascular Access Monitoring ==============================================
    // Intra-Access Blood Flow
    fVAMOneDate: IXMLNode;
    fVAMOneResult: IXMLNode;

    fVAMTwoDate: IXMLNode;
    fVAMTwoResult: IXMLNode;

    fVAMThreeDate: IXMLNode;
    fVAMThreeResult: IXMLNode;

    // StaticVenoue Pressure
    fSVPOneBFR: IXMLNode;
    fSVPOneAVP: IXMLNode;
    fSVPOneVP: IXMLNode;

    fSVPTwoBFR: IXMLNode;
    fSVPTwoAVP: IXMLNode;
    fSVPTwoVP: IXMLNode;

    fSVPThreeBFR: IXMLNode;
    fSVPThreeAVP: IXMLNode;
    fSVPThreeVP: IXMLNode;

    // Dynamic Venous Pressure
    fDVPCurrentAverage: IXMLNode;

    fDVPOneDate: IXMLNode;
    fDVPOneResult: IXMLNode;

    fDVPTwoDate: IXMLNode;
    fDVPTwoResult: IXMLNode;

    fDVPThreeDate: IXMLNode;
    fDVPThreeResult: IXMLNode;

    // Postings ================================================================
    fPostings: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  RxAndLabs  /////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // Order
    // fOrder: IXMLNode;
    fOrderDialyzer: IXMLNode;
    fOrderReuseNumber: IXMLNode;
    fOrderTxLength:IXMLNode;
    fOrderBFR: IXMLNode;
    fOrderDialysateFlow: IXMLNode;
    fOrderTemp: IXMLNode;

    // Dialysate Formula
//  fFormula: IXMLNode;
    fFormulaK: IXMLNode;
    fFormulaHCO3: IXMLNode;
    fFormulaNa: IXMLNode;
    fFormulaCa: IXMLNode;

    //Heparin
  //    fHeparin: IXMLNode;
    fHeparinLoad: IXMLNode;
    fHeparinUnits: IXMLNode;
    fHeparinEndTime: IXMLNode;

    // Modeling
//    fModeling: IXMLNode;
    fModelingNa: IXMLNode;
    fModelingHCO3: IXMLNode;
    fModelingUltrafiltration: IXMLNode;

    // Other
//    fRxOther: IXMLNode;
    fRxOtherAnticoagulants: IXMLNode;
    fRxOtherOrdes: IXMLNode;

    // Labs ====================================================================
    fLabsList: IXMLNode;
    // Microbiology ============================================================
    fMicrobiologyList: IXMLNode;
    // Notes ===================================================================
    fRxNotes: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  Pre-Treatment  /////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // PreAssesments ===========================================================
    // Weight
    fPreWeight: IXMLNode;
    fPreWeightDry: IXMLNode;

    // Temperature
    fPreTemperature: IXMLNode;

    // Pulse
    fPrePulse: IXMLNode;

    // Pain
    fPrePain: IXMLNode;
    fPrePainValue: IXMLNode;

    // BP
    fPreBPSeatedSystolic: IXMLNode;
    fPreBPSeatedDiastolic: IXMLNode;
    fPreBPStandingSystolic: IXMLNode;
    fPreBPStandingDiastolic: IXMLNode;

    // MenthalStatus
    fPreMSAlert: IXMLNode;
    fPreMSConfused: IXMLNode;
    fPreMSSedated: IXMLNode;
    fPreMSUnresponsive: IXMLNode;
    fPreMLethargic: IXMLNode;
    fPreMRestless: IXMLNode;

    // Other
    fPreEdema: IXMLNode;
    fPreRespirations: IXMLNode;
    fPreSOB: IXMLNode;

    // Safety Chacks ===========================================================
    fSCOrdersVerified: IXMLNode;
    fSCAlarmChecked: IXMLNode;
    fSCDisinfectant: IXMLNode;

    // Notes ===================================================================
    fPreAssesmentNotes: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  Access History  ////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    fAHNodtes: IXMLNode;

    fAHSelectedPoints: IXMLNode;// List Of the Points selected for procedure
    fAHStatuses: IXMLNode; // list of descriptions of the available access points

    ////////////////////////////////////////////////////////////////////////////
    //  FlowSheet  /////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    fFlowsheetNotes: IXMLNode;

    fFlowsheetItems: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  Post-Treatment  /////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // PostAssesments ===========================================================
    // Weight
    fPostWeight: IXMLNode;
    fPostWeightDry: IXMLNode;

    // Temperature
    fPostTemperature: IXMLNode;

    // Pulse
    fPostPulse: IXMLNode;

    // Pain
    fPostPain: IXMLNode;
    fPostPainValue: IXMLNode;

    // BP
    fPostBPSeatedSystolic: IXMLNode;
    fPostBPSeatedDiastolic: IXMLNode;
    fPostBPStandingSystolic: IXMLNode;
    fPostBPStandingDiastolic: IXMLNode;

    // MenthalStatus
    fPostMSAlert: IXMLNode;
    fPostMSConfused: IXMLNode;
    fPostMSSedated: IXMLNode;
    fPostMSUnresponsive: IXMLNode;
    fPostMLethargic: IXMLNode;
    fPostMRestless: IXMLNode;

    // Other
    fPostEdema: IXMLNode;
    fPostRespirations: IXMLNode;
    fPostSOB: IXMLNode;

    // Observations ============================================================
    fObsWeightAchieved: IXMLNode;
    fObsMedicineAdministered: IXMLNode;
    fObsNoProblem: IXMLNode;
    fObsVomiting: IXMLNode;
    fObsHypotension: IXMLNode;
    fObsSyncope: IXMLNode;
    fObsOther: IXMLNode;
    fObsOtherText: IXMLNode;

    // Notes ===================================================================
    fPostAssesmentNotes: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  Summary  ///////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    fTreatmentStarted: IXMLNode;
    fTreatmentStoped: IXMLNode;

    fTotalUF: IXMLNode;
    fTotalLP: IXMLNode;
    fTotalKT: IXMLNode;
    fTotalKTV: IXMLNode;

    fMeanURF: IXMLNode;
    fMeanTMP: IXMLNode;
    fMeanTemp: IXMLNode;
    fMeanConductivity: IXMLNode;

    fAverageBFR: IXMLNode;
    fAverageDFR: IXMLNode;

    fSummaryNotes: IXMLNode;

  public
    { Public declarations }
    // Test Properties =========================================================
    property XMLAlert: IXMLNode read fXMLAlert;
    property XMLxcb: IXMLNode read fXMLxcb;
    property XMLyesno: IXMLNode read fXMLyesno;
    property XMLList: IXMLNode read fXMLList;
    property XMLEdit: IXMLNode read fXMLEdit;
    property XMLPanel: IXMLNode read fXMLPanel;
    // Test Properties ===================================================== End
    property TreatmentRecord: IXMLNode read fTreatmentRecord write fTreatmentRecord;
    property TreatmentInfo: IXMLNode read fTreatmentInfo write fTreatmentInfo;
    property Schedule: IXMLNode read fSchedule write fSchedule;

    procedure BindDocToControl(const aControl:TWinControl);
    procedure BindNodetoControl(aNode: IXMLNode;const aControl:TWinControl);

    procedure createNodes;
  end;

var
  dmHemodialysis: TdmHemodialysis;

const
    cfTreatmentRecord = 'TreatmentRecord';

    ////////////////////////////////////////////////////////////////////////////
    //  Cover Sheet  ///////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // Treatment Information ===================================================
    cfTreatmentInfo = 'TreatmentInfo';
    cfToday = 'TreatmentDate';
    cfESDRDiagnosis = 'ESDRDiagnosis';
    cfDiagnosisDate = 'DiagnosisDate';
    cfTherapyDate = 'TherapyDate';
    cfModality = 'Modality';
    cfCodeStatus = 'CodeStatus';
    cfNephrologist = 'Nephrologist';
    cfAdvancedDirectives = 'AdvancedDirectives';

    cfSchedule = 'Schedule';

    // Vaccination =============================================================

    cfHepBAntibodyDate = '';
    cfHepBAntibodyResult = '';
    cfHepBAntigenDate = '';
    cfHepBAntigenResult = '';

    cfHepCAntibodyDate = '';
    cfHepCAntibodyResult = '';

    cfPPDDate = '';
    cfPPDResult = '';

    cfInfluenzaDate = '';
    cfPneumovaxDate = '';

    // TreatmentHistory ========================================================
    cfHistoryDate = '';

    // Weight
    cfHistoryWeightPre = '';
    cfHistoryWeightPost = '';
    cfHistoryWeightGoal = '';

    // BP
    cfHistoryBPPreSeated = '';
    cfHistoryBPPreStanding = '';
    cfHistoryBPPostSeated = '';
    cfHistoryBPPostStanding = '';

    //Temp
    cfHistoryTempPre = '';
    cfHistoryTempPost = '';

    //Pulse
    cfHistoryPulsePre = '';
    cfHistoryPulsePost = '';

    // Vascular Access Monitoring ==============================================
    // Intra-Access Blood Flow
    cfVAMOneDate = '';
    cfVAMOneResult = '';

    cfVAMTwoDate = '';
    cfVAMTwoResult = '';

    cfVAMThreeDate = '';
    cfVAMThreeResult = '';

    // StaticVenoue Pressure
    cfSVPOneBFR = '';
    cfSVPOneAVP = '';
    cfSVPOneVP = '';

    cfSVPTwoBFR = '';
    cfSVPTwoAVP = '';
    cfSVPTwoVP = '';

    cfSVPThreeBFR = '';
    cfSVPThreeAVP = '';
    cfSVPThreeVP = '';

    // Dynamic Venous Pressure
    cfDVPCurrentAverage = '';

    cfDVPOneDate = '';
    cfDVPOneResult = '';

    cfDVPTwoDate = '';
    cfDVPTwoResult = '';

    cfDVPThreeDate = '';
    cfDVPThreeResult = '';

    // Postings ================================================================
    cfPostings = '';

    ////////////////////////////////////////////////////////////////////////////
    //  RxAndLabs  /////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // Order
    // fOrder = '';
    cfOrderDialyzer = '';
    cfOrderReuseNumber = '';
    cfOrderTxLength = '';
    cfOrderBFR = '';
    cfOrderDialysateFlow = '';
    cfOrderTemp = '';

    // Dialysate Formula
//  fFormula = '';
    cfFormulaK = '';
    cfFormulaHCO3 = '';
    cfFormulaNa = '';
    cfFormulaCa = '';

    //Heparin
  //    cfHeparin = '';
    cfHeparinLoad = '';
    cfHeparinUnits = '';
    cfHeparinEndTime = '';

    // Modeling
//    cfModeling = '';
    cfModelingNa = '';
    cfModelingHCO3 = '';
    cfModelingUltrafiltration = '';

    // Other
//    cfRxOther = '';
    cfRxOtherAnticoagulants = '';
    cfRxOtherOrdes = '';

    // Labs ====================================================================
    cfLabsList = '';
    // Microbiology ============================================================
    cfMicrobiologyList = '';
    // Notes ===================================================================
    cfRxNotes = '';

    ////////////////////////////////////////////////////////////////////////////
    //  Pre-Treatment  /////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // PreAssesments ===========================================================
    // Weight
    cfPreWeight = '';
    cfPreWeightDry = '';

    // Temperature
    cfPreTemperature = '';

    // Pulse
    cfPrePulse = '';

    // Pain
    cfPrePain = '';
    cfPrePainValue = '';

    // BP
    cfPreBPSeatedSystolic = '';
    cfPreBPSeatedDiastolic = '';
    cfPreBPStandingSystolic = '';
    cfPreBPStandingDiastolic = '';

    // MenthalStatus
    cfPreMSAlert = '';
    cfPreMSConfused = '';
    cfPreMSSedated = '';
    cfPreMSUnresponsive = '';
    cfPreMLethargic = '';
    cfPreMRestless = '';

    // Other
    cfPreEdema = '';
    cfPreRespirations = '';
    cfPreSOB = '';

    // Safety Chacks ===========================================================
    cfSCOrdersVerified = '';
    cfSCAlarmChecked = '';
    cfSCDisinfectant = '';

    // Notes ===================================================================
    cfPreAssesmentNotes = '';

    ////////////////////////////////////////////////////////////////////////////
    //  Access History  ////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    cfAHNodtes = '';

    cfAHSelectedPoints = '';// List Of the Points selected for procedure
    cfAHStatuses = ''; // list of descriptions of the available access points

    ////////////////////////////////////////////////////////////////////////////
    //  FlowSheet  /////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    cfFlowsheetNotes = '';

    cfFlowsheetItems = '';

    ////////////////////////////////////////////////////////////////////////////
    //  Post-Treatment  /////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // PostAssesments ===========================================================
    // Weight
    cfPostWeight = '';
    cfPostWeightDry = '';

    // Temperature
    cfPostTemperature = '';

    // Pulse
    cfPostPulse = '';

    // Pain
    cfPostPain = '';
    cfPostPainValue = '';

    // BP
    cfPostBPSeatedSystolic = '';
    cfPostBPSeatedDiastolic = '';
    cfPostBPStandingSystolic = '';
    cfPostBPStandingDiastolic = '';

    // MenthalStatus
    cfPostMSAlert = '';
    cfPostMSConfused = '';
    cfPostMSSedated = '';
    cfPostMSUnresponsive = '';
    cfPostMLethargic = '';
    cfPostMRestless = '';

    // Other
    cfPostEdema = '';
    cfPostRespirations = '';
    cfPostSOB = '';

    // Observations ============================================================
    cfObsWeightAchieved = '';
    cfObsMedicineAdministered = '';
    cfObsNoProblem = '';
    cfObsVomiting = '';
    cfObsHypotension = '';
    cfObsSyncope = '';
    cfObsOther = '';
    cfObsOtherText = '';

    // Notes ===================================================================
    cfPostAssesmentNotes = '';

    ////////////////////////////////////////////////////////////////////////////
    //  Summary  ///////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    cfTreatmentStarted = '';
    cfTreatmentStoped = '';

    cfTotalUF = '';
    cfTotalLP = '';
    cfTotalKT = '';
    cfTotalKTV = '';

    cfMeanURF = '';
    cfMeanTMP = '';
    cfMeanTemp = '';
    cfMeanConductivity = '';

    cfAverageBFR = '';
    cfAverageDFR = '';

    cfSummaryNotes = '';


implementation

uses I_XML
  , uAttributenames, frmShellMain;

{$R *.dfm}
procedure TdmHemodialysis.DataModuleCreate(Sender: TObject);
begin
  CreateNodes;
end;

procedure TdmHemodialysis.createNodes;
var
  IRoot,
  IParent,
  INode: IXMLNode;

  function CreateNode(aParent: IXMLNode;aTag,aValue,aGuiClass, aGUIname:String): IXMLNode;
  var
   aNode: IXMLNode;
  begin
    aNode := nil;
    try
      aNode := aParent.AddChild(aTag);
      aNode.NodeValue := aValue;
      aNode.Attributes[anGuitype] := aGUIClass;
      aNode.Attributes[anGUIname] := aGUIName;
    except
    end;
    Result := aNode;
  end;

  procedure AddText(aParent:IXMLNode;aTag,aValue:String);
  var
    aN: IXMLNode;
  begin
    aN := XMLDoc.CreateNode(aTag, ntText, aValue);
    aParent.ChildNodes.Add(aN);
  end;

  function AddElement(aParent: IXMLNode; aTag,aValue:String): IXMLNode;
  var
    aN: IXMLNode;
  begin
    try
      aN := XMLDoc.CreateNode(aTag, ntElement,'');
      aN.NodeValue := aValue;
      aParent.ChildNodes.Add(aN);
      result := aN;
    except
      Result := nil;
    end;
  end;

begin
  try
    XMLDoc.Active := True;
    IRoot := XMLDoc.AddChild('ROOT');
      IRoot.NodeValue := 'The Root';
      IRoot.Attributes[ntRoot] := 'TXMLDOC';

    fTreatmentRecord := CreateNode(XMLDoc.DocumentElement,cfTreatmentRecord,cfTreatmentRecord,'TreatmentRecord','TreatmentRecord');
    ////////////////////////////////////////////////////////////////////////////
    //  Cover Sheet  ///////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // Treatment Information ===================================================
    fTreatmentInfo := CreateNode(TreatmentRecord,cfTreatmentInfo,cfTreatmentInfo,'TxTitledPanel','xTreatmentInfo');

        fToday := CreateNode(fTreatmentInfo,cfToday,'TTT','TxEdit','xEditTest');
          fToday.Attributes[caXEditCaption] := 'Test Caption';
          fToday.Attributes[caXEditUnit] := 'Test Unit';

        fESDRDiagnosis := CreateNode(fTreatmentInfo,cfESDRDiagnosis,'Initial Value','TxComboBox','xESDRDiagnosis');
        fESDRDiagnosis.Attributes[anXComboBoxCaption] := 'ESDRDiagnosis';
        fESDRDiagnosis.Attributes[anXComboBoxUnit] := ' ';
          iParent := AddElement(fESDRDiagnosis,tnXComboCode,'Test Value for the Node Code');
          iParent := AddElement(fESDRDiagnosis,tnXComboText,'Test Value for the Node Text');
          iParent := AddElement(fESDRDiagnosis,tnXComboBoxCodes,tnXComboBoxCodes);
            iNode := AddElement(iParent,'Line1','Code1');
            iNode := AddElement(iParent,'Line1','Code1');
            iNode := AddElement(iParent,'Line12','Code12');
            iNode := AddElement(iParent,'Line13','Code13');

        fDiagnosisDate := CreateNode(fTreatmentInfo,cfDiagnosisDate,'Value','a','s');
            iParent := AddElement(fDiagnosisDate,tnXComboBoxCodes,tnXComboBoxCodes);
            iNode := AddElement(iParent,'NewElement','New Element');
            iNode := CreateNode(fESDRDiagnosis,'Line 2','Code2','1','2');
            iNode := CreateNode(iParent,'Line 3','Code3','1','2');

        fTherapyDate := CreateNode(fTreatmentInfo,cfTherapyDate,'Value','a','s');

        fModality := CreateNode(fTreatmentInfo,cfModality,'Value','TxComboBox','xModality');
          fModality.Attributes[anXComboBoxCaption] := 'Modality';
          fModality.Attributes[anXComboBoxUnit] := ' ';
          iParent := AddElement(fModality,tnXComboCode,'Default Doc Code');
          iParent := AddElement(fModality,tnXComboText,'Default Doc Text');
          iParent := AddElement(fModality,tnXComboBoxCodes,tnXComboBoxCodes);
            iNode := AddElement(iParent,'HD','HD');

        fCodeStatus := CreateNode(fTreatmentInfo,cfCodeStatus,'Value','TxComboBox','xCodeStatus');
          fCodeStatus.Attributes[anXComboBoxCaption] := 'CodeStatus';
          fCodeStatus.Attributes[anXComboBoxUnit] := ' ';
          iParent := AddElement(fCodeStatus,tnXComboCode,'Default Doc Code');
          iParent := AddElement(fCodeStatus,tnXComboText,'Default Doc Text');
          iParent := AddElement(fCodeStatus,tnXComboBoxCodes,tnXComboBoxCodes);
            iNode := AddElement(iParent,'Full','Full');

        fNephrologist := CreateNode(fTreatmentInfo,cfNephrologist,'Value','TxComboBox','xNephrologist');
          fNephrologist.Attributes[anXComboBoxCaption] := 'Nephrologist';
          fNephrologist.Attributes[anXComboBoxUnit] := ' ';
          iParent := AddElement(fNephrologist,tnXComboCode,'Default Doc Code');
          iParent := AddElement(fNephrologist,tnXComboText,'Default Doc Text');
          iParent := AddElement(fNephrologist,tnXComboBoxCodes,tnXComboBoxCodes);
            iNode := AddElement(iParent,'Code1','Nephrolo  11');
            iNode := AddElement(iParent,'Code2','Nephrolo  22');
            iNode := AddElement(iParent,'Code3','Nephrolo  33');

        fAdvancedDirectives := CreateNode(fTreatmentInfo,cfAdvancedDirectives,'No','TxYesNo','xAdvanced');

    // Schedule ================================================================
    fSchedule := CreateNode(TreatmentRecord,cfSchedule,cfSchedule,'TxTitledPanel','xSchedule');
      iNode := CreateNode(fSchedule,'Monday','Unchecked','TxCheckBox','xMon');
      iNode := CreateNode(fSchedule,'Tuesday','Unchecked','TxCheckBox','xTue');
      iNode := CreateNode(fSchedule,'Wednesday','Unchecked','TxCheckBox','xWed');
      iNode := CreateNode(fSchedule,'Thursday','Unchecked','TxCheckBox','xThu');
      iNode := CreateNode(fSchedule,'Friday','Unchecked','TxCheckBox','xFri');
      iNode := CreateNode(fSchedule,'Saturday','Unchecked','TxCheckBox','xSat');
      iNode := CreateNode(fSchedule,'Sunday','Unchecked','TxCheckBox','xSun');
(*
    // Vaccination =============================================================

    fHepBAntibodyDate: IXMLNode;
    fHepBAntibodyResult: IXMLNode;
    fHepBAntigenDate: IXMLNode;
    fHepBAntigenResult: IXMLNode;

    fHepCAntibodyDate: IXMLNode;
    fHepCAntibodyResult: IXMLNode;

    fPPDDate: IXMLNode;
    fPPDResult: IXMLNode;

    fInfluenzaDate: IXMLNode;
    fPneumovaxDate: IXMLNode;

    // TreatmentHistory ========================================================
    fHistoryDate: IXMLNode;

    // Weight
    fHistoryWeightPre: IXMLNode;
    fHistoryWeightPost: IXMLNode;
    fHistoryWeightGoal: IXMLNode;

    // BP
    fHistoryBPPreSeated: IXMLNode;
    fHistoryBPPreStanding: IXMLNode;
    fHistoryBPPostSeated: IXMLNode;
    fHistoryBPPostStanding: IXMLNode;

    //Temp
    fHistoryTempPre: IXMLNode;
    fHistoryTempPost: IXMLNode;

    //Pulse
    fHistoryPulsePre: IXMLNode;
    fHistoryPulsePost: IXMLNode;

    // Vascular Access Monitoring ==============================================
    // Intra-Access Blood Flow
    fVAMOneDate: IXMLNode;
    fVAMOneResult: IXMLNode;

    fVAMTwoDate: IXMLNode;
    fVAMTwoResult: IXMLNode;

    fVAMThreeDate: IXMLNode;
    fVAMThreeResult: IXMLNode;

    // StaticVenoue Pressure
    fSVPOneBFR: IXMLNode;
    fSVPOneAVP: IXMLNode;
    fSVPOneVP: IXMLNode;

    fSVPTwoBFR: IXMLNode;
    fSVPTwoAVP: IXMLNode;
    fSVPTwoVP: IXMLNode;

    fSVPThreeBFR: IXMLNode;
    fSVPThreeAVP: IXMLNode;
    fSVPThreeVP: IXMLNode;

    // Dynamic Venous Pressure
    fDVPCurrentAverage: IXMLNode;

    fDVPOneDate: IXMLNode;
    fDVPOneResult: IXMLNode;

    fDVPTwoDate: IXMLNode;
    fDVPTwoResult: IXMLNode;

    fDVPThreeDate: IXMLNode;
    fDVPThreeResult: IXMLNode;

    // Postings ================================================================
    fPostings: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  RxAndLabs  /////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // Order
    // fOrder: IXMLNode;
    fOrderDialyzer: IXMLNode;
    fOrderReuseNumber: IXMLNode;
    fOrderTxLength:IXMLNode;
    fOrderBFR: IXMLNode;
    fOrderDialysateFlow: IXMLNode;
    fOrderTemp: IXMLNode;

    // Dialysate Formula
//  fFormula: IXMLNode;
    fFormulaK: IXMLNode;
    fFormulaHCO3: IXMLNode;
    fFormulaNa: IXMLNode;
    fFormulaCa: IXMLNode;

    //Heparin
  //    fHeparin: IXMLNode;
    fHeparinLoad: IXMLNode;
    fHeparinUnits: IXMLNode;
    fHeparinEndTime: IXMLNode;

    // Modeling
//    fModeling: IXMLNode;
    fModelingNa: IXMLNode;
    fModelingHCO3: IXMLNode;
    fModelingUltrafiltration: IXMLNode;

    // Other
//    fRxOther: IXMLNode;
    fRxOtherAnticoagulants: IXMLNode;
    fRxOtherOrdes: IXMLNode;

    // Labs ====================================================================
    fLabsList: IXMLNode;
    // Microbiology ============================================================
    fMicrobiologyList: IXMLNode;
    // Notes ===================================================================
    fRxNotes: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  Pre-Treatment  /////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // PreAssesments ===========================================================
    // Weight
    fPreWeight: IXMLNode;
    fPreWeightDry: IXMLNode;

    // Temperature
    fPreTemperature: IXMLNode;

    // Pulse
    fPrePulse: IXMLNode;

    // Pain
    fPrePain: IXMLNode;
    fPrePainValue: IXMLNode;

    // BP
    fPreBPSeatedSystolic: IXMLNode;
    fPreBPSeatedDiastolic: IXMLNode;
    fPreBPStandingSystolic: IXMLNode;
    fPreBPStandingDiastolic: IXMLNode;

    // MenthalStatus
    fPreMSAlert: IXMLNode;
    fPreMSConfused: IXMLNode;
    fPreMSSedated: IXMLNode;
    fPreMSUnresponsive: IXMLNode;
    fPreMLethargic: IXMLNode;
    fPreMRestless: IXMLNode;

    // Other
    fPreEdema: IXMLNode;
    fPreRespirations: IXMLNode;
    fPreSOB: IXMLNode;

    // Safety Chacks ===========================================================
    fSCOrdersVerified: IXMLNode;
    fSCAlarmChecked: IXMLNode;
    fSCDisinfectant: IXMLNode;

    // Notes ===================================================================
    fPreAssesmentNotes: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  Access History  ////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    fAHNodtes: IXMLNode;

    fAHSelectedPoints: IXMLNode;// List Of the Points selected for procedure
    fAHStatuses: IXMLNode; // list of descriptions of the available access points

    ////////////////////////////////////////////////////////////////////////////
    //  FlowSheet  /////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    fFlowsheetNotes: IXMLNode;

    fFlowsheetItems: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  Post-Treatment  /////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    // PostAssesments ===========================================================
    // Weight
    fPostWeight: IXMLNode;
    fPostWeightDry: IXMLNode;

    // Temperature
    fPostTemperature: IXMLNode;

    // Pulse
    fPostPulse: IXMLNode;

    // Pain
    fPostPain: IXMLNode;
    fPostPainValue: IXMLNode;

    // BP
    fPostBPSeatedSystolic: IXMLNode;
    fPostBPSeatedDiastolic: IXMLNode;
    fPostBPStandingSystolic: IXMLNode;
    fPostBPStandingDiastolic: IXMLNode;

    // MenthalStatus
    fPostMSAlert: IXMLNode;
    fPostMSConfused: IXMLNode;
    fPostMSSedated: IXMLNode;
    fPostMSUnresponsive: IXMLNode;
    fPostMLethargic: IXMLNode;
    fPostMRestless: IXMLNode;

    // Other
    fPostEdema: IXMLNode;
    fPostRespirations: IXMLNode;
    fPostSOB: IXMLNode;

    // Observations ============================================================
    fObsWeightAchieved: IXMLNode;
    fObsMedicineAdministered: IXMLNode;
    fObsNoProblem: IXMLNode;
    fObsVomiting: IXMLNode;
    fObsHypotension: IXMLNode;
    fObsSyncope: IXMLNode;
    fObsOther: IXMLNode;
    fObsOtherText: IXMLNode;

    // Notes ===================================================================
    fPostAssesmentNotes: IXMLNode;

    ////////////////////////////////////////////////////////////////////////////
    //  Summary  ///////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    fTreatmentStarted: IXMLNode;
    fTreatmentStoped: IXMLNode;

    fTotalUF: IXMLNode;
    fTotalLP: IXMLNode;
    fTotalKT: IXMLNode;
    fTotalKTV: IXMLNode;

    fMeanURF: IXMLNode;
    fMeanTMP: IXMLNode;
    fMeanTemp: IXMLNode;
    fMeanConductivity: IXMLNode;

    fAverageBFR: IXMLNode;
    fAverageDFR: IXMLNode;

    fSummaryNotes: IXMLNode;
*)
  except
  end;
end;
////////////////////////////////////////////////////////////////////////////////

procedure TdmHemodialysis.BindNodeToControl(aNode: IXMLNode;const aControl:TWinControl);
var
  i: Integer;
  iNode: IXMLNode;
  theControl: TWinControl;

  function FindControl(aStart:TWinControl;aName,aType:String): TWinControl;
  var
    j: integer;
    aC: TWinControl;
  begin
    Result := nil;
    if (aStart.ClassName = aType) and (aStart.Name = aName) then
        Result := aStart
    else
      for j := 0 to aStart.ControlCount - 1 do
        begin
          if aStart.Controls[j] is TWinControl then
            begin
              aC := FindControl(TWinControl(aStart.Controls[j]),aName,aType);
              if aC <> nil then
                begin
                  Result := aC;
                  Exit;
                end;
              end;
        end;
  end;

begin
  TheControl := FindControl(aControl,aNode.Attributes[anGUIname],aNode.Attributes[anGUItype]);
  if theControl = nil then
    begin
      for i := 0 to aNode.ChildNodes.Count - 1 do
        begin
          iNode := aNode.ChildNodes[i];
          if iNode.NodeType <> ntElement then Continue
          else
            BindNodeToControl(iNode,aControl);
        end;
//      if i = aNode.ChildNodes.Count then  exit;
    end
  else
    begin
      if TheControl is TxPanel then TxPanel(TheControl).XMLNode := aNode
      else if TheControl is TxEdit then TxEdit(TheControl).XMLNode := aNode
      else if TheControl is TxYesNo then TxYesNo(TheControl).XMLNode := aNode
      else if TheControl is TxListView then TxListView(TheControl).XMLNode := aNode
      else if TheControl is TxCheckBox then TxCheckBox(TheControl).XMLNode := aNode
    end;
end;

procedure TdmHemodialysis.BindDocToControl(const aControl:TWinControl);
var
  i: Integer;
  ix: IXMLNode;
  sName,sType:String;
begin
  ix := XMLDoc.DocumentElement;
  for i := 0 to ix.ChildNodes.Count - 1 do
    begin
      sName := ix.ChildNodes[i].Attributes[anGUIname];
      sType := ix.ChildNodes[i].Attributes[anGUIType];
      if (sName <>'') and (sType <>'') then
        BindNodeToControl(ix.ChildNodes[i],aControl);
    end;
end;


end.


