unit fMDK_Submit;
{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 4/22/08 9:47a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  Contains the Application Submit form
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Tabs/fMDK_Submit.pas $
  *
  *       $ History: fMDK_Submit.pas $
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Buttons, StdCtrls, ActnList, Grids, DBGrids, ImgList,
  ComCtrls, Menus, System.Actions;

type
  TfrmSubmit = class(TForm)
    pnlSubmit: TPanel;
    pnlChecklist: TPanel;
    alSubmit: TActionList;
    acSubmit: TAction;
    ilStatus: TImageList;
    dbgStatus: TDBGrid;
    Panel4: TPanel;
    pnlItemList: TPanel;
    Panel3: TPanel;
    pnlSignature: TPanel;
    Label6: TLabel;
    lblOperator: TLabel;
    edSignature: TEdit;
    mmReport: TMemo;
    pnlSubmitActions: TPanel;
    Splitter1: TSplitter;
    acAddNote: TAction;
    acNoteList: TAction;
    PopupMenu1: TPopupMenu;
    ReadOnly1: TMenuItem;
    acGoTo: TAction;
    Panel1: TPanel;
    acPrint: TAction;
    PrintDialog1: TPrintDialog;
    Panel5: TPanel;
    tvNotes: TTreeView;
    imgDemoBackground: TImage;
    Panel17: TPanel;
    Panel8: TPanel;
    pnlDocuments: TPanel;
    Panel13: TPanel;
    bvlSignature: TBevel;
    acSignAndSubmit: TAction;
    pnlSubmitOnly: TPanel;
    bbSubmit: TBitBtn;
    pnlSignSubmit: TPanel;
    BitBtn1: TBitBtn;
    pnlLegend: TPanel;
    Image1: TImage;
    Image2: TImage;
    Image3: TImage;
    Image4: TImage;
    Label3: TLabel;
    Label5: TLabel;
    Label2: TLabel;
    Label7: TLabel;
    Bevel2: TBevel;
    Panel9: TPanel;
    btnAddNote: TBitBtn;
    bbPrint: TBitBtn;
    pnlTIUToolbar: TPanel;
    pnlCanvas: TPanel;
    Label1: TLabel;
    pnlStatusToolbar: TPanel;
    Label4: TLabel;
    Panel14: TPanel;
    pnlDocumentDetails: TPanel;
    pnlDocumentsToolbar: TPanel;
    Label8: TLabel;
    Panel7: TPanel;
    acRefresh: TAction;
    SubmitMainMenu: TMainMenu;
    Documents1: TMenuItem;
    Refresh1: TMenuItem;
    AddTIU1: TMenuItem;
    Print1: TMenuItem;
    SignSubmit1: TMenuItem;
    pnlButtons2D: TPanel;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    pnlButtons3D: TPanel;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    BitBtn5: TBitBtn;
    procedure dbgStatusDrawColumnCell(Sender: TObject; const Rect: TRect; DataCol: Integer; Column: TColumn;
      State: TGridDrawState);
    procedure acSubmitExecute(Sender: TObject);
    procedure edSignatureChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure acAddNoteExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure tvNotesChange(Sender: TObject; Node: TTreeNode);
    procedure acNoteListExecute(Sender: TObject);
    procedure acGoToExecute(Sender: TObject);
    procedure dbgStatusKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure edSignatureKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure acPrintExecute(Sender: TObject);
    procedure tvNotesEnter(Sender: TObject);
    procedure acSignAndSubmitExecute(Sender: TObject);
    procedure acRefreshExecute(Sender: TObject);
    procedure tvNotesMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    fShowSignature: Boolean;
    procedure setupSignature(aVisible: Boolean);
    procedure DoSubmit(aSig: string);
    function getOldCommentsSummary: string;
    function isReadyToSubmit: Boolean;
  public
    procedure CleanTV;
    procedure SetUp;
    procedure SetUpAdditionalReports;
    procedure setSubmitStatus(aStatus: string);
    procedure ReportStatus;
    procedure ActivateTabForm;
    function getMainMenu: TMainMenu;
  end;

var
  frmSubmit: TfrmSubmit;

implementation

uses
  uMDK_Engine, uMDK_Common, dMDK_Data, DB, fMDK_Main, System.UITypes, fMDK_Flowsheet, uMDK_Options, uROR_RPCBroker,
  fMDK_TIUNote, uMDK_TIUNote, uMDK_Utils, uMDK_Patients, Printers, uMDK_ReportEngine, fMDK_Comments, Math, uMDK_Summary;
{$R *.dfm}

function newSubmitForm(aStudyID: string): TfrmSubmit;
begin
  Application.CreateForm(TfrmSubmit, Result);
  if Assigned(Result) then
    with Result do begin
      BorderStyle := bsSizeable;
      pnlSignature.Visible := False;
      pnlSubmitActions.Visible := False;
    end;
end;

procedure TfrmSubmit.SetUpAdditionalReports;
var
  i: Integer;
  sl: TStringList;
  tvn, tvnReport: TTreeNode;
const
  ADDITIONAL_REPORTS = 'Additional Reports';
begin
  if not Assigned(Engine) then Exit;
  if not Preferences.bValue(onShowReportTemplates) then Exit;
  tvnReport := nil;
  if Preferences.bValue(onShowReportTemplates) then begin
    for i := 0 to tvNotes.Items.Count - 1 do begin
      if tvNotes.Items[i].Text = ADDITIONAL_REPORTS then begin
        tvnReport := tvNotes.Items[i];
        break;
      end;
    end;
    if tvnReport = nil then begin
      tvnReport := tvNotes.Items.AddChild(nil, ADDITIONAL_REPORTS);
      tvnReport.ImageIndex := 8;
      tvnReport.SelectedIndex := 8;
    end else begin
      tvn := tvnReport.getFirstChild;
      while tvn <> nil do begin
        tvNotes.Items.Delete(tvn);
        tvn := tvnReport.getFirstChild;
      end;
    end;
  end;
  sl := TStringList.Create;
  sl.Text := Engine.getSystemOptionByName(onReportList);
  for i := 0 to sl.Count - 1 do
    if (trim(sl.Names[i]) <> '') then
      if (sl.Names[i] <> Preferences.sValue(onSummaryReportName)) then begin
        if Preferences.bValue(onShowReportTemplates) then begin
          tvn := tvNotes.Items.AddChild(tvnReport, sl.Names[i]);
          tvn.ImageIndex := 7;
          tvn.SelectedIndex := 7;
        end;
      end;
  sl.Free;
  tvNotes.FullExpand;
end;

procedure TfrmSubmit.SetUp;
begin
  pnlChecklist.Visible := Preferences.bValue(onShowStatusPanel);
  dbgStatus.Color := Preferences.iValue(onReadOnlyFieldColor);
  dbgStatus.FixedColor := Preferences.iValue(onReadOnlyFieldColor);
  tvNotes.Color := Preferences.iValue(onReadOnlyFieldColor);
  pnlLegend.Color := Preferences.iValue(onReadOnlyFieldColor);
  edSignature.Color := Preferences.iValue(onProblemFieldColor);
  pnlSignature.Color := Preferences.iValue(onReadOnlyFieldColor);
  mmReport.Color := Preferences.iValue(onReadOnlyFieldColor);
  bvlSignature.Visible := not devOptions.bValue(donShowHorizontalScrollBarsInReports);
  pnlSignature.Visible := Preferences.bValue(onShowReportSignature);
  SetUpAdditionalReports;
  pnlTIUToolbar.Color := Preferences.iValue(onColorToolbar);
  pnlDocumentsToolbar.Color := Preferences.iValue(onColorToolbar);
  pnlStatusToolbar.Color := Preferences.iValue(onColorToolbar);
  pnlButtons3D.Visible := devOptions.bValue(donWindowsXPTheme);
  pnlButtons2D.Visible := not devOptions.bValue(donWindowsXPTheme);
end;

procedure TfrmSubmit.FormCreate(Sender: TObject);
begin
  SetUp;
end;

procedure TfrmSubmit.FormDestroy(Sender: TObject);
begin
  CleanTV;
end;

procedure TfrmSubmit.dbgStatusDrawColumnCell(Sender: TObject; const Rect: TRect; DataCol: Integer; Column: TColumn;
  State: TGridDrawState);
var
  field: TField;
  dataset: TDataSet;
  s, risk: string;
  i: Integer;
  iROColor: Integer;
const
  _Status = 'StatusID';
  _Comments = 'Comments';
  RiskString = '0 1 2';
begin
  iROColor := Preferences.iValue(onReadOnlyFieldColor);
  dataset := dbgStatus.DataSource.dataset;
  field := dataset.FieldByName(_Status);
  risk := field.asString;
  case pos(risk, RiskString) of
    1, 3: dbgStatus.canvas.font.Color := clBlack;
    5: dbgStatus.canvas.font.Color := clRed
  else dbgStatus.canvas.font.Color := clRed
  end;
  case pos(risk, RiskString) of
    1, 3: dbgStatus.canvas.brush.Color := iROColor; // clBtnFace;//clInfoBk;
    5: dbgStatus.canvas.brush.Color := gErrorBkg; // clSilver;
  else dbgStatus.canvas.brush.Color := gErrorBkg; // clSilver;
  end;
  if DataCol = 1 then dbgStatus.canvas.brush.Color := iROColor; // clSilver;
  with Sender as TDBGrid do begin
    if Columns[DataCol].FieldName = _Status then begin
      canvas.FillRect(Rect);
      if risk = '' then DefaultDrawColumnCell(Rect, DataCol, Column, State)
      else begin
        i := Columns[DataCol].field.Value + 1;
        if i > 2 then i := 2;
        case Columns[DataCol].field.Value of
          0 .. 3: ilStatus.Draw(canvas, Rect.Left + 14, Rect.Top, i, True)
        else DefaultDrawColumnCell(Rect, DataCol, Column, State);
        end;
      end;
    end else if Columns[DataCol].FieldName = _Comments then begin
      s := dataset.FieldByName(_Comments).asString;
      if canvas.TextWidth(dataset.FieldByName(_Comments).asString) + 20 > Columns[DataCol].Width then begin
        while (canvas.TextWidth(s) + 20 + canvas.TextWidth('...') > Columns[DataCol].Width) and (Length(s) > 0) do
            s := copy(s, 1, Length(s) - 1);
        s := s + '...';
      end;
      canvas.TextRect(Rect, Rect.Left + 20, Rect.Top + 2, s);
    end
    else DefaultDrawColumnCell(Rect, DataCol, Column, State);
  end;
end;

procedure TfrmSubmit.edSignatureChange(Sender: TObject);
begin
  acSubmit.Enabled := Length(edSignature.Text) >= devOptions.iValue(donMinESigLength);
end;

procedure TfrmSubmit.acAddNoteExecute(Sender: TObject);
begin
  if AddTIUNote(Engine.CurrentPatient.SessionID, Engine.CurrentOperatorID, Engine.UserDUZ) <> mrCancel then
      acNoteListExecute(nil);
end;

procedure TfrmSubmit.CleanTV;
var
  aNote: TMDK_TIUNote;
  i: Integer;
begin
  try
    for i := 1 to tvNotes.Items.Count - 1 do
      if (tvNotes.Items[i].Parent = tvNotes.Items[1]) and (tvNotes.Items[i].Data <> nil) then
        try
          aNote := TMDK_TIUNote(tvNotes.Items[i].Data);
          aNote.Free;
        except
        end;
    while tvNotes.Items.Count > 1 do tvNotes.Items.Delete(tvNotes.Items[1]);
  except
  end;
end;

procedure TfrmSubmit.tvNotesChange(Sender: TObject; Node: TTreeNode);
var
  aNote: TMDK_TIUNote;
  sTemplate: string;
const
  sTemplates = 'Additional Reports';
  sStatus = 'Status Report (Panel)';
  procedure setupLayout;
  begin
    if (Node.Parent = nil) and (Node.Text = sStatus) then begin
      pnlChecklist.Visible := True;
      pnlDocumentDetails.Visible := False;
    end else begin
      pnlChecklist.Visible := False;
      pnlDocumentDetails.Visible := True;
    end;
  end;

begin
  // setupLayout;
  Screen.Cursor := crHourGlass;
  if Node.Text = 'Result Report' then begin
    if Engine.CurrentPatient.Name <> '' then mmReport.Lines.Text := Engine.getReportString
    else mmReport.Lines.Text := '';
    fShowSignature := True;
  end else if Node.Text = sTemplates then begin
    fShowSignature := False;
    mmReport.Lines.Text := #13 + '  Select a template to review the report text';
  end else if (Node.Parent <> nil) and (Node.Parent.Text = sTemplates) then begin
    fShowSignature := False;
    sTemplate := Engine.getSystemOptionByName(Node.Text);
    mmReport.Lines.Text := Engine.getReportValue(sTemplate);
  end else if (Node.Parent = nil) and (Node.Text = sStatus) then begin
    Exit;
  end else begin
    fShowSignature := False;
    aNote := Node.Data;
    mmReport.Lines.Text := aNote.Text;
  end;
  setupSignature(fShowSignature);
  Screen.Cursor := crDefault;
end;

procedure TfrmSubmit.acNoteListExecute(Sender: TObject);
var
  SS, s: string;
  sl: TStringList;
  Note: TMDK_TIUNote;
  tvNode: TTreeNode;
  i: Integer;
  iImage: Integer;
  sKeyword: string;
const
  iiTIU = 6;
  iiFalls = 7;
begin
  CleanTV;
  s := Engine.getStudyNotes;
  if s <> '' then begin
    if pos('-', s) <> 1 then begin
      sl := TStringList.Create;
      sl.Text := s;
      for i := 0 to sl.Count - 1 do begin
        SS := sl[i];
        Note := TMDK_TIUNote.Create;
        Note.StudyID := '';
        Note.TIUID := Piece(SS, '^', 1);
        Note.Title := Piece(SS, '^', 3) + ' (TIU ' + Note.TIUID + ')'; // zzzzzzandria 2008-04-18
        Note.Text := Engine.getNoteByNum(Note.TIUID);
        if Note.Title = '' then begin
          s := Note.Text;
          s := Piece(Piece(s, 'ENTRY DATE:', 1), 'DATE OF NOTE:', 2);
          if trim(s) = '' then Note.Title := Note.TIUID
          else Note.Title := s;
        end;
        Note.AuthorName := Piece(SS, '^', 4);
        tvNode := tvNotes.Items.AddObject(nil, Note.Title, Note);
        sKeyword := devOptions.sValue(donFallRiskKeyword);
        if pos(sKeyword, Note.Text) <> 0 then iImage := 7
        else iImage := 6;
        tvNode.ImageIndex := iImage;
        tvNode.SelectedIndex := iImage;
      end;
      sl.Free;
    end
{$IFDEF AANTEST}
    else if pos('No return', Piece(s, '^', 2)) = 0 then ShowMessage(Piece(s, '^', 2) + ' (from getStudyNotes)');
{$ELSE}
      ;
{$ENDIF}
  end;
  SetUpAdditionalReports;
  if tvNotes.Selected = nil then tvNotes.Selected := tvNotes.Items[0];
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmSubmit.acGoToExecute(Sender: TObject);
var
  s: string;
begin
  s := dmData.cdsStatusPage.asString;
  frmMain.GoToPage(s);
end;

procedure TfrmSubmit.dbgStatusKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_Return then acGoToExecute(nil);
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmSubmit.edSignatureKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_Return then acSubmit.Execute;
end;

procedure TfrmSubmit.setSubmitStatus(aStatus: string);
var
  i: Integer;
begin
  i := pos('Error', aStatus);
  if (i = 0) and not Engine.CurrentPatient.ReadOnly then begin
    edSignature.Enabled := True;
    edSignature.Color := Preferences.iValue(onProblemFieldColor);
    edSignatureChange(nil);
    acSubmit.Enabled := True;
  end else begin
    edSignature.Enabled := False;
    edSignature.Color := clBtnFace;
    acSubmit.Enabled := False;
  end;
end;

procedure TfrmSubmit.ReportStatus;
begin
  tvNotes.Color := Preferences.iValue(onReadOnlyFieldColor);
  pnlSignature.Color := Preferences.iValue(onReadOnlyFieldColor);
  mmReport.Color := Preferences.iValue(onReadOnlyFieldColor);
  if devOptions.bValue(donShowHorizontalScrollBarsInReports) then mmReport.ScrollBars := ssBoth
  else mmReport.ScrollBars := ssVertical;

  setupSignature(fShowSignature);
end;

procedure TfrmSubmit.acPrintExecute(Sender: TObject);
var
  imgBackground: TBitmap;
begin
  if PrintDialog1.Execute then
    with Printer do begin
      Printer.canvas.font.Assign(mmReport.font);
{$IFDEF DEMO}
      imgBackground := imgDemoBackground.Picture.Bitmap;
{$ELSE}
      imgBackground := nil;
{$ENDIF}
      PrintReportString(Printer, mmReport.Lines.Text, imgBackground);
    end;
end;

procedure TfrmSubmit.tvNotesEnter(Sender: TObject);
begin
  if Assigned(tvNotes.Selected) then tvNotesChange(tvNotes, tvNotes.Selected)
  else begin
    mmReport.Lines.Text := '';
    setupSignature(False);
  end;
end;

procedure TfrmSubmit.setupSignature(aVisible: Boolean);
begin
  if Preferences.bValue(onShowReportSignature) then begin
    pnlSignature.Visible := aVisible;
    acSignAndSubmit.Enabled := False;
  end else begin
    acSignAndSubmit.Enabled := aVisible and not Engine.CurrentPatient.ReadOnly;
    pnlSignature.Visible := False;
  end;
end;

procedure TfrmSubmit.ActivateTabForm;
begin
  edSignature.Text := '';
  acNoteListExecute(nil);
  if tvNotes.Selected = nil then begin
    tvNotes.Selected := tvNotes.Items[0];
  end else if tvNotes.Selected = tvNotes.Items[0] then
    try
      Screen.Cursor := crHourGlass;
      mmReport.Lines.Text := Engine.getReportString;
      Screen.Cursor := crDefault;
    except
    end;
  pnlSubmit.Align := alClient;
  try
    setSubmitStatus(frmMain.lblStudyStatus.Caption);
  except
  end;
  pnlTIUToolbar.Color := Preferences.iValue(onColorToolbar);
  pnlDocumentsToolbar.Color := Preferences.iValue(onColorToolbar);
  pnlStatusToolbar.Color := Preferences.iValue(onColorToolbar);
  acSignAndSubmit.Enabled := not Engine.CurrentPatient.ReadOnly and not Preferences.bValue(onShowReportSignature);
  acSubmit.Enabled := not Engine.CurrentPatient.ReadOnly;
  invalidate;
end;

function TfrmSubmit.getOldCommentsSummary: string;
  function getFieldString(aName, aTitle: string): string;
  var
    s: string;
  begin
    s := '';
    try
      s := dmData.cdsSummary.FieldByName(aName).asString;
      if s <> '' then s := aTitle + '"' + s + '"' + #13#10;
    except
      on E: Exception do s := '';
    end;
    Result := s;
  end;

begin
  Result := '';
  Result := Result + getFieldString('Pre-Treatment note: ', fnS_PreNotes);
  Result := Result + getFieldString('Post-Treatment note: ', fnS_PostNotes);
  Result := Result + getFieldString('RxLab note: ', fnS_LabNotes);
  Result := Result + getFieldString('Summary note: ', fnS_SummaryNotes);
end;

function TfrmSubmit.isReadyToSubmit: Boolean;
var
  s: string;
begin
  Result := False;
  s := getOldCommentsSummary;
  if (s <> '') and (MessageDLG('Study contains old style comments' + #13#13 + s + #13#13 +
    'Do you want to submit the study?', mtWarning, [mbNo, mbYes], 0) <> mrYes) then Exit;
  if Preferences.bValue(onSaveVitals) and (Engine.LocationID = '') then
      ShowMessage('Sorry, check the location value...')
  else if not Engine.PCEReadyToSubmit or not Engine.ValidBillingInfo(s, False) then
      ShowMessage('The PCE data should be reviewed/saved prior to Submit')
  else if dmData.cdsSummary.FieldByName(fnS_LastRecord).AsBoolean or
    (Preferences.bValue(onIgnoreUnfinishedStatus)) or
    (MessageDLG(msgIncompletStudy, mtConfirmation, [mbYes, mbNo], 0) = mrYes) then begin
    if not dmData.cdsSummary.FieldByName(fnS_LastRecord).AsBoolean then
        dmData.RegisterEvent(etWarning, 'Not all data records have been received from the instrument.',
        Engine.CurrentPatient.SessionID, 0, True);
    Result := True;
  end;
end;

procedure TfrmSubmit.acSignAndSubmitExecute(Sender: TObject);
var
  sSig: string;
begin
  if isReadyToSubmit and (Submit(mmReport.Text, sSig) = mrOK) then DoSubmit(sSig);
end;

procedure TfrmSubmit.acSubmitExecute(Sender: TObject);
  function ValidESig: Boolean;
  begin
    Result := Engine.isESigValid(edSignature.Text);
    if not Result then
      if (Engine.InvalidESigCount <= 2) then
          MessageDLG('Signature is not valid.' + #13#10 + 'Try again.', mtWarning, [mbOK], 0)
      else begin
        MessageDLG('e-Signature verification failed 3 times' + #13#10 + 'The application will be closed', mtError,
          [mbOK], 0);
        frmMain.ShutDown;
      end;
  end;

begin
  if isReadyToSubmit and ValidESig then DoSubmit(edSignature.Text);
end;

procedure TfrmSubmit.DoSubmit(aSig: string);
var
  sErrorLog, sResult, sReport, sSummary: string;
begin
  sErrorLog := '';
  if Preferences.bValue(onSaveVitals) then begin
    sResult := Engine.SubmitVitalsData;
    if sResult <> '' then sErrorLog := sErrorLog + sResult;
    if uppercase(Preferences.sValue(onSaveFlowsheetVitals)) = 'TRUE' then Engine.SubmitFlowsheet;
  end;
  sSummary := Engine.Submit(Engine.CurrentPatient.iStudy, aSig, mmReport.Text);
  if pos('-1', sSummary) <> 1 then begin
    sReport := 'Study ' + Engine.CurrentPatient.SessionID + ' submitted.';
    if sErrorLog <> '' then sReport := sReport + #13#10 + sErrorLog;
    ShowMessage(sReport);
    frmMain.acPatientSelectExecute(nil);
  end else begin
    frmMain.RefreshStudyStatus;
    ShowMessage(Piece(sSummary, '^', 2));
  end;
end;

procedure TfrmSubmit.acRefreshExecute(Sender: TObject);
begin
  if Assigned(tvNotes.Selected) then tvNotesChange(nil, tvNotes.Selected);
end;

function TfrmSubmit.getMainMenu: TMainMenu;
begin
  Result := SubmitMainMenu;
end;

procedure TfrmSubmit.tvNotesMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Note: TMDK_TIUNote;
begin
  if (Button = mbLeft) and (ssCtrl in Shift) and Assigned(tvNotes.Selected) and Assigned(tvNotes.Selected.Data) and
    (TObject(tvNotes.Selected.Data) is TMDK_TIUNote) then begin
    Note := TMDK_TIUNote(tvNotes.Selected.Data);
    MessageDLG('Date Time: ' + FormatDateTime('mm/dd/yyyy hh:nn:ss', Note.DateTime) + #13#10 + 'Study ID:  ' +
      Note.StudyID + #13#10 + 'TIU ID:    ' + Note.TIUID + #13#10 + 'Author:    ' + Note.AuthorName + #13#10 +
      'Location:  ' + Note.HospitalLocation + #13#10 + 'Title:     ' + Note.Title + #13#10,
      mtInformation, [mbOK], 0);
  end;
end;

end.
