unit fROR_PCall;

{
  ================================================================================
  *
  *       Package:        ROR - Clinical Case Registries
  *       Date Created:   $Revision: 1 $  $Modtime: 9/28/07 10:43a $
  *       Site:           Hines OIFO
  *       Developers:     dddddomain.user@domain.ext
  *
  *       Description:    RPC call and RPC Error Window
  *
  *       Notes:
  *
  ================================================================================
  *       $Archive: /Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/ROR/fROR_PCall.pas $
  *
  * $History: fROR_PCall.pas $
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 1/13/09    Time: 1:16p
  * Created in $/Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/ROR
  * Patch 19 source code
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/24/08    Time: 4:24p
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/HEMODIALYSIS_1_0_6_319_#18/ROR
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/02/08    Time: 9:25a
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI Current/HEMODIALYSIS_1_0_6_319/ROR
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 5/22/07    Time: 9:13a
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/ROR
  *
  * *****************  Version 3  *****************
  * User: Zzzzzzandria Date: 3/29/07    Time: 4:20p
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR
  *
  * *****************  Version 2  *****************
  * User: Zzzzzzandria Date: 3/26/07    Time: 3:32p
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 3/26/07    Time: 1:05p
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 3/26/07    Time: 12:30p
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP_HEMODIALYSIS/CP MODERNIZATION/ROR
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 1/25/07    Time: 9:41a
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 1/22/07    Time: 1:29p
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0.6.308/CP MODERNIZATION/ROR
  * GUI 1.0.6.308 RC January 2007
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/28/06    Time: 5:12p
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR
  *
  * *****************  Version 16  *****************
  * User: Zzzzzzgavris Date: 1/18/05    Time: 3:40p
  * Updated in $/CCR v1.0/Current/Components
  *
  * *****************  Version 15  *****************
  * User: Zzzzzzgavris Date: 1/10/05    Time: 3:49p
  * Updated in $/CCR v1.0/Current/Components
  *
  * *****************  Version 13  *****************
  * User: Zzzzzzgavris Date: 10/15/04   Time: 10:15a
  * Updated in $/CCR v1.0/Current
  *
  * *****************  Version 12  *****************
  * User: Zzzzzzgavris Date: 10/15/04   Time: 10:11a
  * Updated in $/CCR v1.0/Current/Components
  *
  *
  ================================================================================
}

interface

uses
  SysUtils, Classes, Forms, Dialogs, TRPCB, StdCtrls, Controls, CCOWRPCBroker;

type
  TRPCMode = set of (
    RpcSilent, // Do not show any error messages
               // (only return the error codes)
    RpcNoResChk // Do not check the Results array for the errors
                // returned by the remote procedure. This flag must be
                // used for the remote procedures that do not conform
                // to the error reporting format supported by the
                // CheckRPCError function.
    );

  // ------------------------------- TRPCErrorForm ------------------------------

  TRPCErrorForm = class(TForm)
    Msg: TMemo;
    BtnOk: TButton;
    procedure BtnOkClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  // ------------------------------- IRPCInterface ------------------------------

  IRPCInterface = interface(IInterface)
    ['{FAD698BB-63E9-408C-BFF7-8E4A8451B63B}']
    function CallProc(const RemoteProcedure: string; const Parameters: array of string; MultList: TStringList = nil;
      const RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean;
    function CheckError(const RPCName: string; RetList: TStrings = nil; const RPCMode: TRPCMode = []): Integer;
  end;

  // -------------------------------- TCCRLogMode -------------------------------

  TCCRLogMode = class(TPersistent)
  private
    FEnabled: Boolean;
    FLimitParams: Word;
    FLimitResults: Word;
    FParameters: Boolean;
    FResults: Boolean;
  public
    constructor Create;
    procedure Assign(ASource: TPersistent); override;
  published
    property Enabled: Boolean read FEnabled write FEnabled;
    property LimitParams: Word read FLimitParams write FLimitParams default 0;
    property LimitResults: Word read FLimitResults write FLimitResults default 0;
    property Parameters: Boolean read FParameters write FParameters default True;
    property Results: Boolean read FResults write FResults default True;
  end;

  // ------------------------------- TCCRRPCBroker ------------------------------

  TCCRRPCBroker = class(TCCOWRPCBroker, IRPCInterface)
  private
    FLog: TCCRLogMode;
    FLogData: TStringList;
    procedure SetLog(const AValue: TCCRLogMode);
  public
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddLogString(const AValue: string); overload;
    procedure AddLogString(const AFormat: string; const ArgList: array of const); overload;
    function CallProc(const ARemoteProcedure: string; const Parameters: array of string; MultList: TStringList = nil;
      const RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean; virtual;
    function CheckError(const RPCName: string; RetList: TStrings = nil; const RPCMode: TRPCMode = []): Integer;
    property LogData: TStringList read FLogData;
  published
    property Log: TCCRLogMode read FLog write SetLog;
  end;

function CallRemoteProc(Broker: TRPCBroker; RemoteProcedure: string; Parameters: array of string;
  MultList: TStringList = nil; RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean;
function CheckRPCError(RPCName: string; Results: TStrings; RPCMode: TRPCMode = []): Integer;

implementation

{$R *.DFM}

uses
  Math, UROR_CommonUtils; // replaces uROR_Utilities

function CallRemoteProc(Broker: TRPCBroker; RemoteProcedure: string; Parameters: array of string;
  MultList: TStringList = nil; RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean;
var
  I, J: Integer;
begin
  Broker.RemoteProcedure := RemoteProcedure;
  I := 0;
  while I <= high(Parameters) do begin
    if (Copy(Parameters[I], 1, 1) = '@') and (Parameters[I] <> '@') then begin
      Broker.Param[I].Value := Copy(Parameters[I], 2, Length(Parameters[I]));
      Broker.Param[I].PType := Reference;
    end else begin
      Broker.Param[I].Value := Parameters[I];
      Broker.Param[I].PType := Literal;
    end;
    Inc(I);
  end;
  if MultList <> nil then
    if MultList.Count > 0 then begin
      for J := 1 to MultList.Count do Broker.Param[I].Mult[IntToStr(J)] := MultList[J - 1];
      Broker.Param[I].PType := List;
    end;
  try
    Result := True;
    if RetList <> nil then begin
      RetList.Clear;
      Broker.LstCall(RetList);
    end else begin
      Broker.Call;
      RetList := Broker.Results;
    end;
    if not(RpcNoResChk in RPCMode) then
      if CheckRPCError(RemoteProcedure, RetList, RPCMode) <> 0 then Result := False;
  except
    on E: EBrokerError do begin
      if not(RpcSilent in RPCMode) then
          MessageDialog('RPC Error', 'Error encountered retrieving VistA data.' + #13 + 'Server: ' + Broker.Server + #13
          + 'Listener port: ' + IntToStr(Broker.ListenerPort) + #13 + 'Remote procedure: ' + Broker.RemoteProcedure +
          #13 + 'Error is: ' + #13 + E.Message, MtError, [MbOK], MrOK, E.HelpContext);
      with Broker do begin
        Results.Clear;
        Results.Add('-1000^1');
        Results.Add('-1000^' + E.Message);
      end;
      Result := False;
    end;
    else
      raise;
  end;
end;

function CheckRPCError(RPCName: string; Results: TStrings; RPCMode: TRPCMode = []): Integer;
var
  I, N: Integer;
  Buf, Rc: string;
  Form: TRPCErrorForm;
begin
  if Results.Count = 0 then begin
    Result := MrOK;
    Buf := 'The ''' + RPCName + ''' remote procedure returned nothing!';
    if not(RpcSilent in RPCMode) then MessageDialog('RPC Error', Buf, MtError, [MbOK], MrOK, 0);
    Results.Add('-1001^1');
    Results.Add('-1001^' + Buf);
    Exit;
  end;
  Result := 0;
  Rc := Piece(Results[0], '^');
  if StrToIntDef(Rc, 0) < 0 then begin
    Result := MrOK;
    if not(RpcSilent in RPCMode) then begin
      Form := TRPCErrorForm.Create(Application);
      N := StrToIntDef(Piece(Results[0], '^', 2), 0);
      with Form.Msg.Lines do begin
        Buf := 'The error code ''' + Rc + ''' was returned by the ''' + RPCName + ''' remote procedure!';
        if N > 0 then begin
          Buf := Buf + ' The problem had been caused by the following ';
          if N > 1 then Buf := Buf + 'errors (in reverse chronological order):'
          else Buf := Buf + 'error: ';
          Add(Buf);
          for I := 1 to N do begin
            Buf := Results[I];
            Add('');
            Add(' ' + Piece(Buf, '^', 2));
            Add(' Error Code: ' + Piece(Buf, '^', 1) + ';' + #9 + 'Place: ' + StringReplace(Piece(Buf, '^', 3), '~',
              '^', []));
          end;
        end
        else Add(Buf);
      end;
      Result := Form.ShowModal;
      Form.Free;
    end;
  end;
end;
/// ////////////////////////////// TCCRLogMode \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TCCRLogMode.Create;
begin
  inherited;
  FEnabled := False;
  FLimitParams := 0;
  FLimitResults := 0;
  FParameters := True;
  FResults := True;
end;

procedure TCCRLogMode.Assign(ASource: TPersistent);
begin
  if ASource is TCCRLogMode then
    with TCCRLogMode(ASource) do begin
      Self.FEnabled := FEnabled;
      Self.FLimitParams := FLimitParams;
      Self.FLimitResults := FLimitResults;
      Self.FParameters := FParameters;
      Self.FResults := FResults;
    end
  else inherited;
end;
/// ///////////////////////////// TCCRRPCBroker \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TCCRRPCBroker.Create(AnOwner: TComponent);
begin
  inherited;
  FLog := TCCRLogMode.Create;
  FLogData := TStringList.Create;
end;

destructor TCCRRPCBroker.Destroy;
begin
  FreeAndNil(FLogData);
  FreeAndNil(FLog);
  inherited;
end;

procedure TCCRRPCBroker.AddLogString(const AValue: string);
begin
  LogData.Add(AValue);
end;

procedure TCCRRPCBroker.AddLogString(const AFormat: string; const ArgList: array of const);
begin
  AddLogString(Format(AFormat, ArgList));
end;

function TCCRRPCBroker.CallProc(const ARemoteProcedure: string; const Parameters: array of string;
  MultList: TStringList = nil; const RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean;
var
  I, J, N: Integer;
begin
  RemoteProcedure := ARemoteProcedure;
  N := high(Parameters);
  if Log.Enabled then begin
    AddLogString('');
    AddLogString('RPC: %s', [ARemoteProcedure]);
    if Log.Parameters then
      for J := 0 to N do AddLogString('     P[%02d]: ''%s''', [J, Parameters[J]]);
  end;
  I := 0;
  while I <= N do begin
    if (Copy(Parameters[I], 1, 1) = '@') and (Parameters[I] <> '@') then begin
      Param[I].Value := Copy(Parameters[I], 2, Length(Parameters[I]));
      Param[I].PType := Reference;
    end else begin
      Param[I].Value := Parameters[I];
      Param[I].PType := Literal;
    end;
    Inc(I);
  end;
  if MultList <> nil then
    if MultList.Count > 0 then begin
      if Log.Enabled and Log.Parameters then begin
        if Log.LimitParams > 0 then N := Min(MultList.Count, Log.LimitParams) - 1
        else N := MultList.Count - 1;
        for J := 0 to N do AddLogString('     M[%02d]: ''%s''', [J, MultList[J]]);
        if N < (MultList.Count - 1) then AddLogString('     ...');
      end;
      for J := 1 to MultList.Count do Param[I].Mult[IntToStr(J)] := MultList[J - 1];
      Param[I].PType := List;
    end;
  try
    Result := True;
    if RetList <> nil then begin
      RetList.Clear;
      LstCall(RetList);
    end else begin
      Call;
      RetList := Results;
    end;
    if not(RpcNoResChk in RPCMode) then
      if CheckRPCError(ARemoteProcedure, RetList, RPCMode) <> 0 then Result := False;
  except
    on E: EBrokerError do begin
      if not(RpcSilent in RPCMode) then
          MessageDialog('RPC Error', 'Error encountered retrieving VistA data.' + #13 + 'Server: ' + Server + #13 +
          'Listener port: ' + IntToStr(ListenerPort) + #13 + 'Remote procedure: ' + ARemoteProcedure + #13 +
          'Error is: ' + #13 + E.Message, MtError, [MbOK], MrOK, E.HelpContext);
      Results.Clear;
      Results.Add('-1000^1');
      Results.Add('-1000^' + E.Message);
      Result := False;
    end;
    else
      raise;
  end;
  if Log.Enabled and Log.Results then begin
    if Log.LimitResults > 0 then N := Min(RetList.Count, Log.LimitResults) - 1
    else N := RetList.Count - 1;
    for J := 0 to N do AddLogString('     R[%02d]: ''%s''', [J, RetList[J]]);
    if N < (RetList.Count - 1) then AddLogString('     ...');
  end;
end;

function TCCRRPCBroker.CheckError(const RPCName: string; RetList: TStrings; const RPCMode: TRPCMode): Integer;
var
  I, N: Integer;
  Buf, Rc: string;
  Form: TRPCErrorForm;
begin
  if not Assigned(RetList) then RetList := Results;
  if RetList.Count = 0 then begin
    Result := MrOK;
    Buf := 'The ''' + RPCName + ''' remote procedure returned nothing!';
    if not(RpcSilent in RPCMode) then MessageDialog('RPC Error', Buf, MtError, [MbOK], MrOK, 0);
    RetList.Add('-1001^1');
    RetList.Add('-1001^' + Buf);
    Exit;
  end;
  Result := 0;
  Rc := Piece(RetList[0], '^');
  if StrToIntDef(Rc, 0) < 0 then begin
    Result := MrOK;
    if not(RpcSilent in RPCMode) then begin
      Form := TRPCErrorForm.Create(Application);
      try
        N := StrToIntDef(Piece(RetList[0], '^', 2), 0);
        with Form.Msg.Lines do begin
          Buf := 'The error code ''' + Rc + ''' was returned by the ''' + RPCName + ''' remote procedure!';
          if N > 0 then begin
            Buf := Buf + ' The problem had been caused by the following ';
            if N > 1 then Buf := Buf + 'errors (in reverse chronological order):'
            else Buf := Buf + 'error: ';
            Add(Buf);
            for I := 1 to N do begin
              Buf := RetList[I];
              Add('');
              Add(' ' + Piece(Buf, '^', 2));
              Add(' Error Code: ' + Piece(Buf, '^', 1) + ';' + #9 + 'Place: ' + StringReplace(Piece(Buf, '^', 3), '~',
                '^', []));
            end;
          end
          else Add(Buf);
        end;
        Result := Form.ShowModal;
      finally
        Form.Free;
      end;
    end;
  end;
end;

procedure TCCRRPCBroker.SetLog(const AValue: TCCRLogMode);
begin
  FLog.Assign(AValue);
end;
/// ///////////////////////////// TRPCErrorForm \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

procedure TRPCErrorForm.BtnOkClick(Sender: TObject);
begin
  ModalResult := MrOk;
end;

end.
