unit uROR_RPCBroker;

{
  ================================================================================
  *
  *       Package:        ROR - Clinical Case Registries
  *       Date Created:   $Revision: 1 $  $Modtime: 7/29/08 9:37a $
  *       Site:           Hines OIFO
  *       Developers:     dddddomain.user@domain.ext
  *
  *       Description:    RPC call and RPC Error Window
  *
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR/uROR_RPCBroker.pas $
  *
  *       $ History: uROR_RPCBroker.pas $
  *
  ================================================================================
}
interface

uses
  SysUtils, Classes, Controls, UMDK_Utils, // replaces the uROR_Utilities
  TRPCB, CCOWRPCBroker, RpcConf1,
{$IFDEF RORCCOW}
  VERGENCECONTEXTORLib_TLB,
{$ENDIF}
  Dialogs, Forms;

type
  TRPCBrokerParams = record
    Server: string;            // BROKERSERVER;
    ListenerPort: Integer;     // 9200;
    ClearResults: Boolean;     // True;
    ClearParameters: Boolean;  // True;
    AccessVerifyCodes: string; //
    DebugMode: Boolean;
  end;

var
  CheckStartupParams: Boolean;
  RPCBroker: TCCOWRPCBroker;
  ContextName: string = 'MD HEMODIALYSIS USER';

function SelectBroker(Context: string{$IFDEF RORCCOW}; AContextor: TContextorControl{$ENDIF}): TCCOWRPCBroker;
function GMVConnect: Boolean;
procedure GMVDisconnect;

implementation

uses
{$IFDEF DEMO}
  FMDK_Wait,
  UMDK_LocalDB,
  FMDK_LocalLogin,
{$ENDIF}
  System.UITypes, FGMV_AboutDlg;

var
  BrokerParams: TRPCBrokerParams;

function GMVConnect: Boolean;
{$IFDEF DEMO}
var
  SLocal: string;
{$ENDIF}
begin
  Result := False;
  if Assigned(RPCBroker) then GMVDisconnect;
  CmdLineSwitch(['/CONTEXTNAME'], ContextName); // zzzzzzandria 050908
  if ContextName = '' then ContextName := 'MD HEMODIALYSIS USER';
{$IFDEF DEMO}
  SLocal := '';
  if CmdLineSwitch(['/DEMO'], SLocal) then begin
    RPCBroker := nil;
    if not Assigned(DlgLocalLogin) then Application.CreateForm(TdlgLocalLogin, DlgLocalLogin);
    if not Assigned(DlgLocalLogin) or (DlgLocalLogin.ShowModal <> MrOK) then Exit;
  end
  else
{$ENDIF}
  begin
{$IFDEF RORCCOW}
    CCRContextor.Enabled := True;
    RPCBroker := SelectBroker(ContextName, CCRContextor.Contextor);
{$ELSE}
    RPCBroker := SelectBroker(ContextName);
{$ENDIF}
    if not Assigned(RPCBroker) then Exit;
  end;
{$IFDEF DEMO}
  if SLocal <> '' then LOCAL_DIR := SLocal;
  SetLocalMap;
{$ENDIF}
  Result := True;
end;

procedure GMVDisconnect;
begin
  if Assigned(RPCBroker) then begin
    { Disconnect and destroy the RPC Broker }
    RPCBroker.Connected := False;
    FreeAndNil(RPCBroker);
  end;
end;

function GetBrokerParameters(var ParamRecord: TRPCBrokerParams): Boolean;
var
  UseServerList: Boolean;
  SPass, SLServer, SLPort: string;
  procedure SetByParams;
  var
    I: Integer;
  begin
    for I := 1 to ParamCount do begin
{$IFDEF AVTEST}
      if InString(ParamStr(I), ['/pass', '-pass'], False) then SPass := Piece(ParamStr(I), '=', 2);
      if InString(ParamStr(I), ['/av', '-av'], False) then SPass := Piece(ParamStr(I), '=', 2);
{$ENDIF}
      if InString(ParamStr(I), ['/s=', '-s=', 's='], False) or InString(ParamStr(I), ['/server=', '-server='], False)
      then begin
        ParamRecord.Server := Piece(ParamStr(I), '=', 2);
        UseServerList := False;
      end;
      if InString(ParamStr(I), ['/p=', '-p=', 'p='], False) or InString(ParamStr(I), ['/port=', '-port='], False) then
      begin
        ParamRecord.ListenerPort := StrToIntDef(Piece(ParamStr(I), '=', 2), 9200);
        UseServerList := False;
      end;
      if InString(ParamStr(I), ['/debug', '-debug'], False) then ParamRecord.DebugMode := True;
    end;
{$IFDEF AVTEST}
    if SPass <> '' then begin
      UseServerList := False;
      ParamRecord.AccessVerifyCodes := SPass
    end;
{$ENDIF}
  end;

begin
  ParamRecord.Server := 'BROKERSERVER';
  ParamRecord.ListenerPort := 9200;
  ParamRecord.ClearParameters := True;
  ParamRecord.ClearResults := True;
  SPass := '';
  UseServerList := True;
  if CheckStartupParams then SetByParams;
  if UseServerList then begin
    if GetServerInfo(SLServer, SLPort) <> 1 then begin
      MessageDlg('Sign-On Cancelled while setting connection parameters', MtInformation, [Mbok], 0);
      Result := False;
      Exit;
    end else begin
      ParamRecord.Server := SLServer;
      ParamRecord.ListenerPort := StrToIntDef(SLPort, 9200);
      Application.ProcessMessages; { Refresh screen prior to connecting }
    end;
  end;
  Result := True;
end;

function SetBrokerParameters(ParamRecord: TRPCBrokerParams; var RPCB: TCCOWRPCBroker; var ErrorString: string): Boolean;
begin
  RPCB.Server := ParamRecord.Server;
  RPCB.ListenerPort := ParamRecord.ListenerPort;
  RPCB.ClearParameters := ParamRecord.ClearParameters;
  RPCB.ClearResults := ParamRecord.ClearResults;
  RPCB.AccessVerifyCodes := ParamRecord.AccessVerifyCodes;
  RPCB.DebugMode := ParamRecord.DebugMode;
  ErrorString := '';
  if RPCB.Socket > 0 then // ???
  begin
    Result := True;
    Exit;
  end;
  try
    RPCB.Connected := True;
    Application.ProcessMessages;
    Result := True;
  except
    on E: EBrokerError do begin
      ErrorString := E.Message;
      Result := False;
      Exit;
    end;
    else
      raise;
  end;
end;

function SelectBroker(Context: string{$IFDEF RORCCOW}; AContextor: TContextorControl{$ENDIF}): TCCOWRPCBroker;
var
  S: string;
  RPCB: TCCOWRPCBroker;
  NewAttempt: Boolean;
  AttemptCount: Integer;
const
  AttemptLimit = 3;
  procedure ErrorReport;
  begin
    MessageDlg('Error Encountered' + #13 + #13 + 'User Sign-on is not complete.' + #13 +
      'Attempted connection using the following:' + #13 + 'VistA Server: ' + RPCB.Server + #13 + 'Listener Port: ' +
      IntToStr(RPCB.ListenerPort) + #13 + 'Error Message: ' + #13 + S, MtError, [Mbok], 0);
  end;

begin
  if not GetBrokerParameters(BrokerParams) then // process parameter string
  begin
    Result := nil;
    Exit;
  end;
  AttemptCount := 0;
  repeat
    NewAttempt := False;
    try
      RPCB := TCCOWRPCBroker.Create(Application);
    except
      on E: Exception do ShowMessage('TCCOWRPCBroker creation error' + #13 + E.Message);
    end;
    try
{$IFDEF RORCCOW}
      RPCB.Contextor := AContextor;
{$ELSE}
      RPCB.Contextor := nil;
{$ENDIF}
    except
      on E: Exception do ShowMessage('Error assigning Contextro to TCCOWRPCBroker' + #13 + E.Message);
    end;
    if not SetBrokerParameters(BrokerParams, RPCB, S) then begin
      ErrorReport;
      FreeAndNil(RPCB);
    end;
    SplashOff;
    if RPCB <> nil then
      try
        if not RPCB.CreateContext(Context) then FreeAndNil(RPCB);
      except
        on E: Exception do begin
          FreeAndNil(RPCB);
          Inc(AttemptCount);
          if AttemptCount > AttemptLimit then
              MessageDlg('You exceeded the limit of connection attempts.' + #13 + #13 + 'Try again later.', MtError,
              [Mbok], 0)
          else NewAttempt := MessageDlg('User Sign-on is not complete.' + #13 + #13 + 'Error Message: ' + #13 +
              E.Message + #13 + #13 + 'Do you want to repeat the attempt?', MtError, [Mbok, MbCancel], 0) = MrOK;
        end;
      end until (RPCB <> nil) or (AttemptCount > AttemptLimit) or not NewAttempt;
    Result := RPCB;
  end;
  begin
    CheckStartupParams := True;

end.
