unit fGMV_TimeOutManager;
{
  ================================================================================
  *
  *       Application:  Vitals
  *       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:44a $
  *       Developer:    doma.user@domain.ext
  *       Site:         Hines OIFO
  *
  *       Description:  Time Out functionality (Just like Roll'n'Scroll)
  *
  *       Notes:
  *
  ================================================================================
  *       $Archive: /Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/UTILS/fGMV_TimeOutManager.pas $
  *
  *
  *
  ================================================================================
}

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  StdCtrls;

type
  TfrmGMV_TimeOutManager = class(TForm)
    LastChanceTimer: TTimer;
    Label1: TLabel;
    LblSecondsLeft: TLabel;
    LblMessage: TLabel;
    BtnCancelTimeout: TButton;
    procedure LastChanceTimerTimer(Sender: TObject);
  private
    SecondsLeft: Integer;
  public
    { Public declarations }
  end;

type
  TMDShutDownProcedure = procedure of object;

procedure InitTimeOut(ShutDownProcedureName: TMDShutDownProcedure);
procedure UpdateTimeOutInterval(Seconds: Cardinal);
procedure ShutDownTimeOut;

implementation

type
  TMDTimeOut = class(TTimer)
  private
    FHooked: Boolean;
    TimeOutInterval: Cardinal;
    TimeOutKeyHandle: HHOOK;
    TimeOutMouseHandle: HHOOK;
    ShutDownProcedure: TMDShutDownProcedure;
  protected
    procedure ResetTimeOut;
    procedure TimeOutTimer(Sender: TObject);
  end;

var
  MDTimeOut: TMDTimeOut = nil;
function TimeoutKeyHook(Code: Integer; WParam: WPARAM; LParam: LPARAM): LRESULT; stdcall; forward;
function TimeoutMouseHook(Code: Integer; WParam: WPARAM; LParam: LPARAM): LRESULT; stdcall; forward;
{$R *.DFM}

function TimeoutKeyHook(Code: Integer; WParam: WPARAM; LParam: LPARAM): LRESULT;
{ this is called for every keyboard event that occurs }
begin
  Result := 0;
  if LParam shr 31 = 1 then MDTimeOut.ResetTimeout; // on KeyUp only
  try
    Result := CallNextHookEx(MDTimeOut.TimeoutKeyHandle, Code, WParam, LParam);
  except
  end;
end;

function TimeoutMouseHook(Code: Integer; WParam: WPARAM; LParam: LPARAM): LRESULT;
{ this is called for every mouse event that occurs }
begin
  Result := 0;
  if (Code >= 0) and (WParam > WM_MOUSEFIRST) and (WParam <= WM_MOUSELAST) then MDTimeOut.ResetTimeout;
  // all click events
  try
    Result := CallNextHookEx(MDTimeOut.TimeoutMouseHandle, Code, WParam, LParam);
  except
  end;
end;

procedure TMDTimeOut.ResetTimeout;
{ this restarts the timer whenever there is a keyboard or mouse event }
begin
  Enabled := False;
  Interval := TimeoutInterval;
  Enabled := True;
end;

procedure TMDTimeOut.TimeOutTimer(Sender: TObject);
{ when the timer expires, the application is closed after warning the user }
begin
  Enabled := False;
  { Check for minimized main form and then bring to front }
  with Application do begin
    if MainForm <> nil then
      if MainForm.WindowState = WsMinimized then MainForm.WindowState := WsNormal;
    BringToFront;
    ProcessMessages;
  end;
  with TfrmGMV_TimeOutManager.Create(Application) do
    try
      LblMessage.Caption := 'The application ' + Application.Title + ' is about to ' +
        'close due to inactivity.  Press the &Cancel button below to ' + 'continue working.';
      SecondsLeft := 15;
      LastChanceTimer.Interval := 1000;
      LastChanceTimer.Enabled := True;
      ShowModal;
      LastChanceTimer.Enabled := False;
      if ModalResult <> MrCancel then
        if Assigned(ShutDownProcedure) then ShutDownProcedure
        else Application.Terminate
      else Self.Enabled := True;
    finally
      Free;
    end;
end;

procedure InitTimeOut(ShutDownProcedureName: TMDShutDownProcedure);
begin
  if (not Assigned(MDTimeOut)) then begin
    MDTimeOut := TMDTimeOut.Create(Application);
    with MDTimeOut do begin
      ShutDownProcedure := ShutDownProcedureName;
      OnTimer := TimeOutTimer;
      TimeOutInterval := 10000;
      TimeOutKeyHandle := SetWindowsHookEx(WH_KEYBOARD, TimeOutKeyHook, 0, GetCurrentThreadID);
      TimeOutMouseHandle := SetWindowsHookEx(WH_MOUSE, TimeOutMouseHook, 0, GetCurrentThreadID);
      Interval := TimeOutInterval;
      Enabled := True;
    end;
  end;
end;

procedure UpdateTimeOutInterval(Seconds: Cardinal);
begin
  if (Assigned(MDTimeOut)) then
    with MDTimeOut do begin
      Interval := Seconds * 1000;
      TimeOutInterval := Seconds * 1000;
      Enabled := True;
    end;
end;

procedure ShutDownTimeOut;
begin
  if (Assigned(MDTimeOut)) then begin
    with MDTimeOut do begin
      Enabled := False;
      if FHooked then begin
        UnhookWindowsHookEx(TimeOutKeyHandle);
        UnhookWindowsHookEx(TimeOutMouseHandle);
        FHooked := False;
      end;
    end;
    FreeAndNil(MDTimeOut); // AAN 040406
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmGMV_TimeOutManager.LastChanceTimerTimer(Sender: TObject);
begin
  LastChanceTimer.Enabled := False;
  Dec(SecondsLeft);
  LblSecondsLeft.Caption := IntToStr(SecondsLeft);
  LastChanceTimer.Enabled := (SecondsLeft > 0);
  if not LastChanceTimer.Enabled then ModalResult := MrOk;
end;

end.
