unit fMDK_Wait;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:44a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     Wait form
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/UTILS/fMDK_Wait.pas $
  *
  *       $ History: fMDK_Wait.pas $
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls;

type
  TfrmWait = class(TForm)
    PnlAnimate: TPanel;
    Animate: TAnimate;
    Panel1: TPanel;
    Panel2: TPanel;
    Button1: TButton;
    Memo1: TRichEdit;
    Bevel1: TBevel;
    procedure Button1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    FParentControl: TWinControl;
    function CurrText: TTextAttributes;
    procedure PaintError(ILen: Integer; ATarget, ALine: string);
  public
    Stop: Boolean;
    procedure AddLogLine(ALine: string);
    procedure AppendLogLine(ALine: string);
    procedure WakeUp(AParentControl: TWinControl; ACaption: string; AnAVI: TCommonAVI = AviCopyFiles);
    procedure LayDown;
  end;

var
  FrmWait: TfrmWait;

const
  WstOK    = 'OK';
  WstAbort = 'ABORT';

implementation

uses
  FMDK_Main;
{$R *.dfm}

function TfrmWait.CurrText: TTextAttributes;
begin
  if Memo1.SelLength > 0 then Result := Memo1.SelAttributes
  else Result := Memo1.DefAttributes;
end;

procedure TfrmWait.PaintError(ILen: Integer; ATarget, ALine: string);
var
  IPos: Integer;
begin
  IPos := Pos(ATarget, ALine);
  if IPos > 0 then begin
    with Memo1 do begin
      SelStart := ILen + IPos - 1 - 2;
      SelLength := Length(ATarget);
      CurrText.Color := ClRed;
      SelStart := 0;
    end;
    Stop := True;
  end;
end;

procedure TfrmWait.AppendLogLine(ALine: string);
var
  ILen: Integer;
begin
  ILen := Length(Memo1.Lines.Text);
  Memo1.Lines[Memo1.Lines.Count - 1] := Memo1.Lines[Memo1.Lines.Count - 1] + ALine;
  PaintError(ILen, 'Failed', ALine);
  Application.ProcessMessages;
end;

procedure TfrmWait.AddLogLine(ALine: string);
var
  ILen: Integer;
begin
  ILen := Length(Memo1.Lines.Text);
  Memo1.Lines.Add(ALine);
  PaintError(ILen, 'Failed', ALine);
  Application.ProcessMessages;
end;

procedure TfrmWait.Button1Click(Sender: TObject);
begin
  Hide;
end;

procedure TfrmWait.FormActivate(Sender: TObject);
begin
  Stop := False;
end;

procedure TfrmWait.WakeUp(AParentControl: TWinControl; ACaption: string; AnAVI: TCommonAVI);
begin
  try
    FParentControl := AParentControl;
    Caption := ACaption;
    if Animate.CommonAVI <> AnAVI then Animate.CommonAVI := AnAVI;
    PnlAnimate.Visible := True;
    Show;
    Animate.Open := True;
    Animate.Active := True;
    Memo1.Clear;
    Stop := False;
    Panel2.Visible := False;
    if Assigned(FParentControl) then FParentControl.Enabled := False;
  except
  end;
end;

procedure TfrmWait.LayDown;
begin
  Animate.Active := False;
  PnlAnimate.Visible := False;
  Hide;
  if Assigned(FParentControl) then FParentControl.Enabled := True;
  if Stop then Panel2.Visible := True;
end;

procedure TfrmWait.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = Char(VK_ESCAPE)) and (Panel2.Visible) then Hide;
end;

end.
