unit uMDK_CDSCommon;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 7/29/08 10:21a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     Common CDS objects
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/UTILS/uMDK_CDSCommon.pas $
  *
  *       $ History: uMDK_CDSCommon.pas $
}
interface

uses
  DB, DBClient, DBCtrls, Classes, Variants, SysUtils, Dialogs, StdCtrls;

const
  FwDateTime           = 12;
  FwFloat              = 12;
  FwBoolean            = 5;
  CXMLPrefix           = '<?xml version="1.0" standalone="yes"?>';
  CXMLDataPacketPrefix = '<DATAPACKET Version="2.0">';
  CXMLPostfix          = '</ROWDATA></DATAPACKET>';
  CToBeContinued       = '...';
  GMDK_DateFMT         = 'mm/dd/yyyy';
  GMDK_TimeFMT         = 'hh:nn:ss';
  GMDK_DateTimeFMT     = 'mm/dd/yyyy hh:nn:ss';
  GMDK_DebugTimeFMT    = 'hh:MM:ss.zzz';

const
  LENGTH_ID          = 32;
  LENGTH_NAME        = 40;
  LENGTH_DESCRIPTION = 80;
  LENGTH_COMMENTS    = 255;

function NewIntegerField(ACDS: TClientDataSet; AFieldName, ALabel: string; AWidth: Integer): TIntegerField;
function NewFloatField(ACDS: TClientDataSet; AFieldName, ALabel: string; ADisplayFormat: string = '#.##';
  AWidth: Integer = FwFloat): TFloatField;
function NewBooleanField(ACDS: TClientDataSet; AFieldName, ALabel: string; AWidth: Integer = FwBoolean): TBooleanField;
function NewStringField(ACDS: TClientDataSet; AFieldName, ALabel: string; AWidth: Integer; ASize: Integer)
  : TStringField;
function NewTimeField(ACDS: TClientDataSet; AFieldName, ALabel: string; AWidth: Integer = FwDateTime;
  ADisplayFMT: string = GMDK_DateTimeFMT): TDateTimeField;
function NewDateTimeField(ACDS: TClientDataSet; AFieldName, ALabel: string; AWidth: Integer = FwDateTime;
  ADisplayFMT: string = GMDK_DateFMT): TDateTimeField;
function NewAggregateField(ACDS: TClientDataSet; AFieldName, AnExpression: string): TAggregateField;
function CDSGetData(ACDS: TClientDataSet; AValue: string): Boolean;
procedure CDSMerge(ASource, ATarget: TClientDataSet; AnID: string);
procedure CDSAppend(ASource, ATarget: TClientDataSet; FieldMap: string = ''; Defaults: string = '');
function CDSLoadData(ACDS: TClientDataSet; AValue: string; MergeID: string = ''): Boolean;
function CDSInitByString(ADS: TClientDataSet; AString: string; Clear: Boolean = True): string;
function DataTypeName(AType: TFieldType): string;
function DataTypeComment(AType: TFieldType): string;
procedure RestoreLink(AMaster: TDataSource; ADetail: TClientDataSet; AFieldName: string);
procedure BreakLink(ADetail: TClientDataSet);
procedure TestValue(Sender: TObject; Ignore: Boolean = False);
{$IFDEF XMLVIEW}
{$ELSE}
procedure UpdateMemo(Sender: TObject; Signature: string);
procedure UpdateEdit(Sender: TObject; Signature: string);
{$ENDIF}
procedure CopyIntoNullField(CdsFrom, CdsTo: TClientDataSet; AField: string);
procedure MergeCDS(ACDS: TClientDataSet);
procedure CleanUpCDS(ADS: TClientDataSet);
function GetFieldByName(ACDS: TClientDataSet; AField: string): TField;
function GetFieldDefByName(ACDS: TClientDataSet; AField: string): TFieldDef;
function GetActiveField(Sender: TObject): TField;
function CDSsetUpField(AF: TField; TheDisplayFormat, TheEditMask, TheWidth, TheUnit, TheDefault, ThePage, TheLabel,
  TheMax, TheMin: string): string;
function SetCDSFilter(ACDS: TClientDataSet; AFilter: string): string;

type
  TReportEvent = procedure(ATypeName, AComments: string) of object;

implementation

{$IFDEF XMLVIEW}

uses
  UMDK_Utils, System.UITypes;
{$ELSE}

uses
  UMDK_Utils, UMDK_Options, UMDK_Common, System.UITypes;
{$ENDIF}

const
  Al: TAlignment = TaLeftJustify;

function NewAggregateField(ACDS: TClientDataSet; AFieldName, AnExpression: string): TAggregateField;
var
  AF: TAggregateField;
begin
  Result := nil;
  try
    ACDS.Close;
    AF := TAggregateField.Create(ACDS);
    AF.DisplayName := AFieldName;
    AF.Name := ACDS.Name + AFieldName;
    AF.Expression := AnExpression;
    AF.Visible := True;
    AF.ReadOnly := True;
    AF.Active := True;
    AF.GroupingLevel := 1;
    Result := AF;
  except
  end;
end;

function NewIntegerField(ACDS: TClientDataSet; AFieldName, ALabel: string; AWidth: Integer): TIntegerField;
var
  FF: TIntegerField;
begin
  ACDS.Close;
  FF := TIntegerField.Create(ACDS);
  FF.FieldName := AFieldName;
  FF.Name := ACDS.Name + FF.FieldName;
  FF.Index := ACDS.FieldCount;
  FF.DataSet := ACDS;
  FF.DisplayLabel := ALabel;
  FF.DisplayWidth := AWidth;
  FF.Alignment := Al;
  Result := FF;
end;

function NewFloatField(ACDS: TClientDataSet; AFieldName, ALabel: string; ADisplayFormat: string = '#.##';
  AWidth: Integer = FwFloat): TFloatField;
var
  FF: TFloatField;
begin
  ACDS.Close;
  FF := TFloatField.Create(ACDS);
  FF.FieldName := AFieldName;
  FF.Name := ACDS.Name + FF.FieldName;
  FF.Index := ACDS.FieldCount;
  FF.DataSet := ACDS;
  FF.DisplayLabel := ALabel;
  FF.DisplayWidth := AWidth;
{$IFDEF CONTROL_FORMAT}
  FF.DisplayFormat := ADisplayFormat;
{$ENDIF}
  FF.Alignment := Al;
  Result := FF;
end;

function NewBooleanField(ACDS: TClientDataSet; AFieldName, ALabel: string; AWidth: Integer = FwBoolean): TBooleanField;
var
  FF: TBooleanField;
begin
  ACDS.Close;
  FF := TBooleanField.Create(ACDS);
  FF.FieldName := AFieldName;
  FF.Name := ACDS.Name + FF.FieldName;
  FF.Index := ACDS.FieldCount;
  FF.DataSet := ACDS;
  FF.DisplayLabel := ALabel;
  FF.DisplayWidth := AWidth;
  FF.Alignment := Al;
  Result := FF;
end;

function NewStringField(ACDS: TClientDataSet; AFieldName, ALabel: string; AWidth: Integer; ASize: Integer)
  : TStringField;
var
  FS: TStringField;
begin
  ACDS.Close;
  FS := TStringField.Create(ACDS);
  FS.FieldName := AFieldName;
  FS.Name := ACDS.Name + FS.FieldName;
  FS.Index := ACDS.FieldCount;
  FS.DataSet := ACDS;
  FS.Size := ASize;
  FS.DisplayLabel := ALabel;
  FS.DisplayWidth := AWidth;
  FS.Alignment := Al;
  Result := FS;
end;

function NewDateTimeField(ACDS: TClientDataSet; AFieldName, ALabel: string; AWidth: Integer = FwDateTime;
  ADisplayFMT: string = GMDK_DateFMT): TDateTimeField;
var
  FDT: TDateTimeField;
begin
  ACDS.Close;
  FDT := TDateTimeField.Create(ACDS);
  FDT.FieldName := AFieldName;
  FDT.Name := ACDS.Name + FDT.FieldName;
  FDT.Index := ACDS.FieldCount;
  FDT.DataSet := ACDS;
  FDT.DisplayLabel := ALabel;
  FDT.DisplayWidth := AWidth;
  FDT.DisplayFormat := ADisplayFMT;
  FDT.Alignment := Al;
  Result := FDT;
end;

function NewTimeField(ACDS: TClientDataSet; AFieldName, ALabel: string; AWidth: Integer = FwDateTime;
  ADisplayFMT: string = GMDK_DateTimeFMT): TDateTimeField;
var
  FDT: TTimeField;
begin
  ACDS.Close;
  FDT := TTimeField.Create(ACDS);
  FDT.FieldName := AFieldName;
  FDT.Name := ACDS.Name + FDT.FieldName;
  FDT.Index := ACDS.FieldCount;
  FDT.DataSet := ACDS;
  FDT.DisplayLabel := ALabel;
  FDT.DisplayWidth := AWidth;
  FDT.DisplayFormat := ADisplayFMT;
  FDT.Alignment := Al;
  Result := FDT;
end;

function CDSGetData(ACDS: TClientDataSet; AValue: string): Boolean;
var
  S: string;
  I: Integer;
begin
  ACDS.Close;
  try
    S := AValue;
    I := Pos(CXMLPrefix, AValue);
    if I > 0 then begin
      S := Copy(S, I, Length(S));
      I := Pos(CXMLPostfix, S);
      if I > 0 then S := Copy(S, 1, I + Length(CXMLPostfix) - 1);
      ACDS.XMLData := S;
    end
    else S := '';
  except
    S := '';
  end;
  ACDS.Open;
  Result := ACDS.Active;
end;

procedure CDSMerge(ASource, ATarget: TClientDataSet; AnID: string);
var
  IndexName: string;
  function FindRecord: Boolean;
  begin
    Result := False;
    if AnID = '' then Exit;
    ATarget.First;
    Result := ATarget.Locate(AnID, VarArrayOf([ASource.FieldByName(AnID).AsString]), [LoPartialKey]);
  end;
  procedure UpdateFields;
  var
    J: Integer;
    Fn: string;
  begin
    for J := 0 to ATarget.FieldDefs.Count - 1 do begin
      try
        Fn := ATarget.FieldDefs[J].Name;
        ATarget.FieldByName(Fn).Value := ASource.FieldByName(Fn).Value;
      except
      end;
    end;
    ATarget.Post;
  end;
  procedure UpdateRecord;
  begin
    if not FindRecord then ATarget.Insert
    else ATarget.Edit;
    UpdateFields;
  end;

begin
  try
    IndexName := ATarget.IndexFieldNames;
    ATarget.IndexFieldNames := AnID;
  except
  end;
  try
    if not ATarget.Active then ATarget.Open;
    ASource.First;
    while not ASource.EOF do begin
      UpdateRecord;
      ASource.Next;
    end;
  except
    on E: Exception do ShowMessage(E.Message);
  end;
  try
    ATarget.IndexFieldNames := IndexName;
  except
  end;
end;

procedure CDSAppend(ASource, ATarget: TClientDataSet; FieldMap: string = ''; Defaults: string = '');
var
  Sl: TStringList;
  Dl: TStringList;
  procedure UpdateRecord;
  var
    J: Integer;
    Nn, Fn: string;
  begin
    ATarget.Insert;
    for J := 0 to ATarget.FieldDefs.Count - 1 do begin
      try
        Fn := ATarget.FieldDefs[J].Name;
        Nn := Sl.Values[Fn];
        if Nn = '' then Nn := Fn;
        ATarget.FieldByName(Fn).Value := ASource.FieldByName(Nn).Value;
      except
        on E: Exception do begin
          MessageDLG(E.Message, MtWarning, [MbOK], 0);
          Nn := Dl.Values[Fn];
          if Nn <> '' then ATarget.FieldByName(Fn).AsString := Nn;
        end;
      end;
    end;
    ATarget.Post;
  end;

begin
  Sl := TStringList.Create;
  Sl.Text := FieldMap;
  Dl := TStringList.Create;
  Dl.Text := Defaults;
  try
    if not ATarget.Active then ATarget.Open;
    ASource.First;
    while not ASource.EOF do begin
      UpdateRecord;
      ASource.Next;
    end;
  except
    on E: Exception do ShowMessage(E.Message);
  end;
  Sl.Free;
  Dl.Free;
end;

function CDSLoadData(ACDS: TClientDataSet; AValue: string; MergeID: string = ''): Boolean;
var
  S: string;
  I: Integer;
  ACopyCDS: TClientDataSet;
begin
  try
    S := AValue;
    I := Pos(CXMLPrefix, AValue);
    if (I > 0) and (Pos(CXMLDataPacketPrefix, AValue) > 0) then begin
      S := Copy(S, I, Length(S));
      I := Pos(CXMLPostfix, S);
      if I > 0 then S := Copy(S, 1, I + Length(CXMLPostfix) - 1);
    end
    else S := '';
  except
    S := '';
  end;
  if Trim(S) <> '' then begin
    ACopyCDS := TClientDataSet.Create(nil);
    ACopyCDS.XMLData := S;
    try
      if not ACopyCDS.Active then ACopyCDS.Open;
    except
      on E: Exception do FreeAndNil(ACopyCDS);
    end;
    if Assigned(ACopyCDS) then begin
      if MergeID = '' then CDSAppend(ACopyCDS, ACDS)
      else CDSMerge(ACopyCDS, ACDS, MergeID);
      ACopyCDS.Close;
      ACopyCDS.Free;
    end;
  end;
  try
    ACDS.Open;
    Result := ACDS.Active;
  except
    Result := False;
  end;
end;

function CDSInitByString(ADS: TClientDataSet; AString: string; Clear: Boolean = True): string;
var
  SCopyValue, SValue: string;
  CdsCopy: TClientDataSet;
  Fld: TFieldDef;
  Fn: string;
  I: Integer;
begin
  Result := '';
  try
    if not ADS.Active then ADS.Open;
    if Clear then ADS.EmptyDataSet;
  except
    begin
      Result := 'Failed to Empty Data set <' + ADS.Name + '>';
      Exit;
    end;
  end;
  SValue := AString;
  while Pos(#13 + #10, SValue) = 1 do SValue := Copy(SValue, 3, Length(SValue));
  while Pos(#10 + #13, SValue) = 1 do SValue := Copy(SValue, 3, Length(SValue));
  if (AString = '') or (AString = #13 + #10) or (Pos('-1^', SValue) = 1) then begin
    Result := 'Assigning Blank String to the Dataset <' + ADS.Name + '>';
    Exit;
  end;
  CdsCopy := TClientDataSet.Create(nil);
  try
    CdsCopy.XMLData := SValue;
    CdsCopy.Open;
    while not CdsCopy.EOF do begin
      ADS.Insert;
      for I := 0 to ADS.FieldCount - 1 do begin
        if I >= ADS.FieldDefs.Count then Break; // zzzzzzandria 050407
        Fld := ADS.FieldDefs[I];
        Fn := Fld.Name;
        try
          try
            SCopyValue := CdsCopy.FieldByName(Fn).AsString;
          except
            on E: Exception do begin
              SCopyValue := '';
              // report(etError,'Assigning value '+cdsCopy.FieldByName(fn).AsString+' to <'+aDS.Name+'.'+fn+'>'+#13+#10+#13+#10+E.Message,
              // '',0,Classname,'Get Field Data');
            end;
          end;
          ADS.FieldByName(Fn).AsString := SCopyValue;
        except
          // on E: Exception do
          // report(etError,'Assigning value '+cdsCopy.FieldByName(fn).AsString+' to <'+aDS.Name+'.'+fn+'>'+#13+#10+#13+#10+E.Message,
          // '',0,Classname,'Assign Value');
        end;
        try
          if (Fld.DataType = FtBoolean) and (ADS.FieldByName(Fn).AsBoolean <> True) then
              ADS.FieldByName(Fn).AsBoolean := False;
        except
          ADS.FieldByName(Fn).AsBoolean := False;
        end;
      end;
      ADS.Post;
      CdsCopy.Next;
    end;
  except
  end;
  CdsCopy.Free;
end;

function DataTypeComment(AType: TFieldType): string;
begin
  case AType of
    FtUnknown: Result := 'Unknown or undetermined';
    FtString: Result := 'Character or string field';
    FtSmallint: Result := '16-bit integer field';
    FtInteger: Result := '32-bit integer field';
    FtWord: Result := '16-bit unsigned integer field';
    FtBoolean: Result := 'Boolean field';
    FtFloat: Result := 'Floating-point numeric field';
    FtCurrency: Result := 'Money field';
    FtBCD: Result := 'Binary-Coded Decimal field that can be converted to Currency type without a loss of precision.';
    FtDate: Result := 'Date field';
    FtTime: Result := 'Time field';
    FtDateTime: Result := 'Date and time field';
    FtBytes: Result := 'Fixed number of bytes (binary storage)';
    FtVarBytes: Result := 'Variable number of bytes (binary storage)';
    FtAutoInc: Result := 'Auto-incrementing 32-bit integer counter field';
    FtBlob: Result := 'Binary Large OBject field';
    FtMemo: Result := 'Text memo field';
    FtGraphic: Result := 'Bitmap field';
    FtFmtMemo: Result := 'Formatted text memo field';
    FtParadoxOle: Result := 'Paradox OLE field';
    FtDBaseOle: Result := 'dBASE OLE field';
    FtTypedBinary: Result := 'Typed binary field';
    FtCursor: Result := 'Output cursor from an Oracle stored procedure (TParam only)';
    FtFixedChar: Result := 'Fixed character field';
    FtWideString: Result := 'Wide string field';
    FtLargeint: Result := 'Large integer field';
    FtADT: Result := 'Abstract Data Type field';
    FtArray: Result := 'Array field';
    FtReference: Result := 'REF field';
    FtDataSet: Result := 'DataSet field';
    FtOraBlob: Result := 'BLOB fields in Oracle 8 tables';
    FtOraClob: Result := 'CLOB fields in Oracle 8 tables';
    FtVariant: Result := 'Data of unknown or undetermined type';
    FtInterface: Result := 'References to interfaces (IUnknown)';
    FtIDispatch: Result := 'References to IDispatch interfaces';
    FtGuid: Result := 'globally unique identifier (GUID) values';
    FtTimeStamp: Result := 'Date and time field accessed through dbExpress';
    FtFMTBcd: Result := 'Binary-Coded Decimal field that is too large for ftBCD.';
  end;
end;

function DataTypeName(AType: TFieldType): string;
begin
  Result := IntToStr(Integer(AType));
end;

procedure TestValue(Sender: TObject; Ignore: Boolean = False);
var
  AF: TField;
  ADBE: TDBEdit;
  AfVal: Real;
  AIVal: Integer;
  function GetFMax(AMask: string): Real;
  var
    S: string;
    I: Integer;
  begin
    S := Piece(AMask, '.', 1);
    Result := 0;
    if Length(S) < 1 then Exit;
    Result := 1;
    for I := 1 to Length(S) do Result := Result * 10.0;
  end;
  function GetIMax(AMask: string): Integer;
  var
    S: string;
    I: Integer;
  begin
    S := Piece(AMask, '.', 1);
    Result := 0;
    if Length(S) < 1 then Exit;
    Result := 1;
    for I := 1 to Length(S) do Result := Result * 10;
  end;
  procedure SetSelection(ADBEField: TDBEdit);
  begin
    ADBE.SelStart := Length(ADBE.Text) - 1;
    ADBE.SelLength := 1;
  end;

begin
  if Ignore then Exit;
  ADBE := TDBEdit(Sender);
  AF := ADBE.Field;
  if AF is TFloatField then begin
    try
      AfVal := StrToFloat(ADBE.Text);
      if AfVal > TFloatField(AF).MaxValue then begin
        AfVal := TFloatField(AF).MaxValue;
        ShowMessage(Format('The value <%s> exceeds the maximum input value <%f>', [ADBE.Text, AfVal]));
        SetSelection(ADBE);
      end;
    except
      if (Trim(ADBE.Text) <> '-') and (Trim(ADBE.Text) <> '') then begin
        ShowMessage('Invalid Value <' + ADBE.Text + '>');
        SetSelection(ADBE);
      end;
    end;
  end else if AF is TIntegerField then begin
    try
      AIVal := StrToInt(ADBE.Text);
      if AIVal > TIntegerField(AF).MaxValue then begin
        AIVal := TIntegerField(AF).MaxValue;
        ShowMessage(Format('The value <%s> exceeds the maximum input value <%d>', [ADBE.Text, AIVal]));
        SetSelection(ADBE);
      end;
    except
      if (Trim(ADBE.Text) <> '-') and (Trim(ADBE.Text) <> '') then begin
        ShowMessage('Invalid Value <' + ADBE.Text + '>');
        SetSelection(ADBE);
      end;
    end;
  end else if AF is TStringField then begin
  end else if AF is TDateTimeField then begin
  end else if AF is TDateField then begin
  end;
end;

procedure BreakLink(ADetail: TClientDataSet);
begin
  try
    ADetail.Active := False;
    ADetail.MasterFields := '';
    ADetail.MasterSource := nil;
    ADetail.Active := True;
  except
  end;
end;

procedure RestoreLink(AMaster: TDataSource; ADetail: TClientDataSet; AFieldName: string);
begin
  try
    ADetail.Active := False;
    ADetail.MasterFields := AFieldName;
    ADetail.MasterSource := AMaster;
    ADetail.Active := True;
  except
  end;
end;
{$IFDEF XMLVIEW}
{$ELSE}

procedure UpdateMemo(Sender: TObject; Signature: string);
var
  I: Integer;
  S: string;
  Dbe: TCustomMemo;
  // iSignatureLength,
  // iSelStart,
  // iSelLength,
  ILL: Integer;
  function SignatureLength: Integer;
  begin
    Result := 0;
    S := Preferences.SValue(OnSignatureKey);
    if (S = YoUndefined) or (S = '') then Exit;
    I := Pos(S, Dbe.Text);
    if I = 0 then Exit;
    Dbe.SelStart := I - 1;
    Dbe.SelLength := Length(S);
    Dbe.SelText := Signature;
    Result := Length(Signature);
    Dbe.SelLength := Result;
  end;
  procedure CheckLineLength;
  var
    SLine, SHead, STail: string;
    J, Jj: Integer;
  begin
    J := 0;
    while J < Dbe.Lines.Count do begin
      SLine := Dbe.Lines[J];
      if Length(SLine) > ILL then begin
        SHead := Copy(SLine, 1, ILL);
        STail := Copy(SLine, ILL + 1, Length(SLine));
        for Jj := ILL downto 1 do begin
          if Copy(SHead, Jj, 1) = ' ' then begin
            SHead := Copy(SHead, 1, Jj - 1);
            Break;
          end
          else STail := Copy(SHead, Jj, 1) + STail;
        end;
        if Jj = 0 then begin
          STail := Copy(SLine, ILL + 1, Length(SLine));
          SHead := Copy(SLine, 1, ILL);
        end;
        Dbe.Lines[J] := SHead;
        if STail <> '' then begin
          if J = Dbe.Lines.Count - 1 then Dbe.Lines[J] := Dbe.Lines[J] + #13#10 + STail
          else Dbe.Lines[J + 1] := STail + ' ' + Dbe.Lines[J + 1];
        end;
      end;
      Inc(J);
    end;
  end;

begin
  ILL := StrToIntDef(DevOptions.SValue(DonReportLineLength), 78);
  Dbe := TCustomMemo(Sender);
  // iSelStart := dbe.SelStart;
  // iSignatureLength :=
  SignatureLength;
  CheckLineLength;
  // uncomment if need to select the added signature
  // if iSignatureLength > 0 then
  // dbe.SelStart := iSelStart - 1; // set it equal 2 to include '['
  // dbe.SelLength := iSignatureLength;
end;

procedure UpdateEdit(Sender: TObject; Signature: string);
var
  I: Integer;
  S: string;
  Dbe: TDBEdit;
begin
  S := Preferences.SValue(OnSignatureKey);
  if (S = YoUndefined) or (S = '') then Exit;
  Dbe := TDBEdit(Sender);
  I := Pos(S, Dbe.Text);
  if I = 0 then Exit;
  Dbe.SelStart := I - 1;
  Dbe.SelLength := Length(S);
  Dbe.SelText := Signature;
  Dbe.SelStart := I + 3 + Length(Signature);
  Dbe.SelLength := 0;
end;
{$ENDIF}

procedure CopyIntoNullField(CdsFrom, CdsTo: TClientDataSet; AField: string);
begin
  // if cdsFrom.Eof then
  // Exit;
  try
    if CdsTo.FieldByName(AField).AsString = '' then
      if CdsFrom.FieldByName(AField).AsString <> '' then begin
        CdsTo.Edit;
        CdsTo.FieldByName(AField).AsString := CdsFrom.FieldByName(AField).AsString;
        CdsTo.Post;
      end;
  except
  end;
end;

procedure MergeCDS(ACDS: TClientDataSet);
begin
  try
    if ACDS.State = DsEdit then ACDS.Post;
    ACDS.MergeChangeLog;
  except
  end;
end;

procedure CleanUpCDS(ADS: TClientDataSet);
begin
  if not Assigned(ADS) then Exit;
  try
    if not ADS.Active then
      try
        ADS.Open;
      except
        Exit;
      end;
    if ADS.Active then ADS.EmptyDataSet;
    ADS.Close;
  except
  end
end;

function GetFieldByName(ACDS: TClientDataSet; AField: string): TField;
var
  I: Integer;
begin
  Result := nil;
  with ACDS do begin
    for I := 0 to Fields.Count - 1 do begin
      if Fields[I].Name <> AField then Continue;
      Result := Fields[I];
      Exit;
    end;
  end;
end;

function GetFieldDefByName(ACDS: TClientDataSet; AField: string): TFieldDef;
var
  I: Integer;
begin
  Result := nil;
  with ACDS do begin
    for I := 0 to FieldDefs.Count - 1 do begin
      if FieldDefs[I].Name <> AField then Continue;
      Result := FieldDefs[I];
      Exit;
    end;
  end;
end;

function GetActiveField(Sender: TObject): TField;
var
  FField: TField;
begin
  FField := nil;
  if Sender is TDBMemo then FField := TDBMemo(Sender).Field
  else if Sender is TDBComboBox then FField := TDBComboBox(Sender).Field
  else if Sender is TDBRadioGroup then FField := TDBRadioGroup(Sender).Field
  else if Sender is TDBEdit then FField := TDBEdit(Sender).Field
  else if Sender is TDBCheckBox then FField := TDBCheckBox(Sender).Field;
  Result := FField;
end;

function CDSsetUpField(AF: TField; TheDisplayFormat, TheEditMask, TheWidth, TheUnit, TheDefault, ThePage, TheLabel,
  TheMax, TheMin: string): string;
const
  FLOAT_IGNORE = -999.0;
  procedure ReportIssue(AMessage: string);
  begin
    Result := AMessage;
    // Report(rtError,'Update table Field <'+aF.Name+'> "'+#13+#10+'Format error '+#13+#10+aMessage);
  end;

begin
  Result := '';
  case AF.DataType of
    FtString: // ' 	Character or string field';
      try
        if TheLabel <> '' then TStringField(AF).DisplayLabel := TheLabel;
        if TheWidth <> '' then TStringField(AF).DisplayWidth := StrToIntDef(TheWidth, 12);
      except
        on E: Exception do ReportIssue(E.Message);
      end;
    (* *)
    FtInteger: // ' 	32-bit integer field';
      try
        if TheLabel <> '' then TIntegerField(AF).DisplayLabel := TheLabel;
        if TheDisplayFormat <> '' then TIntegerField(AF).DisplayFormat := TheDisplayFormat;
        if TheEditMask <> '' then TIntegerField(AF).EditMask := TheEditMask;
        if (TheMin <> TheMax) then begin
          if Trim(TheMin) <> '' then TIntegerField(AF).MinValue := Round(StrToFloat(TheMin));
          if Trim(TheMax) <> '' then TIntegerField(AF).MaxValue := Round(StrToFloat(TheMax));
        end;
      except
        on E: Exception do ReportIssue(E.Message);
      end;
    (* *)
    FtFloat: // ' 	Floating-point numeric field';
      try
        if TheLabel <> '' then TFloatField(AF).DisplayLabel := TheLabel;
{$IFDEF CONTROL_FORMAT}
        if TheDisplayFormat <> '' then TFloatField(AF).DisplayFormat := TheDisplayFormat;
        if TheEditMask <> '' then TFloatField(AF).EditMask := TheEditMask;
{$ENDIF}
        if (TheMin <> TheMax) and (StrToFloat(TheMin) <> FLOAT_IGNORE) then begin
          if Trim(TheMin) <> '' then TFloatField(AF).MinValue := StrToFloat(TheMin);
          if Trim(TheMax) <> '' then TFloatField(AF).MaxValue := StrToFloat(TheMax);
        end;
      except
        on E: Exception do ReportIssue(E.Message);
      end;
    FtDateTime: // ' 	Date and time field';
      try
        if TheLabel <> '' then TDateTimeField(AF).DisplayLabel := TheLabel;
        if TheDisplayFormat <> '' then TDateTimeField(AF).DisplayFormat := TheDisplayFormat;
        if TheEditMask <> '' then TDateTimeField(AF).EditMask := TheEditMask;
      except
        on E: Exception do ReportIssue(E.Message);
      end;
    FtUnknown:;  // ' 	Unknown or undetermined';
    FtSmallint:; // ' 	16-bit integer field';
    FtWord:;     // ' 	16-bit unsigned integer field';
    FtBoolean:;  // ' 	Boolean field';
    FtCurrency:; // ' 	Money field';
    FtBCD:;
    // ' 	Binary-Coded Decimal field that can be converted to Currency type without a loss of precision.';
    FtDate:;        // ' 	Date field';
    FtTime:;        // ' 	Time field';
    FtBytes:;       // ' 	Fixed number of bytes (binary storage)';
    FtVarBytes:;    // ' 	Variable number of bytes (binary storage)';
    FtAutoInc:;     // ' 	Auto-incrementing 32-bit integer counter field';
    FtBlob:;        // ' 	Binary Large OBject field';
    FtMemo:;        // ' 	Text memo field';
    FtGraphic:;     // ' 	Bitmap field';
    FtFmtMemo:;     // ' 	Formatted text memo field';
    FtParadoxOle:;  // ' 	Paradox OLE field';
    FtDBaseOle:;    // ' 	dBASE OLE field';
    FtTypedBinary:; // ' 	Typed binary field';
    FtCursor:;
    // ' 	Output cursor from an Oracle stored procedure (TParam only)';
    FtFixedChar:;  // ' 	Fixed character field';
    FtWideString:; // ' 	Wide string field';
    FtLargeint:;   // ' 	Large integer field';
    FtADT:;        // ' 	Abstract Data Type field';
    FtArray:;      // ' 	Array field';
    FtReference:;  // ' 	REF field';
    FtDataSet:;    // ' 	DataSet field';
    FtOraBlob:;    // ' 	BLOB fields in Oracle 8 tables';
    FtOraClob:;    // ' 	CLOB fields in Oracle 8 tables';
    FtVariant:;    // ' 	Data of unknown or undetermined type';
    FtInterface:;  // ' 	References to interfaces (IUnknown)';
    FtIDispatch:;  // ' 	References to IDispatch interfaces';
    FtGuid:;       // ' 	globally unique identifier (GUID) values';
    FtTimeStamp:;  // ' 	Date and time field accessed through dbExpress';
    FtFMTBcd:;     // ' 	Binary-Coded Decimal field that is too large for ftBCD.';
  end;
end;

function SetCDSFilter(ACDS: TClientDataSet; AFilter: string): string;
var
  BActive: Boolean;
begin
  BActive := ACDS.Active;
  try
    if Trim(AFilter) <> '' then ACDS.Filter := AFilter;
    Result := '';
  except
    on E: Exception do Result := 'setCDSFilter:' + #13#10 + E.Message;
  end;
  if ACDS.Active <> BActive then ACDS.Active := BActive;
end;

end.
