unit uMDK_Printer;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:44a $
*       Site Name:    Hines OIFO
*       Date Created: 11/18/06
*       Developers:   dddddddddomain.user@domain.ext
*       Description:  Contains simple printing routine
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/UTILS/uMDK_Printer.pas $
*
*       $ History: uMDK_Printer.pas $
*
}

interface

uses
  Printers
  , Graphics
  , Types
  ;

var
  LEFT_MARGIN: Integer = 100;
  RIGHT_MARGIN: Integer = 180;
  LEFT_TOP_X: Integer = 580;
  LEFT_TOP_Y: Integer = 180;

procedure MDK_SimplePrint(
  aPrinter: TPrinter;
  aString: string;
  aFont: TFont = nil;
  aBackgroundBitmap: TBitmap = nil;
  aTopheaderFormat: string = '';
  aBottomHeaderFormat: string = ''
  );

implementation

uses
  Classes
  , SysUtils
  ;

procedure MDK_SimplePrint(
  aPrinter: TPrinter;
  aString: string;
  aFont: TFont = nil;
  aBackgroundBitmap: TBitmap = nil;
  aTopheaderFormat: string = '';
  aBottomHeaderFormat: string = ''
  );
var
  rr: TRect;
  iPage, i, j, iTotal: Integer;
  SL: TStringList;

  procedure DrawFrame(aR: TRect);
  begin
    aPrinter.Canvas.Brush.Color := clBlack;
    aPrinter.Canvas.FrameRect(aR);
    aPrinter.Canvas.Brush.Style := bsClear;
  end;

  procedure DrawBackground(aR: TRect);
  begin
    if aBackgroundBitmap <> nil then
      aPrinter.Canvas.StretchDraw(aR, aBackgroundBitmap);
  end;

  function SimpleBottomLine: string;
  var
    sFormat: string;
  begin
    if aTopHeaderFormat = '' then
      sFormat := '*** Page %d of %d ***'
    else
      sFormat := aTopHeaderFormat;
    Result := Format(sFormat, [iPage, iTotal]);
  end;

  function SimpleHeaderLine: string;
  var
    sFormat: string;
  begin
    if aBottomHeaderFormat = '' then
      sFormat := '*** WORK COPY ONLY *** %s ***'
    else
      sFormat := aBottomHeaderFormat;
    Result := Format(sFormat, [FormatDateTime('dd/mm/yyyy hh:mm:ss', Now)]);
  end;

  procedure ClosePage;
  var
    s: string;
    iPos: Integer;
  begin
    s := SimpleBottomLine;
    iPos := aPrinter.PageWidth - aPrinter.Canvas.TextWidth(s) - RIGHT_MARGIN;
    aPrinter.Canvas.TextOut(iPos, aPrinter.PageHeight - 100, s);

    s := SimpleHeaderLine;
    iPos := aPrinter.PageWidth - aPrinter.Canvas.TextWidth(s) - RIGHT_MARGIN;
    aPrinter.Canvas.TextOut(iPos, 60, SimpleHeaderLine);
    inc(iPage);
  end;

  function TotalPages: Integer;
  var
    jCount, jHeight, jPage, i: integer;
  begin
    Result := 1;
    if SL.Count <= 1 then
      Exit;
    jCount := 0;
    jPage := 1;
    for i := 0 to SL.Count - 1 do
      begin
        jHeight := aPrinter.Canvas.TextHeight(SL[i]);
        if jCount + jHeight > Printer.PageHeight - LEFT_TOP_Y * 2 then
          begin
            inc(jPage);
            jCount := jHeight
          end
        else
          jCount := jCount + jHeight;
      end;
    Result := jPage;
  end;

begin
  with aPrinter do
    begin
      SL := TStringList.Create;
      SL.Text := aString;

      if aFont <> nil then
        Canvas.Font.Assign(aFont);

      iTotal := TotalPages;
      rr := Rect(LEFT_TOP_X, LEFT_TOP_Y,
        (Pagewidth - LEFT_TOP_Y), (PageHeight - LEFT_TOP_Y));

      BeginDoc;

      DrawBackground(rr);
      Canvas.Brush.Style := bsClear;

      j := 0;
      iPage := 1;
      for i := 0 to SL.Count - 1 do
        begin
          if ((j + 1) * Canvas.TextHeight(SL[i])) >
            (Pageheight - LEFT_TOP_Y * 2) then
            begin
              DrawFrame(rr);
              ClosePage;
              Newpage;

              DrawBackground(rr);

              j := 0;
            end;
          Canvas.TextOut(LEFT_TOP_X + 40, LEFT_TOP_Y + 40 + (j *
            Canvas.TextHeight(SL[i])), SL[i]);
          inc(j);
        end;

      SL.Free;

      DrawFrame(rr);
      ClosePage;
      EndDoc;
    end;
end;

(*
The following code loads a bitmap from a file and assigns it to the Brush of the Canvas of Form1:

var
  Bitmap: TBitmap;
begin
  Bitmap := TBitmap.Create;
  try
    Bitmap.LoadFromFile('MyBitmap.bmppng');
    Form1.Canvas.Brush.Bitmap := Bitmap;
    Form1.Canvas.FillRect(Rect(0,0,100,100));
  finally
    Form1.Canvas.Brush.Bitmap := nil;
    Bitmap.Free;
  end;

end;
*)

end.
