unit uMDK_Utils;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 4/02/08 7:08a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     Common routines
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/UTILS/uMDK_Utils.pas $
  *
  *       $ History: uMDK_Utils.pas $
}
interface

uses
  Forms, Classes, Controls, SysUtils, Windows;

type
  TFMDateTimeMode = set of (FmdtDateOnly, // Convert date only
    FmdtShortDate,                        // Short date (digits only)
    FmdtYear2,                            // 2 digit year
    FmdtTimeOnly,                         // Convert time only
    FmdtShortTime,                        // Do not convert seconds
    FmdtTime24                            // Military time format
    );
procedure SetFormParent(AForm: TForm; ATab: TWinControl);
procedure SetUpForm(AForm: TForm; ATab: TWinControl);
function GetWorkstationName: string;
function Piece(Value, Delimiter: string; StartPiece: Integer = 1): string; overload;
function Piece(Value, Delimiter: string; StartPiece: Integer; EndPiece: Integer): string; overload;
function CmdLineSwitch(Swlst: array of string): Boolean; overload;
function CmdLineSwitch(Swlst: array of string; var Swval: string): Boolean; overload;
function CheckOption(AName, AValue, AMin, AMax: string; var Reason: string): Boolean;
function InString(S: string; SubStrs: array of string; CaseSensitive: Boolean = True): Boolean;
function IsValidDateString(AString: string): Boolean;
function FMDateTimeStr(DateTime: string; Mode: TFMDateTimeMode = []; DfltValue: string = ''): string;
function WindowsDateTimeToFMDateTime(WinDate: TDateTime): Double;

var
  FMDateFormat: array [1 .. 2, 1 .. 3] of string;

implementation

function IntVal(Str: string): Integer;
begin
  if Str <> '' then
    try
      Result := StrToInt(Str);
    except
      on EConvertError do Result := 0;
      else
        raise;
    end
  else Result := 0;
end;

function IsValidDateString(AString: string): Boolean;
begin
  Result := False;
  try
    StrToDateTime(AString);
    Result := True;
  except
  end;
end;

procedure SetFormParent(AForm: TForm; ATab: TWinControl);
begin
  if AForm.Parent <> ATab then begin
    AForm.Parent := ATab;
    AForm.Align := AlClient;
  end;
end;

procedure SetUpForm(AForm: TForm; ATab: TWinControl);
begin
  if AForm.Parent <> ATab then begin
    AForm.BorderStyle := BsNone;
    AForm.Parent := ATab;
    AForm.Align := AlClient;
    // aForm.Show;
  end;
end;

function GetWorkstationName: string;
var
  Buffer: array [0 .. 200] of Char;
  IBuffer: DWord;
begin
  IBuffer := 200;
  Result := 'unknown';
  try
    GetComputerName(Buffer, IBuffer);
    Result := Copy(Buffer, 1, IBuffer);
  except
  end;
end;

function Piece(Value, Delimiter: string; StartPiece: Integer = 1): string;
begin
  Result := Piece(Value, Delimiter, StartPiece, StartPiece);
end;

function Piece(Value, Delimiter: string; StartPiece: Integer; EndPiece: Integer): string;
var
  Dlen, I, Pnum: Integer;
  Buf: string;
begin
  Result := '';
  if (Value <> '') and (StartPiece > 0) and (EndPiece >= StartPiece) then begin
    Dlen := Length(Delimiter);
    I := Pos(Delimiter, Value) - 1;
    if I >= 0 then begin
      Buf := Value;
      Pnum := 1;
      repeat
        if Pnum > EndPiece then Break;
        if I < 0 then I := Length(Buf);
        if Pnum = StartPiece then Result := Copy(Buf, 1, I)
        else if Pnum > StartPiece then Result := Result + Delimiter + Copy(Buf, 1, I);
        Delete(Buf, 1, I + Dlen);
        I := Pos(Delimiter, Buf) - 1;
        Inc(Pnum);
      until (I < 0) and (Buf = '');
    end else if StartPiece = 1 then Result := Value;
  end;
end;

function InString(S: string; SubStrs: array of string; CaseSensitive: Boolean = True): Boolean;
var
  I: Integer;
begin
  I := 0;
  Result := False;
  while (I <= high(SubStrs)) and (Result = False) do begin
    if CaseSensitive then begin
      if Pos(SubStrs[I], S) = 1 then Result := True
      else Inc(I)
    end else begin
      if Pos(LowerCase(SubStrs[I]), LowerCase(S)) = 1 then Result := True
      else Inc(I)
    end;
  end
end;

function CmdLineSwitch(Swlst: array of string): Boolean;
var
  Swval: string;
begin
  Result := CmdLineSwitch(Swlst, Swval);
end;

function CmdLineSwitch(Swlst: array of string; var Swval: string): Boolean;
var
  I: Integer;
begin
  Result := False;
  Swval := '';
  for I := 1 to ParamCount do begin
    if InString(ParamStr(I), Swlst, False) then begin
      Swval := Piece(ParamStr(I), '=', 2);
      Result := True;
      Break;
    end;
  end;
end;

function CheckOption(AName, AValue, AMin, AMax: string; var Reason: string): Boolean;
var
  ICode, IMin, IMax, IVal: Integer;
  SV, SMin, SMax, UpValue, UpName: string;
const
  ValidPainLevel = '0,1,2,3,4,5,6,7,8,9,99,';
begin
  Result := True;
  SV := AValue;
  UpValue := Uppercase(SV);
  UpName := Uppercase(AName);
  if (UpName = 'PAIN LEVEL') and (Pos(UpValue + ',', ValidPainLevel) = 0) then begin
    Reason := 'Invalid Pain Level value of "' + AValue + '"' + #13#10 + 'Select one of: 0,1,2,3,4,5,6,7,8,9,99';
    Result := False;
  end else if ((Pos('ALLOW', UpName) = 1) or (Pos('ALWAYS', UpName) = 1) or (Pos('AUTOSAVE', UpName) = 1) or
    (Pos('DISABLE', UpName) = 1) or (Pos('FALLS ASSESSMENT', UpName) = 1) or (Pos('IGNORE', UpName) = 1) or
    (Pos('KEEP', UpName) = 1) or (Pos('NOTIFY', UpName) = 1) or (Pos('REGISTER', UpName) = 1) or
    (Pos('REVERSE', UpName) = 1) or (Pos('SAVE', UpName) = 1) or (Pos('SHOW', UpName) = 1) or (Pos('SMART', UpName) = 1)
    or (Pos('STUDY LIST AUTOREFRESH', UpName) = 1)) and (('TRUE' <> UpValue) and ('FALSE' <> UpValue)) then begin
    Result := False;
    Reason := 'Option ' + AName + ':' + #13#10 + 'Value "' + SV + '" is not valid Boolean' + #13#10 +
      'Select TRUE or FALSE';
  end else if ((Pos('(SEC)', UpName) <> 0) or (Pos('(MIN)', UpName) <> 0) or (Pos('QUANTITY', UpName) <> 0) or
    (Pos('LENGTH', UpName) <> 0) or (Pos('LIMIT', UpName) <> 0)
    // or (pos('IEN', upName) <> 0)
    ) then begin
    Val(SV, IVal, ICode);
    if ICode <> 0 then begin
      Result := False;
      Reason := 'Option ' + AName + ':' + #13#10 + 'Value "' + SV + '" is not valid integer';
    end else begin
      SMin := AMin; // PreferenceComments.sMinValue(aName);
      SMax := AMax; // PreferenceComments.sMaxValue(aName);
      if (SMax <> '') and (SMin <> '') then
        try
          IMin := StrToInt(SMin);
          IMax := StrToInt(SMax);
          if (IVal < IMin) or (IMax < IVal) then begin
            Result := False;
            Reason := 'Option ' + AName + ':' + #13#10 + 'Value "' + SV + '" is outside of the range ' + SMin +
              '..' + SMax;
          end
          else Result := True;
        except
        end;
    end;
  end else if Pos('COLOR', UpName) <> 0 then begin
    Val(SV, IVal, ICode);
    if ICode <> 0 then begin
      Result := False;
      Reason := 'Option ' + AName + ':' + #13#10 + 'Value "' + SV + '" is not valid integer';
    end else begin
      Result := True;
      ICode := IVal;
    end;
  end
  else Result := True;
end;

// copy from uROR_CommonUtils
function FMDateTimeStr(DateTime: string; Mode: TFMDateTimeMode; DfltValue: string): string;
var
  Buf, Format: string;
  Day, Month, Year: Word;
  Hour, Min, Sec: Word;
  Date, Time: TDate;
  DateType, DatePart: Integer;
begin
  if DateTime = '' then begin
    Result := DfltValue;
    Exit;
  end;
  Result := '';
  Buf := Piece(DateTime, '.', 1);
  if (not(FmdtTimeOnly in Mode)) and (Buf <> '') then begin
    Year := IntVal(Copy(Buf, 1, 3)) + 1700;
    Month := IntVal(Copy(Buf, 4, 2));
    Day := IntVal(Copy(Buf, 6, 2));
    if FmdtShortDate in Mode then DateType := 2
    else DateType := 1;
    DatePart := 1;
    if Day = 0 then begin
      Day := 1;
      DatePart := 2;
      if Month = 0 then begin
        Month := 1;
        DatePart := 3;
      end;
    end;
    Format := FMDateFormat[DateType][DatePart];
    if FmdtYear2 in Mode then Format := StringReplace(Format, 'YYYY', 'YY', []);
    Date := EncodeDate(Year, Month, Day);
    Result := FormatDateTime(Format, Date);
  end;
  Buf := Piece(DateTime, '.', 2);
  if (not(FmdtDateOnly in Mode)) and (Buf <> '') then begin
    Buf := Copy(Buf + '000000', 1, 6);
    Hour := IntVal(Copy(Buf, 1, 2));
    Min := IntVal(Copy(Buf, 3, 2));
    Sec := IntVal(Copy(Buf, 5, 2));
    if Hour >= 24 then begin
      Hour := 23;
      Min := 59;
      Sec := 59;
    end else if Min >= 60 then begin
      Min := 59;
      Sec := 59;
    end else if Sec >= 60 then Sec := 59;
    Time := EncodeTime(Hour, Min, Sec, 0);
    if FmdtTime24 in Mode then
      if FmdtShortTime in Mode then Format := 'HH:NN'
      else Format := 'HH:NN:SS'
    else if FmdtShortTime in Mode then Format := 'T'
    else Format := 'TT';
    if Result <> '' then Result := Result + '  ' + FormatDateTime(Format, Time)
    else Result := FormatDateTime(Format, Time);
  end;
end;

procedure FMInitFormatArray;
var
  Format: string;
begin
  Format := Uppercase(FormatSettings.ShortDateFormat);
  if Pos('M', Format) > Pos('D', Format) then begin
    FMDateFormat[1][1] := 'DD MMM YYYY';
    FMDateFormat[1][2] := 'MMM YYYY';
    FMDateFormat[1][3] := 'YYYY';
    FMDateFormat[2][1] := 'DD/MM/YYYY';
    FMDateFormat[2][2] := 'MM/YYYY';
    FMDateFormat[2][3] := 'YYYY';
  end else begin
    FMDateFormat[1][1] := 'MMM DD, YYYY';
    FMDateFormat[1][2] := 'MMM YYYY';
    FMDateFormat[1][3] := 'YYYY';
    FMDateFormat[2][1] := 'MM/DD/YYYY';
    FMDateFormat[2][2] := 'MM/YYYY';
    FMDateFormat[2][3] := 'YYYY';
  end;
end;

function WindowsDateToFMDate(WinDate: TDate): Double;
var
  Year, Month, Day: Word;
begin
  DecodeDate(WinDate, Year, Month, Day);
  Result := ((Year - 1700) * 10000) + (Month * 100) + Day;
end;

function WindowsDateTimeToFMDateTime(WinDate: TDateTime): Double;
var
  Hour, Min, Sec, Msec: Word;
begin
  Result := WindowsDateToFMDate(WinDate);
  DecodeTime(WinDate, Hour, Min, Sec, Msec);
  Result := Result + (Hour * 0.01) + (Min * 0.0001) + (Sec * 0.000001);
end;

initialization

FMInitFormatArray;

end.
