unit uZZ_RPCEvent;
{
  ================================================================================
  *
  *       Package:        development tool
  *       Date Created:   5/08/12
  *       Site:           Hines OIFO
  *       Developers:
  *                       dddddomain.user@domain.ext
  *
  *       Description:    Implementation of RPCEventItem of the Event log
  *
  ================================================================================
}

interface

uses
  Classes, UZZ_DescribedItem, TRPCB, Dialogs;

type
  TRPCEventItem = class(TmDescribedItem)
  public
    Start: TDateTime;
    Stop: TDateTime;
    RPCContext: string;
    BrokerInfo: string;
    RPC: string;
    Params: TStrings;
    Results: TStrings;
    Errors: TStrings;
    constructor Create;
    destructor Destroy; override;
    procedure AppendResults(AResults: TStrings; AStart, AStop: TDateTime);
    procedure AppendError(AnError: string);
  end;

function GetRPCEventItem(AStart, AStop: TDateTime; AConnection: string; ABrokerInfo: string; RemoteProcedure: string;
  Parameters: TStrings; // array of string;
  MultList: TStringList = nil; RetList: TStrings = nil): TRPCEventItem;
function GetTRPCBEventItem(ARPCB: TRPCBroker): TRPCEventItem;

var
  ServerDelay: TDateTime;
  CheckBrokerFlag: Boolean;
  RPCCount: Integer;

implementation

uses
  SysUtils;

/// /////////////////////////////////////////////////////////////////////////////
const
  FmtParams   = 'Params:   --------------------------------------------------';
  FmtResults  = 'Results:  --------------------------------------------------';
  FmtErrors   = 'Errors:   --------------------------------------------------';
  FmtRef      = 'Reference ''%s''';
  FmtLit      = 'Literal   ''%s''';
  FmtOutput   = '%-7.7d   %s';
  FmtContext  = 'Context:  ''%s''';
  FmtRPC      = 'RPC:      ''%s''';
  FmtRPCBInfo = 'Broker:   ''%s''';
  FmtConnect  = 'Account:  %s:%d %s (%s)';

function GetRPCEventItem(AStart, AStop: TDateTime; AConnection: string; ABrokerInfo: string; RemoteProcedure: string;
  Parameters: TStrings; // array of string;
  MultList: TStringList = nil; RetList: TStrings = nil): TRPCEventItem;
var
  Item: TRPCEventItem;
  I, J: Integer;
begin
  Item := TRPCEventItem.Create;
  Item.Start := AStart;
  Item.Stop := AStop;
  Item.RPC := RemoteProcedure;
  Item.RPCContext := AConnection;
  Item.BrokerInfo := ABrokerInfo;
  Item.Params.Add(FmtParams);
  for I := 0 to Parameters.Count - 1 do begin
    if (Copy(Parameters[I], 1, 1) = '@') and (Parameters[I] <> '@') then
        Item.Params.Add(Format(FmtRef, [Copy(Parameters[I], 2, Length(Parameters[I]))]))
    else Item.Params.Add(Format(FmtLit, [Parameters[I]]));
  end;
  if MultList <> nil then
    if MultList.Count > 0 then begin
      Item.Params.Add('List:');
      for J := 1 to MultList.Count do Item.Params.Add(MultList[J - 1]);
    end;
  if RetList <> nil then
    if RetList.Count > 0 then begin
      for J := 0 to RetList.Count - 1 do Item.Results.Add(RetList[J]);
    end;
  Result := Item;
  Inc(RPCCount);
  Item.Description.Text := '          ' + FormatDateTime('hh:mm:ss.zzz -- ', AStart) +
    FormatDateTime('hh:mm:ss.zzz    ', AStop) + '  ' + FormatDateTime('(hh:mm:ss.zzz)', AStop - AStart) + #13#10 +
    Format(FmtContext, [Item.RPCContext]) + #13#10 + Format(FmtRPC, [Item.RPC]) + #13#10 +
    Format(FmtRPCBinfo, [Item.BrokerInfo]) + #13#10 + #13#10 + Item.Params.Text + #13#10;
  Item.Description.Text := Item.Description.Text + FmtResults + #13#10;
  for I := 0 to Item.Results.Count - 1 do
      Item.Description.Text := Item.Description.Text + Format(FmtOutput, [I, Item.Results[I]]) + #13#10;
  Item.Description.Text := Item.Description.Text + FmtResults;
end;

function GetTRPCBEventItem(ARPCB: TRPCBroker): TRPCEventItem;
var
  I, J: Integer;
  Item: TRPCEventItem;
  Ss, Sss: string;
begin
  Item := TRPCEventItem.Create;
  Item.RPC := ARPCB.RemoteProcedure;
  Item.RPCContext := ARPCB.CurrentContext;
  for I := 0 to ARPCB.Param.Count - 1 do begin
    if ARPCB.Param[I].PType = Literal then Item.Params.Add(Format(FmtLit, [ARPCB.Param[I].Value]))
    else if ARPCB.Param[I].PType = Reference then Item.Params.Add(Format(FmtRef, ['@' + ARPCB.Param[I].Value]))
    else if ARPCB.Param[I].PType = List then begin
      for J := 0 to ARPCB.Param[I].Mult.Count - 1 do begin
        Ss := ARPCB.Param[I].Mult.Subscript(J);
        Sss := Format('List     [%-20.20s] = ''%s''', [Ss, ARPCB.Param[I].Mult[Ss]]);
        Item.Params.Add(Sss);
      end;
    end
  end;
  Item.Description.Text := Format(FmtContext, [Item.RPCContext]) + #13#10 + Format(FmtRPC, [Item.RPC]) + #13#10 +
    Format(FmtConnect, [ARPCB.Server, ARPCB.ListenerPort, ARPCB.User.Name, ARPCB.User.DUZ]) + #13#10#13#10 + FmtParams +
    #13#10 + Item.Params.Text + #13#10 + FmtResults + #13#10;
  Result := Item;
end;
/// /////////////////////////////////////////////////////////////////////////////

constructor TRPCEventItem.Create;
begin
  inherited Create(nil);
  Params := TStringList.Create;
  Results := TStringList.Create;
  Errors := TStringList.Create;
end;

destructor TRPCEventItem.Destroy;
begin
  Params.Free;
  Results.Free;
  Errors.Free;
  inherited;
end;

procedure TRPCEventItem.AppendResults(AResults: TStrings; AStart, AStop: TDateTime);
var
  I: Integer;
begin
  Start := AStart;
  Stop := AStop;
  Description.Text := '          ' + FormatDateTime('hh:mm:ss.zzz -- ', AStart) + FormatDateTime('hh:mm:ss.zzz    ',
    AStop) + '  ' + FormatDateTime('(hh:mm:ss.zzz)', AStop - AStart) + #13#10 + Description.Text;
  if Assigned(AResults) then begin
    Results.Text := AResults.Text;
    for I := 0 to Results.Count - 1 do
        Description.Text := Description.Text + Format(FmtOutput, [I, Results[I]]) + #13#10;
    Description.Text := Description.Text + FmtResults;
  end;
end;

procedure TRPCEventItem.AppendError(AnError: string);
var
  I: Integer;
begin
  Errors.Text := Errors.Text + AnError;
  Description.Text := Description.Text + #13#10 + FmtErrors + #13#10;
  for I := 0 to Errors.Count - 1 do Description.Text := Description.Text + Errors[I];
  Description.Text := Description.Text + FmtErrors + #13#10;
end;
/// /////////////////////////////////////////////////////////////////////////////

initialization

CheckBrokerFlag := False;
RPCCount := 0;

finalization

end.
